@@ -34,7 +34,6 @@
 #include <qfile.h>
 #include <qfiledialog.h>
 #include <qsplitter.h>
-#include <qtextcodec.h>
 #include <qinputdialog.h>
 #include <qtabwidget.h>
 
@@ -53,6 +52,7 @@
 #include <dciconloader.h>
 
 #include "dcchat.h"
+#include "dchubsearch.h"
 
 #include <dclib/cfilemanager.h>
 #include <dclib/dcos.h>
@@ -78,6 +78,7 @@ const char * p_LinkList[] = {
 	"DCHUB://",
 	"DCHUBS://",	
 	"DCCMD://",
+	"MAGNET:",
 	NULL
 };
 
@@ -194,22 +195,22 @@ bool DCChat::eventFilter( QObject * object, QEvent * event )
 		bool send = FALSE;
 		if ( e->state() == AltButton )
 		{
-			if ( (e->key() == Key_S) && (g_pConfig->GetSendChat() == "Alt + S") )
+			if ( (e->key() == Key_S) && (g_pConfig->GetSendChat() == tr("Alt + S").ascii()) )
 			{
 				send = TRUE;
 			}
 			else if( ((e->key() == Key_Enter) || (e->key() == Key_Return )) &&
-				   (g_pConfig->GetSendChat() == "Alt + Enter"))
+				   (g_pConfig->GetSendChat() == tr("Alt + Enter").ascii()))
 			{
 				send = TRUE;
 			}
 		}
-		else if( (g_pConfig->GetSendChat() == "Ctrl + Enter") &&
+		else if( (g_pConfig->GetSendChat() == tr("Ctrl + Enter").ascii()) &&
 			 (e->state() == ControlButton) && ( e->key() == Key_Enter || e->key() == Key_Return) )
 		{
 			send = TRUE;
 		}
-		else if( (g_pConfig->GetSendChat() == "Enter" && (e->state() != ControlButton) &&
+		else if( (g_pConfig->GetSendChat() == tr("Enter").ascii() && (e->state() != ControlButton) &&
 			 ( e->key() == Key_Enter || e->key() == Key_Return)) )
 		{
 			send = TRUE;
@@ -484,6 +485,82 @@ void DCChat::slotClickedChatOutput( int, int )
 			{
 				pressedLink = pressedLink.Right( pressedLink.Length()-8 );
 			}
+			else if ( p_LinkList[i] == CString("MAGNET:") )
+			{
+				//magnet:?xt=urn:tree:tiger:EOSA5AGTL5SD3VWCF3R2OH2WMGXV3S3R7SYN4YA&xl=708780032&dn=FC-6-i386-disc1.iso
+				// TTHs are always 39 characters long
+				QPopupMenu *m = new QPopupMenu(this);
+				int id, searchID, copyID, lookupID, i;
+				QString tth;
+				
+				i = pressedLink.Find("xt=urn:tree:tiger:");
+				if (i == -1)
+				{
+					i = pressedLink.Find("xt.1=urn:tree:tiger:");
+					tth = pressedLink.Mid(i + 20, 39).Data();
+				}
+				else
+				{
+					tth = pressedLink.Mid(i + 18, 39).Data();
+				}
+				
+				searchID = m->insertItem( QIconSet(g_pIconLoader->GetPixmap(eiFILEFIND)), QObject::tr("Search by TTH") );
+				m->setItemEnabled(searchID, TRUE);
+				copyID = m->insertItem( QIconSet(g_pIconLoader->GetPixmap(eiEDITCOPY)), QObject::tr("Copy TTH to clipboard") );
+				m->setItemEnabled(copyID, TRUE);
+				lookupID = m->insertItem( QIconSet(g_pIconLoader->GetPixmap(eiFILEFIND)), QObject::tr("Look up TTH at Bitzi.com") );
+				m->setItemEnabled(lookupID, TRUE);
+				
+				id = m->exec(QCursor::pos());
+				
+				delete m;
+				
+				if (id == searchID)
+				{
+					g_pHubSearch->Combobox_SEARCH->setEditText(tth);
+					g_pHubSearch->LineEdit_SEARCHSIZE->setText("0");
+					g_pHubSearch->ComboBox_SEARCHTYPE->setCurrentItem(9);
+					g_pHubSearch->ComboBox_SEARCHUNIT->setCurrentItem(0);
+					g_pHubSearch->ComboBox_SEARCHLIMIT->setCurrentItem(0);
+		
+					if ( g_pHubSearch->isMinimized() )
+					{
+						g_pHubSearch->showNormal();
+					}
+					else if ( g_pHubSearch->isVisible()==FALSE )
+					{
+						g_pHubSearch->setEnabled(TRUE);
+						g_pHubSearch->show();
+					}
+
+					if ( g_pHubSearch->TabWidget_HUBSEARCH->isVisible()==FALSE )
+					{
+						g_pHubSearch->TabWidget_HUBSEARCH->show();
+					}
+					
+					// put search dialog above chat
+					g_pHubSearch->raise();
+					g_pHubSearch->setActiveWindow();
+				}
+				else if (id == copyID)
+				{
+					QApplication::clipboard()->setText(tth);
+				}
+				else if (id == lookupID)
+				{
+					// http://bitzi.com/lookup/tree:tiger:PKQRUF7BKROIPCX2XWTJ7XUKRXRPUMW5V6REAMY
+					if ( g_pConfig->GetBrowser() != "" )
+					{
+						QProcess * pr = new QProcess();
+						QByteArray buf;
+						pr->addArgument(g_pConfig->GetBrowser().Data());
+						pr->addArgument("http://bitzi.com/lookup/tree:tiger:" + tth);
+						pr->launch(buf);
+						delete pr;
+					}
+				}
+				
+			}
 			else if ( g_pConfig->GetBrowser() != "" )
 			{
 				QProcess * pr = new QProcess();
@@ -500,13 +577,125 @@ void DCChat::slotClickedChatOutput( int, int )
 /** */
 void DCChat::slotRightButtonClickedChatOutput( const QPoint& )
 {
+	int para;
+	QString pressedPara, pressedNick, userCommand;
+	CString cPressedNick;
+	QPoint p;
+	int x,y, nickStart, nickLen;
 	int id;
-
-	QPopupMenu *m,*menc;
-	QString s;
-
+	bool isNickOnline;
+	QPopupMenu *m,*muser,*mslot;
+	QString s;	
+	CList<DC_UserMenuCommand> usercommands;
+	DC_UserMenuCommand * umc = 0;
+	
+	// get the paragraph of text where the right click was
+	p = m_pTextEdit_CHATOUTPUT->mapFromGlobal(QCursor::pos());
+	m_pTextEdit_CHATOUTPUT->viewportToContents(p.x(),p.y(),x,y);
+	p.setX(x);
+	p.setY(y);
+	para = m_pTextEdit_CHATOUTPUT->paragraphAt(p);
+	pressedPara = m_pTextEdit_CHATOUTPUT->text(para);
+	
+	// cut out the nick
+	nickStart = 4 + pressedPara.find("&lt;");
+	nickLen = pressedPara.find("&gt;") - nickStart;
+	
+	// sanity check
+	if ( (nickStart == 3) || (nickLen < 0) )
+	{
+		if (m_bPrivateChat)
+		{
+			pressedNick = QString(m_sNick.Data());
+		}
+		else
+		{
+			pressedNick = "Unknown";
+		}
+	}
+	else
+	{
+		pressedNick = pressedPara.mid(nickStart, nickLen);
+	}
+	
+	// check for joins / parts message
+	if ( pressedNick == "VALKNUT" )
+	{
+		int lenJoin = QString(tr("Joins: ")).length();
+		int lenPart = QString(tr("Parts: ")).length();
+		int joinPos = pressedPara.find(tr("Joins: "));
+		int partPos = pressedPara.find(tr("Parts: "));
+		
+		if ( joinPos == -1 )
+		{
+			nickStart = lenPart + partPos;
+		}
+		else
+		{
+			nickStart = lenJoin + joinPos;	
+		}
+		
+		if ( (joinPos != -1) || (partPos != -1) )
+		{
+			nickLen = pressedPara.findRev("</span>") - nickStart;
+			pressedNick = pressedPara.mid(nickStart, nickLen);
+		}
+	}
+	
+	// check for private chat
+	if ( (pressedNick == "VALKNUT") && m_bPrivateChat )
+	{
+		pressedNick = QString(m_sNick.Data());
+	}
+	
+	//printf("Debug pressedNick: %s\n",pressedNick.ascii());
+	
+	cPressedNick = CString(pressedNick.ascii());
+	isNickOnline = m_pClient->UserList()->IsUserOnline(cPressedNick);
+	
+	// select the user in the userlist
+	if (isNickOnline)
+	{
+		QListViewItem * item = m_pClient->ListView_USERLIST->findItem(pressedNick, 0);
+		m_pClient->ListView_USERLIST->clearSelection();
+		m_pClient->ListView_USERLIST->setSelected(item, TRUE);
+		m_pClient->ListView_USERLIST->setCurrentItem(item);
+		m_pClient->ListView_USERLIST->ensureItemVisible(item);
+	}
+	
 	m = new QPopupMenu(this);
-
+	
+	muser = DCMenuHandler::InsertMenu( m , emisCHAT_RIGHTCLICK_USER, TRUE, pressedNick );
+	DCMenuHandler::InsertMenu( muser, emiPRIVATE_CHAT, isNickOnline );
+	DCMenuHandler::InsertMenu( muser, emiADD_FRIEND,  isNickOnline );
+	DCMenuHandler::InsertMenu( muser, emiBROWSE_USER_FILES, isNickOnline );
+	DCMenuHandler::InsertMenu( muser, emiSEPARATOR );
+	
+	mslot = DCMenuHandler::InsertMenu( muser, emiUPLOAD_SLOT, TRUE);
+	DCMenuHandler::InsertMenu( mslot, emiADD, TRUE);
+	DCMenuHandler::InsertMenu( mslot, emiADD_PERMANENT, TRUE);
+	DCMenuHandler::InsertMenu( mslot, emiREMOVE, TRUE);
+	
+	DCMenuHandler::InsertMenu( muser, emiSEPARATOR );
+	DCMenuHandler::InsertMenu( muser, emiKICK, (isNickOnline && m_pClient->UserList()->IsAdmin(m_pClient->GetNick())) );
+	DCMenuHandler::InsertMenu( muser, emiFORCE_MOVE, (isNickOnline && m_pClient->UserList()->IsAdmin(m_pClient->GetNick())) );
+	
+	DCMenuHandler::InsertMenu( muser, emiSEPARATOR );
+	
+	g_pConfig->GetUserMenuCommands(&usercommands);
+	
+	umc = 0;
+	while ( (umc=usercommands.Next(umc)) != 0 )
+	{
+		if ( (umc->m_nContext) == 2)
+		{
+			umc->m_nID = muser->insertItem(umc->m_sName.Data(), -1);
+			muser->setItemEnabled( umc->m_nID, isNickOnline );
+		}
+	}
+	
+	DCMenuHandler::InsertMenu( m, emiSEPARATOR );
+	
 	DCMenuHandler::InsertMenu( m, emiINSERTSMILEY, g_pConfig->GetEnableEmoticons() );
 	DCMenuHandler::InsertMenu( m, emiSEPARATOR );
 	DCMenuHandler::InsertMenu( m, emiCOPY, m_pTextEdit_CHATOUTPUT->hasSelectedText() );
@@ -516,11 +705,6 @@ void DCChat::slotRightButtonClickedChatOutput( const QPoint& )
 	DCMenuHandler::InsertMenu( m, emiTRANSLATE, m_pTextEdit_CHATOUTPUT->hasSelectedText() );
 	DCMenuHandler::InsertMenu( m, emiTRANSLATOR );
 
-	// encoding menu
-	menc = DCMenuHandler::InsertMenu( m, emisTEXT_ENCODING );
-	DCMenuHandler::InsertMenu( menc, emiTEXT_INPUT_ENCODING );
-	DCMenuHandler::InsertMenu( menc, emiTEXT_OUTPUT_ENCODING );
-
 	DCMenuHandler::InsertMenu( m, emiSEPARATOR );
 	DCMenuHandler::InsertMenu( m, emiREFRESH );
 	DCMenuHandler::InsertMenu( m, emiSEPARATOR );
@@ -546,7 +730,21 @@ void DCChat::slotRightButtonClickedChatOutput( const QPoint& )
 		DCMenuHandler::InsertMenu( m, emiHIDE, m_bPrivateChat && !g_pConfig->GetShowChatInTab() );
 		DCMenuHandler::InsertMenu( m, emiCLOSE, m_bPrivateChat );
 	}
-
+	
+	if ( !m_bPrivateChat )
+	{
+		DCMenuHandler::InsertMenu( m, emiSEPARATOR );
+		
+		umc = 0;
+		while ( (umc=usercommands.Next(umc)) != 0 )
+		{
+			if ( (umc->m_nContext & 1) == 1)
+			{
+				umc->m_nID = m->insertItem(umc->m_sName.Data(), -1);
+			}
+		}
+	}
+	
 	id = m->exec(QCursor::pos());
 
 	delete m;
@@ -605,43 +803,6 @@ void DCChat::slotRightButtonClickedChatOutput( const QPoint& )
 	{
 		m_sLanguage = g_pTranslator->SelectLanguage(m_sLanguage);
 	}
-	else if ( (id == emiTEXT_INPUT_ENCODING) ||
-		  (id == emiTEXT_OUTPUT_ENCODING) )
-	{
-		QTextCodec *codec;
-		int i;
-		bool ok = FALSE;
-		QStringList lst;
-		QString res;
-
-		for (i=0;(codec = QTextCodec::codecForIndex(i));i++)
-		{
-			lst << codec->name();
-		}
-
-		if ( id == emiTEXT_INPUT_ENCODING )
-			i = lst.findIndex(g_pConfig->GetChatTextCodec(ectcINPUT).Data());
-		else
-			i = lst.findIndex(g_pConfig->GetChatTextCodec(ectcOUTPUT).Data());
-
-		if ( i == -1 )
-		{
-			i = 0;
-		}
-
-		res = QInputDialog::getItem( tr("Text Encoding"), tr("Choose a codec"), lst, i, FALSE, &ok, 0 );
-
-		if ( ok )
-		{
-			if ( QTextCodec::codecForName(res) )
-			{
-				if ( id == emiTEXT_INPUT_ENCODING )
-					g_pConfig->SetChatTextCodec( ectcINPUT, res.ascii() );
-				else
-					g_pConfig->SetChatTextCodec( ectcOUTPUT, res.ascii() );
-			}
-		}
-	}
 	else if ( id == emiCOPY )
 	{
 		m_pTextEdit_CHATOUTPUT->copy();
@@ -734,6 +895,69 @@ void DCChat::slotRightButtonClickedChatOutput( const QPoint& )
 		setEnabled(FALSE);
 		hide();
 	}
+	else if ( id == emiPRIVATE_CHAT )
+	{
+		m_pClient->DC_PrivateChat( pressedNick, "", "", TRUE );
+	}
+	else if ( id == emiADD_FRIEND )
+	{
+		g_pUsersList->AddFriend( cPressedNick,
+				m_pClient->GetHubName(), m_pClient->GetIP(), "" );
+	}
+	else if ( id == emiBROWSE_USER_FILES )
+	{
+		g_pTransferView->DLM_QueueAdd( cPressedNick, m_pClient->GetHubName(),
+				m_pClient->GetIP()+":"+QString().setNum(m_pClient->GetPort()).ascii(),
+				DC_USER_FILELIST_HE3, DC_USER_FILELIST_HE3, "", "", eltBUFFER,
+				0, 0, 0, ""  );
+	}
+	else if ( id == emiKICK )
+	{
+		QString kickMessage;
+		
+		if ( !m_pClient->GetOPKickMessage(kickMessage) )
+		{
+			return;
+		}
+		m_pClient->OPKick( pressedNick, kickMessage );
+		
+	}
+	else if ( id == emiFORCE_MOVE )
+	{
+		QString host, message;
+		if ( !m_pClient->GetOPForceMoveMessage(message,host) )
+		{
+			return;
+		}
+		m_pClient->OPForceMove(pressedNick, message, host);
+	}
+	else if ( id == emiADD )
+	{
+		g_pTransferView->DLM_AddUserSlot(cPressedNick, m_pClient->GetHubName(), 1);
+	}
+	else if ( id == emiADD_PERMANENT )
+	{
+		g_pTransferView->DLM_AddUserSlot(cPressedNick, m_pClient->GetHubName(), 0);
+		g_pTransferView->DLM_AddUserSlot(cPressedNick, m_pClient->GetHubName(), 0, TRUE);
+	}
+	else if ( id == emiREMOVE )
+	{
+		g_pTransferView->DLM_AddUserSlot(cPressedNick, m_pClient->GetHubName(), 0);
+	}
+	else
+	{
+		umc = 0;
+		while ( (umc=usercommands.Next(umc)) != 0 )
+		{
+			if ( id == umc->m_nID )
+			{
+				userCommand = m_pClient->replaceCommandTags( umc->m_sCommand.Data(), pressedNick );
+				
+				AddStatus(userCommand.ascii());
+				m_pClient->SendString(userCommand.ascii());
+			}
+		}
+	}
 }
 
 /** */
@@ -790,11 +1014,11 @@ void DCChat::SetNick( CString nick, CString hubname )
 
 	if ( nick != "" )
 	{
-		setCaption(tr("Private Chat:")+" "+nick.Data()+" ["+hubname.Data()+"]");
+		setCaption(tr("Private Chat:")+" "+QString::fromUtf8(nick.Data())+" ["+QString::fromUtf8(hubname.Data())+"]");
 	}
 	else
 	{
-		setCaption(tr("Chat:")+" ["+hubname.Data()+"]");
+		setCaption(tr("Chat:")+" ["+QString::fromUtf8(hubname.Data())+"]");
 	}
 }
 
@@ -822,7 +1046,6 @@ void DCChat::AddOutput( CString message )
 {
 	bool bscroll;
 	CString s;
-	QTextCodec *codec;
 
 	if ( m_pTextEdit_CHATOUTPUT->verticalScrollBar()->maxValue() == m_pTextEdit_CHATOUTPUT->verticalScrollBar()->value() )
 	{
@@ -836,6 +1059,9 @@ void DCChat::AddOutput( CString message )
 	// convert newline
 	message = message.Replace( "\xa\\r", "\xa" );
 	message = message.Replace( "\xa", "<br />" );
+	
+	// convert tabs to spaces
+	message = message.Replace( "	", "&nbsp;&nbsp;&nbsp;&nbsp;" );
 
 	if ( (g_pConfig->GetChatMaxParagraph()!= 0) && (m_pTextEdit_CHATOUTPUT->paragraphs() > (g_pConfig->GetChatMaxParagraph()+5)) )
 	{
@@ -845,18 +1071,8 @@ void DCChat::AddOutput( CString message )
 		m_pTextEdit_CHATOUTPUT->removeParagraph(0);
 		m_pTextEdit_CHATOUTPUT->removeParagraph(0);
 	}
-
-	// use text encoder
-	s = g_pConfig->GetChatTextCodec( ectcOUTPUT );
-
-	if ( (s != "") && ( ( codec = QTextCodec::codecForName(s.Data())) != 0) )
-	{
-		m_pTextEdit_CHATOUTPUT->append( codec->toUnicode(message.Data()) );
-	}
-	else
-	{
-		m_pTextEdit_CHATOUTPUT->append( message.Data() );
-	}
+	
+	m_pTextEdit_CHATOUTPUT->append( QString::fromUtf8(message.Data()) );
 
 	if ( bscroll )
 	{
@@ -969,6 +1185,7 @@ void DCChat::AddMessage( CString nick, CString message, bool bremote, bool forwa
 	bool b1,b2;
 	bool bSay = FALSE;
 	bool bMe = FALSE;
+	bool isOP = m_pClient->UserList()->IsAdmin( nick );
 
 	//printf("ADD MESSAGE '%s'\n",message.Data());
 
@@ -996,7 +1213,7 @@ void DCChat::AddMessage( CString nick, CString message, bool bremote, bool forwa
 
 	s1 = m_pClient->GetNick() + ": ";
 
-	if ( message.Left(4).ToUpper() == "/ME " )
+	if ( ( message.Left(4).ToUpper() == "/ME " ) || ( message.Left(4).ToUpper() == "+ME " ) )
 	{
 		bMe    = TRUE;
 		message = message.Right( message.Length() - 4 );
@@ -1005,7 +1222,7 @@ void DCChat::AddMessage( CString nick, CString message, bool bremote, bool forwa
 	if ( message.Left(s1.Length()) == s1 )
 	{
 		bSay    = TRUE;
-		message = message.Right( message.Length() - s1.Length() );
+		// message = message.Right( message.Length() - s1.Length() );
 	}
 
 	msg = message;
@@ -1139,7 +1356,12 @@ void DCChat::AddMessage( CString nick, CString message, bool bremote, bool forwa
 	if ( m_bPrivateChat )
 	{
 		if ( nick != m_pClient->GetNick().Data() )
-			message = "<font color=\"" + g_pConfig->GetChatColor(eccCHATREMOTENICK) + "\">" + "<b>" + nick + "</b>: " + message + "</font>";
+		{
+			if ( isOP )
+				message = "<font color=\"" + g_pConfig->GetChatColor(eccOPNICK) + "\">" + "<b>" + nick + "</b>: " + message + "</font>";
+			else
+				message = "<font color=\"" + g_pConfig->GetChatColor(eccCHATREMOTENICK) + "\">" + "<b>" + nick + "</b>: " + message + "</font>";
+		}
 		else
 			message = "<font color=\"" + g_pConfig->GetChatColor(eccCHATLOCALNICK) + "\">" + "<b>" + nick + "</b>: " + message + "</font>";
 	}
@@ -1151,7 +1373,12 @@ void DCChat::AddMessage( CString nick, CString message, bool bremote, bool forwa
 			if ( bMe )
 				message = "<font color=\"" + g_pConfig->GetChatColor(eccPUBLICCHATMENICK) + "\">" + "<b>" + nick + "</b> " + "</font>" + message;
 			else
-				message = "<font color=\"" + g_pConfig->GetChatColor(eccPUBLICCHATREMOTENICK) + "\">" + "<b>&lt;" + nick + "&gt;</b> " + "</font>" + message;
+			{
+				if ( isOP )
+					message = "<font color=\"" + g_pConfig->GetChatColor(eccOPNICK) + "\">" + "<b>&lt;" + nick + "&gt;</b> " + "</font>" + message;
+				else
+					message = "<font color=\"" + g_pConfig->GetChatColor(eccPUBLICCHATREMOTENICK) + "\">" + "<b>&lt;" + nick + "&gt;</b> " + "</font>" + message;
+			}
 		else
 			if ( bMe )
 				message = "<font color=\"" + g_pConfig->GetChatColor(eccPUBLICCHATMENICK) + "\">" + "<b>" + nick + "</b> " + "</font>" + message;			
@@ -1176,6 +1403,7 @@ void DCChat::AddMessage( CString nick, CString message, bool bremote, bool forwa
 			{
 				m_pClient->SendPrivateMessage( m_pClient->GetNick(), m_sNick, s );
 				m_bSendAway = FALSE;
+				AddStatus("Sent away message: " + s);
 			}
 		}
 		else
@@ -1189,7 +1417,6 @@ void DCChat::AddMessage( CString nick, CString message, bool bremote, bool forwa
 void DCChat::SendMessage( CString message )
 {
 	int err;
-	QTextCodec *codec;
 	CString s;
 	bool local = FALSE;
 	bool cmd = FALSE;
@@ -1201,17 +1428,7 @@ void DCChat::SendMessage( CString message )
 
 	if ( message == "" )
 	{
-		// use text encoder
-		s = g_pConfig->GetChatTextCodec( ectcINPUT );
-
-		if ( (s != "") && ( ( codec = QTextCodec::codecForName(s.Data())) != 0) )
-		{
-			message = (const char*)codec->fromUnicode(TextEdit_CHATINPUT->text());
-		}
-		else
-		{
-			message = TextEdit_CHATINPUT->text().ascii();
-		}
+		message = TextEdit_CHATINPUT->text().ascii();
 	}
 	else
 	{
@@ -1372,11 +1589,36 @@ int DCChat::ConvertEmoticons( CString msg, CString & s )
 bool DCChat::CheckForCommand()
 {
 	bool res = FALSE;
+	bool addedSpace = FALSE;
 	QString timestamp;
 	CString cmd,s;
-
+	CList<DC_UserChatCommand> usercommands;
+	DC_UserChatCommand * ucc = 0;
+	
 	cmd = TextEdit_CHATINPUT->text().ascii();
-
+	
+	// replace custom commands with their actions
+	if ( cmd.Right(1) != " " )
+	{
+		cmd = cmd + " ";
+		addedSpace = TRUE;
+	}
+	
+	g_pConfig->GetUserChatCommands(&usercommands);
+	
+	while( (ucc=usercommands.Next(ucc)) != 0 )
+	{
+		if ( cmd.Left(ucc->m_sName.Length() + 1) == (ucc->m_sName + " ") )
+		{
+			cmd = cmd.Replace( ucc->m_sName, ucc->m_sCommand );
+		}
+	}
+		
+	if (addedSpace)
+	{
+		cmd = cmd.Left(cmd.Length() - 1);
+	}
+	
 	if ( (cmd != "") && (cmd.Left(1) == '/') )
 	{
 		// help
@@ -1390,6 +1632,10 @@ bool DCChat::CheckForCommand()
 			s += "\n";
 			s += tr("/refresh - refresh share").ascii();
 			s += "\n";
+			s += tr("/rebuild - remove no longer present files from hash database").ascii();
+			s += "\n";
+			s += tr("/slots &lt;N&gt; - set number of upload slots to N");
+			s += "\n";
 			s += tr("/ts - switch time display in chat on/off").ascii();
 			s += "\n";
 			s += tr("/adv - send an advertisment to the hub").ascii();
@@ -1402,7 +1648,7 @@ bool DCChat::CheckForCommand()
 			s += "\n";
 			s += tr("/bye &lt;message&gt; - disconnect from channel with an optional message").ascii();
 			s += "\n";
-			s += tr("/uptime - show valknut uptime").ascii();
+			s += tr("/uptime [show] - show valknut uptime [to other users]").ascii();
 			s += "\n";
 			s += tr("/ls &lt;nick&gt; - get share list from user").ascii();
 			s += "\n";
@@ -1414,12 +1660,22 @@ bool DCChat::CheckForCommand()
 			s += "\n";
 			s += tr("/fav - add hub to bookmark list").ascii();
 			s += "\n";
-			s += tr("/info - show user info").ascii();
+			s += tr("/info &lt;nick&gt; - show user info").ascii();
 			s += "\n";
 			s += tr("/now - send current time to the chat").ascii();
 			s += "\n";
+			s += tr("/sh &lt;command&gt; &lt;args...&gt; - send output of shell command to chat").ascii();
+			s += "\n";
 			s += tr("/raw - send raw message").ascii();
 			s += "\n";
+			s += tr("/disablesorting - disable sorting of the user list").ascii();
+			s += "\n";
+			s += tr("/enablesorting - enable sorting of the user list").ascii();
+			s += "\n";
+			s += tr("/ignore &lt;nick&gt; - do not show main chat lines from the user").ascii();
+			s += "\n";
+			s += tr("/unignore &lt;nick&gt - show main chat lines from the user again").ascii();
+			s += "\n";
 			AddStatus( s, TRUE );
 			res = TRUE;
 		}
@@ -1432,11 +1688,21 @@ bool DCChat::CheckForCommand()
 			res = TRUE;
 		}
 		// /info - show user info
-		else if ( cmd == "/info" )
+		else if ( cmd.Left(5) == "/info" )
 		{
+			CString nick;
 			CMessageMyInfo myinfo;
-
-			if ( !m_bPrivateChat || (m_pClient->UserList()->GetUserMyInfo( GetNick(), &myinfo ) == FALSE) )
+			
+			if ( m_bPrivateChat && (cmd.Length() == 5) )
+			{
+				nick = m_sNick;
+			}
+			else
+			{
+				nick = cmd.Mid(6,cmd.Length()-6);	
+			}
+						
+			if (m_pClient->UserList()->GetUserMyInfo( nick, &myinfo ) == FALSE)
 			{
 				s = tr("Get info failed.");
 			}
@@ -1486,6 +1752,12 @@ bool DCChat::CheckForCommand()
 					case eucvDCGUI:
 						s += "DCGUI";
 						break;
+					case eucvMICRODC:
+						s += "microdc";
+						break;
+					case eucvSHAKESPEER:
+						s += "ShakesPeer";
+						break;
 					default:
  						s += tr("Unknown");
 						break;
@@ -1500,7 +1772,7 @@ bool DCChat::CheckForCommand()
 			res = TRUE;
 		}
 		// /uptime - show valknut uptime
-		else if ( cmd == "/uptime" )
+		else if ( cmd.Left(7) == "/uptime" )
 		{
 			int d,h,m;
 			int i = time(0)-dclibUptime();
@@ -1511,7 +1783,7 @@ bool DCChat::CheckForCommand()
 			i = i%(60*60);
 			m = i/60;
 
-			s  = tr("Uptime") + ": ";
+			s  = tr("Valknut uptime") + ": ";
 
 			if ( d > 0 )
 			{
@@ -1537,8 +1809,12 @@ bool DCChat::CheckForCommand()
 				s += tr("minute");
 			else
 				s += tr("minutes");
+			
+			if ( cmd.Mid(8, 4).ToLower() == "show" )
+				SendMessage(s);
+			else
+				AddStatus( s, TRUE );
 
-			AddStatus( s, TRUE );
 			res = TRUE;
 		}
 		// /mode - show current mode
@@ -1584,6 +1860,38 @@ bool DCChat::CheckForCommand()
 			AddStatus( s, TRUE );
 			res = TRUE;
 		}
+		// /rebuild - clean up share databases
+		else if ( cmd == "/rebuild" )
+		{
+			if ( CFileManager::Instance()->RebuildLists() == TRUE )
+			{
+				s = tr("Rebuild share in progress.").ascii();
+			}
+			else
+			{
+				s = tr("Refresh/rebuild already in progress.").ascii();
+			}
+			
+			AddStatus( s, TRUE );
+			res = TRUE;
+		}
+		// /slots - set number of slots
+		else if ( cmd.Left(7) == "/slots " )
+		{
+			int i = (cmd.Mid(7, cmd.Length() - 7)).asINT();
+			
+			if ( (i >= 0) && (i <= 99) )
+			{
+				g_pConfig->SetMaxUpload(i);
+				AddStatus( tr("Setting upload slots to: " + QString::number(i)).ascii(), TRUE );
+			}
+			else
+			{
+				AddStatus( tr("Invalid number of upload slots specified.").ascii(), TRUE );
+			}
+			
+			res = TRUE;
+		}
 		// /ts - switch time display in chat on/off 
 		else if ( cmd == "/ts" )
 		{
@@ -1615,7 +1923,7 @@ bool DCChat::CheckForCommand()
 		{
 			AddStatus( tr("Send advertisment.").ascii(), TRUE );
 
-			s = tr("\nDownload Valknut from http://dcgui.berlios.de\nIt does everything !\nWorks on almost any platform you can think of and makes you coffee.\n");
+			s = tr("\nDownload Valknut from http://wxdcgui.sourceforge.net\nIt does everything !\nWorks on almost any platform you can think of and makes you coffee.\n");
 
 			SendMessage(s);
 
@@ -1647,6 +1955,7 @@ bool DCChat::CheckForCommand()
 			
 				if ( msg != "" )
 				{
+					AddStatus( msg, TRUE );
 					m_pClient->SendString(msg);
 				}
 			}
@@ -1821,6 +2130,8 @@ bool DCChat::CheckForCommand()
 		{
 			int i;
 			CString nick,message="";
+			
+			AddStatus( cmd, TRUE );
 
 			cmd = cmd.Mid(5,cmd.Length()-5);
 
@@ -1873,6 +2184,81 @@ bool DCChat::CheckForCommand()
 			AddStatus( s, TRUE );
 			res = TRUE;
 		}
+		else if ( cmd.Left(4) == "/sh " )
+		{
+			// display the command the user entered
+			AddStatus( cmd, TRUE );
+			
+			// actually this is better - more portable, doesn't depend on /bin/sh
+			QStringList shellargs = QStringList::split( QString(" "), QString((cmd.Mid(4, cmd.Length() - 4)).Data()) );
+			
+			// QStringList shellargs;
+			// shellargs.append( QString("/bin/sh") );
+			// shellargs.append( QString("-c") );
+			// shellargs.append( QString((cmd.Mid(4, cmd.Length() - 4)).Data()) );
+			
+			// debug code - show the args we are going to run
+			//AddStatus( (shellargs.join( QString(",") )).ascii(), TRUE );
+			
+			QProcess shellproc( shellargs, this );
+			shellproc.setCommunication( QProcess::Stdout );
+			
+			if ( shellproc.start() )
+			{
+				// wait up to 5 seconds for the process to finish
+				int i = 0;
+				while ( (shellproc.isRunning()) && (i < 5) )
+				{
+					sleep(1);
+					i++;
+				}
+				
+				shellproc.closeStdin();
+				
+				// kill the process if it's still running
+				if ( shellproc.isRunning() )
+				{
+					AddStatus( tr("Process still running after 5 seconds, killing process...").ascii(), TRUE );
+					shellproc.tryTerminate();
+					sleep(1);
+					if ( shellproc.isRunning() )
+					{
+						shellproc.kill();
+					}
+				}
+	
+				if ( shellproc.normalExit() )
+				{
+					int exitcode = shellproc.exitStatus();
+					if ( exitcode == 0 )
+					{
+						s = QString( shellproc.readStdout() ).stripWhiteSpace();
+						if ( s == "" )
+						{
+							AddStatus( tr("Command produced no visible output.").ascii(), TRUE );
+						}
+						else
+						{
+							SendMessage(s);
+						}
+					}
+					else
+					{
+						AddStatus( tr("Process exited with status " + QString::number(exitcode)).ascii(), TRUE);
+					}
+				}
+				else
+				{
+					AddStatus( tr("Process was killed or crashed.").ascii(), TRUE);
+				}
+			}
+			else
+			{
+				AddStatus( tr("Failed to start shell command.").ascii(), TRUE );
+			}
+			
+			res = TRUE;
+		}
 		else if ( cmd == "/sendphoto" )
 		{
 			if ( !m_bPrivateChat )
@@ -1936,6 +2322,66 @@ bool DCChat::CheckForCommand()
 				f.Close();
 			}
 
+			AddStatus( s, TRUE );
+			res = TRUE;
+		}
+		else if ( cmd == "/disablesorting" )
+		{
+			m_pClient->ListView_USERLIST->setSortColumn(-1);
+			s = tr("Disabled user list sorting.").ascii();
+			AddStatus( s, TRUE );
+			res = TRUE;
+		}
+		else if ( cmd == "/enablesorting" )
+		{
+			m_pClient->ListView_USERLIST->setSortColumn(0);
+			s = tr("Enabled user list sorting.").ascii();
+			AddStatus( s, TRUE );
+			res = TRUE;
+		}
+		else if (cmd.Left(7) == "/ignore")
+		{
+			CString nick = cmd.Mid(8, cmd.Length()-8);
+			if (nick != "")
+			{
+				if (g_pUsersList->isNickInList(nick) == FALSE)
+				{
+					g_pUsersList->AddFriend( nick, m_pClient->GetHubName(), m_pClient->GetHost(), "" );
+				}
+				
+				g_pUsersList->setIgnore( nick, TRUE );
+				s = tr(QString("Ignoring main chat lines from ") + QString(nick.Data()));
+			}
+			else
+			{
+				s = tr("No nick, try /dchelp");
+			}
+			
+			AddStatus( s, TRUE );
+			res = TRUE;
+			
+		}
+		else if (cmd.Left(9) == "/unignore")
+		{
+			CString nick = cmd.Mid(10, cmd.Length()-10);
+			if (nick != "")
+			{
+				if (g_pUsersList->ignoreNick( nick ) == TRUE)
+				{
+					g_pUsersList->setIgnore( nick, FALSE);
+					s = tr(QString("Showing main chat for ") + QString(nick.Data()));
+				}
+				else
+				{
+					s = tr(QString(nick.Data()) + QString(" is not in ignore list"));
+				}
+				
+			}
+			else
+			{
+				s = tr("No nick, try /dchelp");
+			}
+			
 			AddStatus( s, TRUE );
 			res = TRUE;
 		}
