@@ -181,6 +181,10 @@ void DCClient::InitDocument()
 		delete page;
 
 		TabWidget_CHAT->setCurrentPage(0);
+		
+		TabWidget_CHAT->setCornerWidget( new QPushButton( QIconSet( g_pIconLoader->GetPixmap(eiFILECLOSE) ), "", TabWidget_CHAT ) );
+		QToolTip::add( TabWidget_CHAT->cornerWidget(), tr("Close chat tab") );
+		TabWidget_CHAT->cornerWidget()->setEnabled(FALSE);
 	}
 	else
 	{
@@ -245,12 +249,40 @@ void DCClient::InitDocument()
 	// set viewable columns
 	ResizeListViewColumn();
 
+	// load column order from config
+	if ( g_pConfig->GetMap("USERLISTCOLUMNORDER",map) == TRUE)
+	{
+		if ( (*map)[QString().setNum(eclcNICK)].toInt() >= 0 )
+			ListView_USERLIST->header()->moveSection(eclcNICK, (*map)[QString().setNum(eclcNICK)].toInt());
+		if ( (*map)[QString().setNum(eclcCOMMENT)].toInt() >= 0 )
+			ListView_USERLIST->header()->moveSection(eclcCOMMENT, (*map)[QString().setNum(eclcCOMMENT)].toInt());
+		if ( (*map)[QString().setNum(eclcTAG)].toInt() >= 0 )
+			ListView_USERLIST->header()->moveSection(eclcTAG, (*map)[QString().setNum(eclcTAG)].toInt());
+		if ( (*map)[QString().setNum(eclcEMAIL)].toInt() >= 0 )
+			ListView_USERLIST->header()->moveSection(eclcEMAIL, (*map)[QString().setNum(eclcEMAIL)].toInt());
+		if ( (*map)[QString().setNum(eclcSPEED)].toInt() >= 0 )
+			ListView_USERLIST->header()->moveSection(eclcSPEED, (*map)[QString().setNum(eclcSPEED)].toInt());
+		if ( (*map)[QString().setNum(eclcSHARE)].toInt() >= 0 )
+			ListView_USERLIST->header()->moveSection(eclcSHARE, (*map)[QString().setNum(eclcSHARE)].toInt());
+		if ( (*map)[QString().setNum(eclcIP)].toInt() >= 0 )
+			ListView_USERLIST->header()->moveSection(eclcIP, (*map)[QString().setNum(eclcIP)].toInt());
+		if ( (*map)[QString().setNum(eclcLOCKPK)].toInt() >= 0 )
+			ListView_USERLIST->header()->moveSection(eclcLOCKPK, (*map)[QString().setNum(eclcLOCKPK)].toInt());
+		if ( (*map)[QString().setNum(eclcSUPPORTS)].toInt() >= 0 )
+			ListView_USERLIST->header()->moveSection(eclcSUPPORTS, (*map)[QString().setNum(eclcSUPPORTS)].toInt());
+	}
+	
 	connect( ListView_USERLIST,SIGNAL(doubleClicked(QListViewItem*)), this, SLOT(slotDoubleClickedUserList(QListViewItem*)) );
 	connect( ListView_USERLIST,SIGNAL(contextMenuRequested( QListViewItem *, const QPoint &, int )), this, SLOT(slotRightButtonClickedUserList(QListViewItem*, const QPoint &, int )) );
 	connect( ListView_USERLIST->header(),SIGNAL(sizeChange(int, int, int)), this, SLOT(slotSaveColumnWidth(int, int, int)) );
 	connect( TabWidget_CHAT,SIGNAL(currentChanged(QWidget*)), this, SLOT(slotTabWidgetChatCurrentChange(QWidget*)) );
 	connect( ToolButton_CONNECT,SIGNAL(clicked()), this, SLOT(slotHubConnect()) );
 	connect( ToolButton_SSL,SIGNAL(clicked()), this, SLOT(slotSSLInfo()) );
+	
+	if (m_bUseTabWidget)
+	{
+		connect( TabWidget_CHAT->cornerWidget(), SIGNAL(clicked()), this, SLOT(slotTabCornerCloseWidgetClicked()) );
+	}
 
 	// set connect disconnect button pixmap
 	SetConnection(FALSE);
@@ -281,6 +313,23 @@ void DCClient::DeInitDocument()
 
 	SaveClientColumn();
 
+	// save column order to config
+	/* for (int i = 0; i < ListView_USERLIST->columns(); i ++)
+	 *{
+	 *	printf("Column %d is at position %d\n", i, ListView_USERLIST->header()->mapToIndex(i) );
+	 *}
+	 */
+	 g_pConfig->GetMap("USERLISTCOLUMNORDER",map);
+	 (*map)[QString().setNum(eclcNICK)] = QString().setNum(ListView_USERLIST->header()->mapToIndex(eclcNICK));
+	 (*map)[QString().setNum(eclcCOMMENT)] = QString().setNum(ListView_USERLIST->header()->mapToIndex(eclcCOMMENT));
+	 (*map)[QString().setNum(eclcTAG)] = QString().setNum(ListView_USERLIST->header()->mapToIndex(eclcTAG));
+	 (*map)[QString().setNum(eclcEMAIL)] = QString().setNum(ListView_USERLIST->header()->mapToIndex(eclcEMAIL));
+	 (*map)[QString().setNum(eclcSPEED)] = QString().setNum(ListView_USERLIST->header()->mapToIndex(eclcSPEED));
+	 (*map)[QString().setNum(eclcSHARE)] = QString().setNum(ListView_USERLIST->header()->mapToIndex(eclcSHARE));
+	 (*map)[QString().setNum(eclcIP)] = QString().setNum(ListView_USERLIST->header()->mapToIndex(eclcIP));
+	 (*map)[QString().setNum(eclcLOCKPK)] = QString().setNum(ListView_USERLIST->header()->mapToIndex(eclcLOCKPK));
+ 	 (*map)[QString().setNum(eclcSUPPORTS)] = QString().setNum(ListView_USERLIST->header()->mapToIndex(eclcSUPPORTS));
+	
 	if ( m_pChatList )
 	{
 		while( m_pChatList->Next( (CObject *&)chat ) )
@@ -408,7 +457,19 @@ void DCClient::slotTabWidgetChatCurrentChange( QWidget * w )
 		TabWidget_CHAT->setTabIconSet( w, QIconSet() );
 		
 		// reset nick color
-		SetNickColor( chat->GetNick().Data(), ListView_USERLIST->colorGroup().text() );
+		SetNickColor( QString::fromUtf8(chat->GetNick().Data()), ListView_USERLIST->colorGroup().text() );
+	}
+	
+	if (m_bUseTabWidget)
+	{
+		if (TabWidget_CHAT->currentPage() == m_pHubChat)
+		{
+			TabWidget_CHAT->cornerWidget()->setEnabled(FALSE);
+		}
+		else
+		{
+			TabWidget_CHAT->cornerWidget()->setEnabled(TRUE);
+		}
 	}
 }
 
@@ -438,6 +499,7 @@ void DCClient::UpdateClientColumn( bool enabled, int index, int width )
 	}
 	else
 	{
+		if ( width < 10 ) { width = 10; } // otherwise newly-enabled columns remain hidden
 		ListView_USERLIST->header()->setResizeEnabled(TRUE, index);
 		ListView_USERLIST->setColumnWidth( index, width );
 	}
@@ -518,14 +580,19 @@ void DCClient::UpdateCaption()
 	}
 
 	s += " ";
-	s += GetHubName().Data();
+	s += QString::fromUtf8(GetHubName().Data());
 
 	if ( GetHubTopic() != "" )
 	{
 		s += " (";
-		s += GetHubTopic().Data();
+		s += QString::fromUtf8(GetHubTopic().Data());
 		s += ")";
 	}
+	
+	if (s.length() > 50)
+	{
+		s = s.left(50) + "...";
+	}
 
 	setCaption(s);
 }
@@ -687,7 +754,7 @@ void DCClient::timerDone()
 				{
 					if ( g_pConfig->GetSendMessageOnActiveModeRequest() == TRUE )
 					{
-						SendPrivateMessage( GetNick(), msg->m_sDstNick, tr("Sorry, this client works in passive mode").ascii() );
+						SendPrivateMessage( GetNick(), msg->m_sDstNick, tr("Sorry, you will not be able to download from me, because we are both in passive mode. (automated message)").ascii() );
 					}
 				}
 
@@ -848,6 +915,7 @@ void DCClient::timerDone()
 						{
 							m_pHubChat->AddStatus(tr("Use password from profile").ascii());
 							SendPass(pConfigHubProfile.m_sPassword);
+							SetUsedPassword(TRUE);
 							m_bUserPassword = TRUE;
 							break;
 						}
@@ -870,6 +938,7 @@ void DCClient::timerDone()
 				if ( ok && !text.isEmpty() )
 				{
 					SendPass(text.ascii());
+					SetUsedPassword(TRUE);
 				}
 				else
 				{
@@ -882,6 +951,7 @@ void DCClient::timerDone()
 			case DC_MESSAGE_BADPASS:
 			{
 				m_pHubChat->AddStatus(tr("Wrong password").ascii());
+				SetUsedPassword(FALSE);
 				break;
 			}
 
@@ -930,6 +1000,13 @@ int DCClient::DC_CallBack( CObject * Object )
 /** chat */
 void DCClient::DC_Chat( CMessageChat * MessageChat )
 {
+	// check for ignored nick
+	if ( g_pUsersList->ignoreNick( MessageChat->m_sNick ) == TRUE )
+	{
+		printf("Ignoring \"%s\" from \"%s\"\n", MessageChat->m_sMessage.Data(), MessageChat->m_sNick.Data());
+		return;
+	}
+	
 	m_pHubChat->AddMessage(MessageChat->m_sNick.Data(),MessageChat->m_sMessage.Data(),TRUE);
 
 	if ( !m_pHubChat->isVisible() )
@@ -939,6 +1016,12 @@ void DCClient::DC_Chat( CMessageChat * MessageChat )
 	
 	// send chat event
 	g_pConnectionManager->HubEvent(this);
+	
+	// check for auto responses
+	if (g_pConfig->GetAutoResponderEnabled() == TRUE)
+	{
+		this->doAutoResponse( MessageChat->m_sNick.Data(), MessageChat->m_sMessage.Data(), m_pHubChat );
+	}
 }
 
 /** myinfo */
@@ -957,9 +1040,9 @@ void DCClient::DC_MyInfo( CMessageMyInfo * MessageMyInfo )
 
 #ifndef WIN32
 		pm = *g_pConfig->GetUserIcon(MessageMyInfo->m_bOperator,MessageMyInfo->m_eClientMode,MessageMyInfo->m_eAwayMode,MessageMyInfo->m_eUserSpeed,MessageMyInfo->m_eClientVersion);
-		pui->pItem = new DC_UserListItem( ListView_USERLIST, MessageMyInfo->m_sNick.Data(), pm );
+		pui->pItem = new DC_UserListItem( ListView_USERLIST, QString::fromUtf8(MessageMyInfo->m_sNick.Data()), pm );
 #else
-		pui->pItem = new DC_UserListItem( ListView_USERLIST, MessageMyInfo->m_sNick.Data() );
+		pui->pItem = new DC_UserListItem( ListView_USERLIST, QString::fromUtf8(MessageMyInfo->m_sNick.Data()) );
 #endif
 		// user not in the list, add it
 		addUser( MessageMyInfo->m_sNick.Data() );
@@ -981,10 +1064,10 @@ void DCClient::DC_MyInfo( CMessageMyInfo * MessageMyInfo )
 
 	pui->MyInfo = *MessageMyInfo;
 
-	pui->pItem->setText(1,MessageMyInfo->m_sComment.Data());
+	pui->pItem->setText(1,QString::fromUtf8(MessageMyInfo->m_sComment.Data()));
 	pui->pItem->setText(2,MessageMyInfo->m_sVerComment.Data());
 	pui->pItem->setText(3,MessageMyInfo->m_sUserSpeed.Data());
-	pui->pItem->setText(4,MessageMyInfo->m_sEMail.Data());
+	pui->pItem->setText(4,QString::fromUtf8(MessageMyInfo->m_sEMail.Data()));
 
 	pui->pItem->myvalue = MessageMyInfo->m_nShared;
 	pui->pItem->mycol   = 5;
@@ -1025,9 +1108,9 @@ void DCClient::DC_Hello( CMessageHello * MessageHello )
 
 #ifndef WIN32
 		pm = *g_pConfig->GetUserIcon(FALSE,ecmNONE,euamNONE,eusUNKNOWN,eucvNONE);
-		pui->pItem = new DC_UserListItem( ListView_USERLIST, MessageHello->m_sNick.Data(), pm );
+		pui->pItem = new DC_UserListItem( ListView_USERLIST, QString::fromUtf8(MessageHello->m_sNick.Data()), pm );
 #else
-		pui->pItem = new DC_UserListItem( ListView_USERLIST, MessageHello->m_sNick.Data() );
+		pui->pItem = new DC_UserListItem( ListView_USERLIST, QString::fromUtf8(MessageHello->m_sNick.Data()) );
 #endif
 		pui->MyInfo.m_sNick = MessageHello->m_sNick;
 		pui->MyInfo.m_bOperator = FALSE;
@@ -1074,7 +1157,17 @@ void DCClient::DC_Quit( CMessageQuit * MessageQuit )
 
 	if ( g_pConfig->GetChatShowJoinsAndParts() == TRUE )
 	{
-		m_pHubChat->AddStatus(tr("Parts: ") + MessageQuit->m_sNick);
+		if ( g_pConfig->GetChatShowJoinsAndPartsOnlyFav() == TRUE )
+		{
+			if ( g_pUsersList->isNickInList( MessageQuit->m_sNick ) )
+			{
+				m_pHubChat->AddStatus(tr("Parts: ") + MessageQuit->m_sNick);
+			}
+		}
+		else
+		{
+			m_pHubChat->AddStatus(tr("Parts: ") + MessageQuit->m_sNick);
+		}
 	}
 
 	UpdateStatusBar();
@@ -1107,9 +1200,9 @@ void DCClient::DC_NickList( CMessageNickList * MessageNickList )
 	
 #ifndef WIN32
 				pm = *g_pConfig->GetUserIcon(FALSE,ecmNONE,euamNONE,eusUNKNOWN,eucvNONE);
-				pui->pItem = new DC_UserListItem( ListView_USERLIST, (*Nick).Data(), pm );
+				pui->pItem = new DC_UserListItem( ListView_USERLIST, QString::fromUtf8((*Nick).Data()), pm );
 #else
-				pui->pItem = new DC_UserListItem( ListView_USERLIST, (*Nick).Data() );
+				pui->pItem = new DC_UserListItem( ListView_USERLIST, QString::fromUtf8((*Nick).Data()) );
 #endif
 				pui->pItem->mycol = 5;
 				pui->MyInfo.m_sNick = (*Nick);
@@ -1162,9 +1255,9 @@ void DCClient::DC_OpList( CMessageNickList * MessageOpList )
 
 #ifndef WIN32
 			pm = *g_pConfig->GetUserIcon(TRUE,ecmNONE,euamNONE,eusUNKNOWN,eucvNONE);
-			pui->pItem = new DC_UserListItem( ListView_USERLIST, (*Nick).Data(), pm );
+			pui->pItem = new DC_UserListItem( ListView_USERLIST, QString::fromUtf8((*Nick).Data()), pm );
 #else
-			pui->pItem = new DC_UserListItem( ListView_USERLIST, (*Nick).Data() );
+			pui->pItem = new DC_UserListItem( ListView_USERLIST, QString::fromUtf8((*Nick).Data()) );
 #endif
 			pui->pItem->mycol = 5;
 			pui->MyInfo.m_sNick = (*Nick);
@@ -1217,6 +1310,13 @@ void DCClient::DC_PrivateChat( CMessagePrivateChat * msg, bool bLocal )
 	{
 		return;
 	}
+	
+	// also check for ignored nick for PMs
+	if ( g_pUsersList->ignoreNick( msg->m_sSrcNick ) == TRUE )
+	{
+		printf("Ignoring PM \"%s\" from \"%s\"\n", msg->m_sMessage.Data(), msg->m_sSrcNick.Data() );
+		return;
+	}
 
 	// create a new chat window
 	if( m_pChatList->Get( msg->m_sSrcNick, (CObject*&)chat ) != 0 )
@@ -1230,7 +1330,7 @@ void DCClient::DC_PrivateChat( CMessagePrivateChat * msg, bool bLocal )
 		else
 		{
 			chat = new DCChat( m_pWorkspace, "chat", WDestructiveClose, this );
-			new QListViewItem(ListView_CHATUSERLIST,msg->m_sSrcNick.Data());
+			new QListViewItem(ListView_CHATUSERLIST,QString::fromUtf8(msg->m_sSrcNick.Data()));
 			TabWidget_CHAT->setTabLabel( TabWidget_CHAT->page(0), tr("Chat List")+" ("+QString().setNum(ListView_CHATUSERLIST->childCount())+")" );
 		}
 
@@ -1253,7 +1353,31 @@ void DCClient::DC_PrivateChat( CMessagePrivateChat * msg, bool bLocal )
 		{
 			bShowChat = FALSE;
 		}
-
+		
+		// check for nicks to not open chat windows
+		DCConfigHubProfile pConfigHubProfile;
+		QString nickregexp = "";
+		
+		if ( g_pConfig->GetBookmarkHubProfile( GetHubName(), GetHost(), &pConfigHubProfile ) == TRUE )
+		{
+			nickregexp = pConfigHubProfile.m_sSuppressedNicks.Data();
+		}
+		
+		if (nickregexp == "")
+		{
+			nickregexp = g_pConfig->GetSuppressedNicks();
+		}
+		
+		if (nickregexp != "")
+		{
+			QRegExp rx(nickregexp);
+			rx.setCaseSensitive(FALSE);
+			if (rx.search(msg->m_sSrcNick.Data()) != -1)
+			{
+				bShowChat = FALSE;
+			}
+		}
+		
 		if ( m_bUseTabWidget )
 		{
 			TabWidget_CHAT->insertTab( chat, msg->m_sSrcNick.Data() );
@@ -1331,6 +1455,11 @@ void DCClient::DC_PrivateChat( CMessagePrivateChat * msg, bool bLocal )
 	
 	// send chat event
 	g_pConnectionManager->HubEvent(this);
+	
+	if (g_pConfig->GetAutoResponderEnabledForPM() == TRUE)
+	{
+		this->doAutoResponse( msg->m_sSrcNick.Data(), msg->m_sMessage.Data(), chat );
+	}	
 }
 
 /** handling if user join the hub */
@@ -1344,7 +1473,17 @@ void DCClient::addUser( QString nick )
 	// show joins
 	if ( g_pConfig->GetChatShowJoinsAndParts() == TRUE )
 	{
-		m_pHubChat->AddStatus( tr("Joins: ") + CString(nick.ascii()) );
+		if ( g_pConfig->GetChatShowJoinsAndPartsOnlyFav() == TRUE )
+		{
+			if ( g_pUsersList->isNickInList( nick.ascii() ) == TRUE )
+			{
+				m_pHubChat->AddStatus( tr("Joins: ") + CString(nick.ascii()) );
+			}
+		}
+		else
+		{
+			m_pHubChat->AddStatus( tr("Joins: ") + CString(nick.ascii()) );
+		}
 	}
 
 	// show rejoin on private chat
@@ -1429,7 +1568,27 @@ void DCClient::slotDoubleClickedUserList( QListViewItem * item )
 		return;
 	}
 
-	DC_PrivateChat( item->text(0), "", "", TRUE );
+	if ( g_pConfig->GetDoubleClickAction() == tr("Downloads filelist") )
+	{
+		g_pTransferView->DLM_QueueAdd(
+			item->text(0).ascii(),
+			GetHubName(),
+			GetIP() + ":" + QString().setNum(GetPort()).ascii(),
+			DC_USER_FILELIST_HE3,
+			DC_USER_FILELIST_HE3,
+			"",
+			"",
+			eltBUFFER,
+			0,
+			0,
+			0,
+			""
+		);
+	}
+	else
+	{
+		DC_PrivateChat( item->text(0), "", "", TRUE );
+	}
 }
 
 /** */
@@ -1515,9 +1674,12 @@ void DCClient::slotRightButtonClickedUserList( QListViewItem *, const QPoint &,
 	QPopupMenu *m,*mslot;
 	QString username;
 	CList<CString> selitems;
+	CList<DC_UserMenuCommand> usercommands;
+	DC_UserMenuCommand * umc = 0; 
 	QListViewItem * curitem;
 	CString * ps;
 	int numSelected;
+	QString usercommand, origUserCommand;
 
 	numSelected = selectedItems(ListView_USERLIST,&selitems);
 
@@ -1562,7 +1724,20 @@ void DCClient::slotRightButtonClickedUserList( QListViewItem *, const QPoint &,
 		DCMenuHandler::InsertMenu( m, emiKICK, UserList()->IsAdmin(GetNick()) && (numSelected > 0) );
 		DCMenuHandler::InsertMenu( m, emiFORCE_MOVE, UserList()->IsAdmin(GetNick()) && (numSelected > 0) );
 	}
+	
+	DCMenuHandler::InsertMenu( m, emiSEPARATOR );
 
+	g_pConfig->GetUserMenuCommands(&usercommands);
+	
+	while( (umc=usercommands.Next(umc)) != 0 )
+	{
+		if ( (umc->m_nContext & 2) == 2 )
+		{
+			umc->m_nID = m->insertItem(umc->m_sName.Data(), -1);
+			m->setItemEnabled( umc->m_nID, numSelected > 0 );
+		}
+	}
+	
 	id = m->exec(QCursor::pos());
 
 	delete m;
@@ -1571,7 +1746,7 @@ void DCClient::slotRightButtonClickedUserList( QListViewItem *, const QPoint &,
 	{
 		for(ps=0;(ps=selitems.Next(ps)) != 0;)
 		{
-			DC_PrivateChat( ps->Data(), "", "", TRUE );
+			DC_PrivateChat( QString::fromUtf8(ps->Data()), "", "", TRUE );
 		}
 	}
 	else if ( id == emiADD_FRIEND )
@@ -1700,6 +1875,27 @@ void DCClient::slotRightButtonClickedUserList( QListViewItem *, const QPoint &,
 	{
 		RequestNickList();
 	}
+	else
+	{
+		umc = 0;
+		while( (umc=usercommands.Next(umc)) != 0 )
+		{
+			if ( id == umc->m_nID )
+			{
+				usercommand = umc->m_sCommand.Data();
+				
+				origUserCommand = usercommand;
+				
+				for(ps=0;(ps=selitems.Next(ps)) != 0;)
+				{
+					usercommand = replaceCommandTags( origUserCommand, ps->Data() );
+					
+					m_pHubChat->AddStatus(usercommand.ascii());
+					SendString(usercommand.ascii());
+				}
+			}
+		}
+	}
 }
 
 void DCClient::slotSaveColumnWidth(int section, int oldSize, int newSize)
@@ -1856,3 +2052,174 @@ QString DCClient::findNick( QString part, uint which )
 
 	return part;
 }
+
+void DCClient::slotTabCornerCloseWidgetClicked()
+{
+	CloseChat( (DCChat*) TabWidget_CHAT->currentPage() );
+}
+
+/** Replaces the %[tag] s in the command with the proper values */
+QString DCClient::replaceCommandTags(QString command, QString remotenick)
+{
+	QString usercommand = command;
+	
+	usercommand.replace( "%[mynick]", GetNick().Data() );
+	usercommand.replace( "%[myNI]", GetNick().Data() );
+	
+	QString comtag = GetComment().Data();
+	QString comment = comtag.mid(0, comtag.find('<'));
+	QString tag = comtag.mid(comtag.find('<'));
+	
+	usercommand.replace( "%[mytag]", tag );
+	usercommand.replace( "%[myTAG]", tag );
+	
+	usercommand.replace( "%[mydescription]", comment );
+	usercommand.replace( "%[myDE]", comment );
+	
+	usercommand.replace( "%[myemail]", GetEMail().Data() );
+	usercommand.replace( "%[myEM]", GetEMail().Data() );
+	
+	usercommand.replace( "%[myshare]", GetShareSize().Data() );
+	usercommand.replace( "%[mySS]", GetShareSize().Data() );
+	
+	usercommand.replace( "%[myshareshort]", CUtils::GetSizeString(GetShareSize().asULL(), euAUTO).Data() );
+	usercommand.replace( "%[mySSshort]", CUtils::GetSizeString(GetShareSize().asULL(), euAUTO).Data() );
+	
+	usercommand.replace( "%[myip]", g_pConfig->GetTCPHostString( FALSE ).Data() );
+	usercommand.replace( "%[myI4]", g_pConfig->GetTCPHostString( FALSE ).Data() );
+	
+	usercommand.replace( "%[nick]", remotenick );
+	usercommand.replace( "%[userNI]", remotenick );
+	
+	QListViewItem * item = ListView_USERLIST->findItem(remotenick ,0);
+	
+	if ( item == 0 )
+	{
+		//printf("Warning: nick \"%s\" not found in userlist.\n", remotenick.ascii());
+	}
+	else
+	{
+		usercommand.replace( "%[userTAG]", item->text(2) );
+		usercommand.replace( "%[tag]", item->text(2) );
+		
+		usercommand.replace( "%[userDE]", item->text(1) );
+		usercommand.replace( "%[description]", item->text(1) );
+		
+		usercommand.replace( "%[userSSshort]", item->text(5) );
+		usercommand.replace( "%[shareshort]", item->text(5) );
+		
+		usercommand.replace( "%[userEM]", item->text(4) );
+		usercommand.replace( "%[email]", item->text(4) );
+		
+		usercommand.replace( "%[userI4]", item->text(6) );
+		usercommand.replace( "%[ip]", item->text(6) );
+	}
+	
+	while ( usercommand.find("%[line:") != -1 )
+	{
+		bool ok;
+		int loc = usercommand.find("%[line:");
+		int end = usercommand.find("]", loc);
+		
+		// 7 is length of "%[line:"
+		QString reason = usercommand.mid(loc + 7, end - (loc + 7));
+		
+		QString text = QInputDialog::getText(
+		tr("User command for ") + remotenick,
+		reason,
+		QLineEdit::Normal,
+		QString::null,
+		&ok,
+		this
+		);
+	
+		//if ( ok && !text.isEmpty() )
+		//{
+			usercommand.replace( "%[line:" + reason + "]", text );
+		//}
+	}
+					
+	{
+		struct tm * t;
+		time_t ti;
+		char timed[2048];
+						
+		ti = time(0);
+		t  = localtime(&ti);
+		strftime(timed, sizeof(timed), usercommand.ascii(), t);
+		usercommand = timed;
+	}
+	
+	return usercommand;
+}
+
+void DCClient::doAutoResponse( QString nick, QString message, DCChat * chatobject )
+{
+	long delay = g_pConfig->GetAutoResponseDelay();
+	long now = QDateTime::currentDateTime().toTime_t();
+	
+	if ( (delay > 0) &&
+	     (nick == lastAutoNick) &&
+	     (now < delay + lastAutoResponseTime)
+	   )
+	{
+		// avoid spamming
+		//printf("Auto response spam protection!\n");
+		return;
+	}
+	
+	// null pointer check
+	if (chatobject == 0)
+	{
+		printf("Error: DCClient::doAutoResponse called with NULL chatobject!\n");
+		return;
+	}
+	
+	// do not respond to ourself
+	if (nick == this->GetNick())
+	{
+		//printf("Not auto-responding to ourself.\n");
+		return;
+	}
+	
+	// check for ignored nicks
+	CString ignores = g_pConfig->GetAutoResponseIgnores();
+	if (ignores != "")
+	{
+		QRegExp ignore_re;
+		ignore_re.setPattern(ignores.Data());
+		ignore_re.setCaseSensitive(FALSE);
+		if (ignore_re.search(nick) != -1)
+		{
+			//printf("Not auto responding to %s\n", nick.ascii());
+			return;
+		}
+	}
+	
+	CList<DC_AutoResponseObject> arlist;
+	DC_AutoResponseObject * aro = 0;
+	
+	g_pConfig->GetAutoResponses(&arlist);
+	
+	QRegExp re;
+	
+	while ((aro=arlist.Next(aro)) != 0)
+	{
+		re.setPattern(aro->m_sTrigger.Data());
+		re.setCaseSensitive(aro->m_bCaseSensitive);
+		if (re.search(message) != -1) // pattern matches
+		{
+			QString chat = this->replaceCommandTags( aro->m_sResponse.Data(), nick );
+			//this->SendChat( this->GetNick(), chat.ascii() );
+			chatobject->TextEdit_CHATINPUT->setText(chat);
+			chatobject->SendMessage( "" );
+			
+			// update last nick and time for anti-spam
+			lastAutoNick = nick;
+			lastAutoResponseTime = QDateTime::currentDateTime().toTime_t();
+			
+			// stop searching list of triggers
+			break;
+		}
+	}
+}
