@@ -65,6 +65,7 @@
 #define XML_TV_WIDTH			"tvwidth"
 #define XML_TV_HEIGHT			"tvheight"
 #define XML_TV_DOCKED			"tvdocked"
+#define XML_TABBAR			"tabbar"
 #define XML_TOOLBAR			"toolbar"
 #define XML_STATUSBAR			"statusbar"
 #define XML_EXTERNAL_SOUND_PLAYER	"externalsoundplayer"
@@ -95,8 +96,12 @@
 #define XML_QUERYONFILEDELETE		"queryonfiledelete"
 #define XML_SHOWCHATSENDBUTTON		"showchatsendbutton"
 #define XML_CHATMAXPARAGRAPH		"chatmaxparagraph"
-#define XML_CHATINPUTCODEC		"chatinputcodec"
-#define XML_CHATOUTPUTCODEC		"chatoutputcodec"
+#define XML_DEFAULTFBREMOTEENCODING	"defaultfilelistremoteencoding"
+#define XML_VALKNUT_DATA_PATH		"valknutdatapath"
+
+#define XML_APP_ICON_THEME		"appicontheme"
+#define XML_EMOTICON_THEME		"emoticontheme"
+#define XML_USER_ICON_THEME		"userlisticontheme"
 
 #define XML_TV_CHUNKPERCENT		"tvchunkpercent"
 #define XML_TV_FILEPERCENT		"tvfilepercent"
@@ -155,6 +160,7 @@
 #define XML_CC_PUBLICPRIVATECHATTEXT		"CC_PUBLICPRIVATECHATTEXT"
 #define XML_CC_CHATSAY				"CC_CHATSAY"
 #define XML_CC_PUBLICCHATMENICK			"CC_PUBLICCHATMENICK"
+#define XML_CC_OPNICK				"CC_OPNICK"
 
 #define XML_CLIENT_COLUMN_COMMENT		"clientcolumncomment"
 #define XML_CLIENT_COLUMN_TAG			"clientcolumntag"
@@ -168,12 +174,21 @@
 #define XML_AUTO_AWAY_MODE		"autoawaymode"
 #define XML_AUTO_AWAY_TIME		"autoawaytime"
 #define XML_CHAT_SHOW_JOINS_AND_PARTS	"chatshowjoinsandparts"
+#define XML_CHAT_SHOW_JOINS_AND_PARTS_ONLY_FAV	"chatshowjoinsandpartsonlyfav"
+#define XML_CHAT_SUPPRESSED_NICKS	"suppressednicks"
+#define XML_DOUBLE_CLICK_ACTION		"doubleclickaction"
 
 #define XML_FRIEND			"friend"
 #define XML_IMAGE			"image"
 #define XML_SEND_IMAGE			"sendimage"
 #define XML_AUTO_SEC			"autosec"
 #define XML_USER_PHOTO			"userphoto"
+#define XML_PERM_SLOT			"permslot"
+#define XML_IGNORE			"ignore"
+
+#define XML_USERCHATCOMMAND		"userchatcommand"
+#define XML_USERCHATCOMMAND_NAME	"command_name"
+#define XML_USERCHATCOMMAND_ACT		"command_action"
 
 #define XML_UNIT_AUTO	"auto"
 #define XML_UNIT_BYTE	"byte"
@@ -181,6 +196,25 @@
 #define XML_UNIT_MBYTE	"mbyte"
 #define XML_UNIT_GBYTE	"gbyte"
 
+#define XML_USERMENUCOMMAND		"usermenucommand"
+#define XML_USERMENUCOMMAND_NAME	"command_name"
+#define XML_USERMENUCOMMAND_ACT		"command_action"
+#define XML_USERMENUCOMMAND_CONTEXT	"command_context"
+
+#define XML_AUTORESPONSE_ENABLED	"autoresponse_enabled"
+#define XML_AUTORESPONSE_ENABLED_PM	"autoresponse_enabled_pm"
+#define XML_AUTORESPONSE		"autoresponse"
+#define XML_AUTORESPONSE_TRIGGER	"trigger"
+#define XML_AUTORESPONSE_RESPONSE	"response"
+#define XML_AUTORESPONSE_CASE_SENS	"case_sensitive"
+#define XML_AUTORESPONSE_DELAY		"autoresponse_delay"
+#define XML_AUTORESPONSE_IGNORES	"autoresponse_ignores"
+
+#define XML_FILELIST_NAMING_SCHEME	"filelist_naming_scheme"
+
+#define XML_FOLDERS_RIGHT_PANE		"folders_in_right_pane"
+#define XML_SIZES_IN_LEFT_PANE		"sizes_in_left_pane"
+
 DCConfig * g_pConfig = 0;
 
 DCConfig::DCConfig( CString configpath ) : CConfig(configpath)
@@ -196,6 +230,7 @@ DCConfig::DCConfig( CString configpath ) : CConfig(configpath)
 	iTransferViewHeight     = 100;
 	iTransferViewWidth      = 640;
 	bTransferViewDocked     = TRUE;
+	bTabBar			= TRUE;
 	bToolBar		= TRUE;
 	bStatusBar		= TRUE;
 
@@ -258,6 +293,7 @@ DCConfig::DCConfig( CString configpath ) : CConfig(configpath)
 	m_nAutoAwayTime         = 300;
 
 	m_bChatShowJoinsAndParts = FALSE;
+	m_bChatShowJoinsAndPartsOnlyFav = FALSE;
 	bSendHidePrivateChatToPublicChat = TRUE;
 
 	m_UserIconCache = QPixmap("");
@@ -286,8 +322,7 @@ DCConfig::DCConfig( CString configpath ) : CConfig(configpath)
 	bOpenClientWindowsMinimized = FALSE;
 	m_EmoticonSettingList = 0;
 
-	m_sChatInputCodec  = "";
-	m_sChatOutputCodec = "";
+	m_sDefaultFBRemoteEncoding = "DEFAULT";
 
 	m_bLogChatOption_1 = FALSE;
 	m_bLogChatOption_2 = FALSE;
@@ -315,6 +350,28 @@ DCConfig::DCConfig( CString configpath ) : CConfig(configpath)
 	m_sChatColor_16 = "#000000";
 	m_sChatColor_17 = "#0000ff";
 	m_sChatColor_18 = "#ff0c52";
+	m_sChatColor_19 = "#000000";
+
+	m_sApplicationIconTheme = "default";
+	m_sEmoticonTheme        = "default";
+	m_sUserListIconTheme    = "default";
+	
+	m_sSuppressedNicks	= "Hub";
+	
+	// auto responder defaults to disabled
+	m_bAutoResponderEnabled = FALSE;
+	m_bAutoResponderEnabledForPM = FALSE;
+	m_nAutoResponseDelay = 5;
+	m_sAutoResponseIgnores = "";
+	
+	/* 0 = 0.3.8 integers
+	 * 1 = 0.3.9 base64 encoded
+	 * 2 = 0.3.10 human readable
+	 */
+	m_nFilelistNamingSchemeVersion = 0;
+	
+	m_bFoldersInRightPane = FALSE;
+	m_bFolderSizesInLeftPane = TRUE;
 
 #if defined(Q_OS_MACX) 
 	CFURLRef pluginRef = CFBundleCopyBundleURL(CFBundleGetMainBundle()); 
@@ -380,6 +437,8 @@ void DCConfig::ParseDCGuiConfig( xmlNodePtr node )
 				        bEnableEmoticons = xml.getBoolChild(n2);
 				else if ( xml_name == XML_USER_PHOTO )
 				        m_sUserPhotoFileName = xml.content(n2);
+				else if ( xml_name == XML_TABBAR )
+				        bTabBar = xml.getBoolChild(n2);
 				else if ( xml_name == XML_TOOLBAR )
 				        bToolBar = xml.getBoolChild(n2);
 				else if ( xml_name == XML_STATUSBAR )
@@ -500,10 +559,16 @@ void DCConfig::ParseDCGuiConfig( xmlNodePtr node )
 					m_nAutoAwayTime = xml.content(n2).asINT();
 				else if ( xml_name == XML_CHAT_SHOW_JOINS_AND_PARTS )
 					m_bChatShowJoinsAndParts = xml.getBoolChild(n2);
-				else if ( xml_name == XML_CHATINPUTCODEC )
-					m_sChatInputCodec = xml.content(n2);
-				else if ( xml_name == XML_CHATOUTPUTCODEC )
-					m_sChatOutputCodec = xml.content(n2);
+				else if ( xml_name == XML_CHAT_SHOW_JOINS_AND_PARTS_ONLY_FAV )
+					m_bChatShowJoinsAndPartsOnlyFav = xml.getBoolChild(n2);
+				else if ( xml_name == XML_CHAT_SUPPRESSED_NICKS )
+					m_sSuppressedNicks = xml.content(n2).Data();
+				else if ( xml_name == XML_DOUBLE_CLICK_ACTION )
+					m_sDoubleClickAction = xml.content(n2).Data();
+				else if ( xml_name == XML_DEFAULTFBREMOTEENCODING )
+					m_sDefaultFBRemoteEncoding = xml.content(n2).Data();
+				else if ( xml_name == XML_VALKNUT_DATA_PATH )
+					m_sValknutDataPath = xml.content(n2).Data();
 				else if ( xml_name == "MAP" )
 				{
 					StringMap * map = new StringMap();
@@ -583,14 +648,16 @@ void DCConfig::ParseDCGuiConfig( xmlNodePtr node )
 					SetChatColor( eccCHATSAY, xml.content(n2) );
 				else if ( xml_name == XML_CC_PUBLICCHATMENICK )
 					SetChatColor( eccPUBLICCHATMENICK, xml.content(n2) );
+				else if ( xml_name == XML_CC_OPNICK )
+					SetChatColor( eccOPNICK, xml.content(n2) );
 				else if ( xml_name == XML_CLIENT_COLUMN_COMMENT )
 					SetClientColumn( eclcCOMMENT, xml.getBoolChild(n2) );
 				else if ( xml_name == XML_CLIENT_COLUMN_TAG )
 					SetClientColumn( eclcTAG, xml.getBoolChild(n2) );
 				else if ( xml_name == XML_CLIENT_COLUMN_SPEED )
-					SetClientColumn( eclcEMAIL, xml.getBoolChild(n2) );
-				else if ( xml_name == XML_CLIENT_COLUMN_EMAIL )
 					SetClientColumn( eclcSPEED, xml.getBoolChild(n2) );
+				else if ( xml_name == XML_CLIENT_COLUMN_EMAIL )
+					SetClientColumn( eclcEMAIL, xml.getBoolChild(n2) );
 				else if ( xml_name == XML_CLIENT_COLUMN_SHARE )
 					SetClientColumn( eclcSHARE, xml.getBoolChild(n2) );
 				else if ( xml_name == XML_CLIENT_COLUMN_IP )
@@ -599,6 +666,85 @@ void DCConfig::ParseDCGuiConfig( xmlNodePtr node )
 					SetClientColumn( eclcLOCKPK, xml.getBoolChild(n2) );
 				else if ( xml_name == XML_CLIENT_COLUMN_SUPPORTS )
 					SetClientColumn( eclcSUPPORTS, xml.getBoolChild(n2) );
+				else if ( xml_name == XML_USERMENUCOMMAND )
+				{
+					DC_UserMenuCommand * umc = new DC_UserMenuCommand();
+					
+					for( n3 = n2->xmlChildrenNode; n3 != 0; n3 = n3->next )
+					{
+						if ( xml.name(n3) == XML_USERMENUCOMMAND_NAME )
+							umc->m_sName = xml.content(n3);
+						else if ( xml.name(n3) == XML_USERMENUCOMMAND_ACT )
+							umc->m_sCommand = xml.content(n3);
+						else if ( xml.name(n3) == XML_USERMENUCOMMAND_CONTEXT )
+							umc->m_nContext = xml.content(n3).asINT();
+					}
+					
+					if ( umc->m_sName.IsEmpty() || umc->m_sCommand.IsEmpty() )
+						delete umc;
+					else
+						m_lUserMenuCommands.Add( umc );
+				}
+				else if ( xml_name == XML_USERCHATCOMMAND )
+				{
+					DC_UserChatCommand * ucc = new DC_UserChatCommand();
+					
+					for ( n3 = n2->xmlChildrenNode; n3 != 0; n3 = n3->next )
+					{
+						if ( xml.name(n3) == XML_USERCHATCOMMAND_NAME )
+							ucc->m_sName = xml.content(n3);
+						else if ( xml.name(n3) == XML_USERCHATCOMMAND_ACT )
+							ucc->m_sCommand = xml.content(n3);
+					}
+					
+					if ( ucc->m_sName.IsEmpty() || ucc->m_sCommand.IsEmpty() )
+						delete ucc;
+					else
+						m_lUserChatCommands.Add( ucc );
+				}
+				else if ( xml_name == XML_AUTORESPONSE_ENABLED )
+					SetAutoResponderEnabled( xml.getBoolChild(n2) );
+				else if ( xml_name == XML_AUTORESPONSE_ENABLED_PM )
+					SetAutoResponderEnabledForPM( xml.getBoolChild(n2) );
+				else if ( xml_name == XML_AUTORESPONSE )
+				{
+					DC_AutoResponseObject * aro = new DC_AutoResponseObject();
+					
+					for ( n3 = n2->xmlChildrenNode; n3 != 0; n3 = n3->next )
+					{
+						if ( xml.name(n3) == XML_AUTORESPONSE_TRIGGER )
+							aro->m_sTrigger = xml.content(n3);
+						else if ( xml.name(n3) == XML_AUTORESPONSE_RESPONSE )
+							aro->m_sResponse = xml.content(n3);
+						else if ( xml.name(n3) == XML_AUTORESPONSE_CASE_SENS )
+							aro->m_bCaseSensitive = xml.getBoolChild(n3);
+					}
+					
+					if ( aro->m_sTrigger.IsEmpty() || aro->m_sResponse.IsEmpty() )
+					{
+						delete aro;
+					}
+					else
+					{
+						m_lAutoResponses.Add( aro );
+					}
+				}
+				else if ( xml_name == XML_AUTORESPONSE_DELAY )
+					SetAutoResponseDelay( xml.content(n2).asLONG() );
+				else if ( xml_name == XML_AUTORESPONSE_IGNORES )
+					m_sAutoResponseIgnores = xml.content(n2);
+				else if ( xml_name == XML_APP_ICON_THEME )
+					m_sApplicationIconTheme = xml.content(n2).Data();
+				else if ( xml_name == XML_EMOTICON_THEME )
+					m_sEmoticonTheme = xml.content(n2).Data();
+				else if ( xml_name == XML_USER_ICON_THEME )
+					m_sUserListIconTheme = xml.content(n2).Data();
+				else if ( xml_name == XML_FILELIST_NAMING_SCHEME )
+					m_nFilelistNamingSchemeVersion = xml.content(n2).asINT();
+				else if ( xml_name == XML_FOLDERS_RIGHT_PANE )
+					SetFoldersInRightPane( xml.getBoolChild(n2) );
+				else if ( xml_name == XML_SIZES_IN_LEFT_PANE )
+					SetFolderSizesInLeftPane( xml.getBoolChild(n2) );
 			}
 		}
 	}
@@ -627,9 +773,6 @@ int DCConfig::LoadDCGui()
 	xmlNodePtr node;
 	CXml xml;
 
-	// get usericons
-	m_UserIcons = g_pIconLoader->GetPixmap(eiUSERICONS);
-
 	s = sConfigPath + DCGUI_CONFIG;
 
 	if ( xml.ParseFile(s) == TRUE )
@@ -647,6 +790,9 @@ int DCConfig::LoadDCGui()
 		err = -1;
 	}
 
+	// get usericons - cannot be done here since icons not loaded yet
+	m_UserIcons = g_pIconLoader->GetPixmap(eiUSERICONS);
+
 	InitMimeSourceFactory();
 
 	return err;
@@ -671,6 +817,7 @@ int DCConfig::SaveDCGui()
 	xml.xmlNewBoolChild( node, 0, (const xmlChar*)XML_OPENCLIENTWINDOWSMINIMIZED, bOpenClientWindowsMinimized );
 	xml.xmlNewBoolChild( node, 0, (const xmlChar*)XML_OPENPRIVATECHATWIN, bOpenPrivateChatWindow );
 	xml.xmlNewBoolChild( node, 0, (const xmlChar*)XML_EMOTICONS, bEnableEmoticons );
+	xml.xmlNewBoolChild( node, 0, (const xmlChar*)XML_TABBAR, bTabBar );
 	xml.xmlNewBoolChild( node, 0, (const xmlChar*)XML_TOOLBAR, bToolBar );
 	xml.xmlNewBoolChild( node, 0, (const xmlChar*)XML_STATUSBAR, bStatusBar );
 	xml.xmlNewBoolChild( node, 0, (const xmlChar*)XML_USERLISTRIGHTALIGNMENT, bUserListRightAlignment );
@@ -714,6 +861,8 @@ int DCConfig::SaveDCGui()
 	xml.xmlNewBoolChild( node, 0, (const xmlChar*)XML_ENABLETRAY, m_bEnableTray );
 	xml.xmlNewBoolChild( node, 0, (const xmlChar*)XML_QUERYONEXIT, bQueryOnExit );
 	xml.xmlNewBoolChild( node, 0, (const xmlChar*)XML_QUERYONFILEDELETE, bQueryOnFileDelete );
+	xml.xmlNewBoolChild( node, 0, (const xmlChar*)XML_FOLDERS_RIGHT_PANE, m_bFoldersInRightPane );
+	xml.xmlNewBoolChild( node, 0, (const xmlChar*)XML_SIZES_IN_LEFT_PANE, m_bFolderSizesInLeftPane );
 
 	// sound settings
 	xml.xmlNewStringChild( node, 0, (const xmlChar*)XML_SOUND_FILE_CONNECT, sSoundFileConnect );
@@ -751,9 +900,11 @@ int DCConfig::SaveDCGui()
 		xmlNewChild( node, 0, (const xmlChar*)"NICKNAMEFILTER", (const xmlChar*) xml.ToUTF8(*ps).Data() );
 	}
 
-	// chat text codec
-	xml.xmlNewStringChild( node, 0, (const xmlChar*)XML_CHATINPUTCODEC, m_sChatInputCodec );
-	xml.xmlNewStringChild( node, 0, (const xmlChar*)XML_CHATOUTPUTCODEC, m_sChatOutputCodec );
+	// default filebrowser remote encoding
+	xml.xmlNewStringChild( node, 0, (const xmlChar*)XML_DEFAULTFBREMOTEENCODING, m_sDefaultFBRemoteEncoding.ascii() );
+	
+	// valknut data path
+	xml.xmlNewStringChild( node, 0, (const xmlChar*)XML_VALKNUT_DATA_PATH, m_sValknutDataPath.ascii() );
 
 	xml.xmlNewStringChild( node, 0, (const xmlChar*)XML_CC_CHATTIMESTAMP, GetChatColor(eccCHATTIMESTAMP) );
 	xml.xmlNewStringChild( node, 0, (const xmlChar*)XML_CC_CHATLOCALNICK, GetChatColor(eccCHATLOCALNICK) );
@@ -773,6 +924,7 @@ int DCConfig::SaveDCGui()
 	xml.xmlNewStringChild( node, 0, (const xmlChar*)XML_CC_PUBLICPRIVATECHATTEXT, GetChatColor(eccPUBLICPRIVATECHATTEXT) );
 	xml.xmlNewStringChild( node, 0, (const xmlChar*)XML_CC_CHATSAY, GetChatColor(eccCHATSAY) );
 	xml.xmlNewStringChild( node, 0, (const xmlChar*)XML_CC_PUBLICCHATMENICK, GetChatColor(eccPUBLICCHATMENICK) );
+	xml.xmlNewStringChild( node, 0, (const xmlChar*)XML_CC_OPNICK, GetChatColor(eccOPNICK) );
 
 	// transfer view settings
 	xml.xmlNewBoolChild( node, 0, (const xmlChar*)XML_TV_CHUNKPERCENT, bTVChunkPercent );
@@ -795,17 +947,61 @@ int DCConfig::SaveDCGui()
 	xml.xmlNewBoolChild( node, 0, (const xmlChar*)XML_AUTO_AWAY_MODE, m_bAutoAwayMode );
 	xml.xmlNewStringChild( node, 0, (const xmlChar*)XML_AUTO_AWAY_TIME, CString().setNum(m_nAutoAwayTime) );
 	xml.xmlNewBoolChild( node, 0, (const xmlChar*)XML_CHAT_SHOW_JOINS_AND_PARTS, m_bChatShowJoinsAndParts );
+	xml.xmlNewBoolChild( node, 0, (const xmlChar*)XML_CHAT_SHOW_JOINS_AND_PARTS_ONLY_FAV, m_bChatShowJoinsAndPartsOnlyFav );
+	xml.xmlNewStringChild( node, 0, (const xmlChar*)XML_CHAT_SUPPRESSED_NICKS, m_sSuppressedNicks.ascii() );
+	xml.xmlNewStringChild( node, 0, (const xmlChar*)XML_DOUBLE_CLICK_ACTION, m_sDoubleClickAction.ascii() );
 
 	// save client column settings
 	xml.xmlNewBoolChild( node, 0, (const xmlChar*)XML_CLIENT_COLUMN_COMMENT, GetClientColumn(eclcCOMMENT) );
 	xml.xmlNewBoolChild( node, 0, (const xmlChar*)XML_CLIENT_COLUMN_TAG, GetClientColumn(eclcTAG) );
-	xml.xmlNewBoolChild( node, 0, (const xmlChar*)XML_CLIENT_COLUMN_SPEED, GetClientColumn(eclcEMAIL) );
-	xml.xmlNewBoolChild( node, 0, (const xmlChar*)XML_CLIENT_COLUMN_EMAIL, GetClientColumn(eclcSPEED) );
+	xml.xmlNewBoolChild( node, 0, (const xmlChar*)XML_CLIENT_COLUMN_EMAIL, GetClientColumn(eclcEMAIL) );
+	xml.xmlNewBoolChild( node, 0, (const xmlChar*)XML_CLIENT_COLUMN_SPEED, GetClientColumn(eclcSPEED) );
 	xml.xmlNewBoolChild( node, 0, (const xmlChar*)XML_CLIENT_COLUMN_SHARE, GetClientColumn(eclcSHARE) );
 	xml.xmlNewBoolChild( node, 0, (const xmlChar*)XML_CLIENT_COLUMN_IP, GetClientColumn(eclcIP) );
 	xml.xmlNewBoolChild( node, 0, (const xmlChar*)XML_CLIENT_COLUMN_LOCKPK, GetClientColumn(eclcLOCKPK) );
 	xml.xmlNewBoolChild( node, 0, (const xmlChar*)XML_CLIENT_COLUMN_SUPPORTS, GetClientColumn(eclcSUPPORTS) );
 	
+	// user menu commands
+	DC_UserMenuCommand * umc = 0;
+	while( (umc=m_lUserMenuCommands.Next(umc)) != 0 )
+	{
+		node1 = xmlNewChild( node, 0, (const xmlChar*)XML_USERMENUCOMMAND, 0);
+		xml.xmlNewStringChild( node1, 0, (const xmlChar*)XML_USERMENUCOMMAND_NAME, umc->m_sName );
+		xml.xmlNewStringChild( node1, 0, (const xmlChar*)XML_USERMENUCOMMAND_ACT, umc->m_sCommand );
+		xml.xmlNewStringChild( node1, 0, (const xmlChar*)XML_USERMENUCOMMAND_CONTEXT, CString().setNum(umc->m_nContext) );
+	}
+	
+	// user chat commands
+	DC_UserChatCommand * ucc = 0;
+	while ( (ucc=m_lUserChatCommands.Next(ucc)) != 0 )
+	{
+		node1 = xmlNewChild( node, 0, (const xmlChar*)XML_USERCHATCOMMAND, 0);
+		xml.xmlNewStringChild( node1, 0, (const xmlChar*)XML_USERCHATCOMMAND_NAME, ucc->m_sName );
+		xml.xmlNewStringChild( node1, 0, (const xmlChar*)XML_USERCHATCOMMAND_ACT, ucc->m_sCommand );
+	}
+	
+	// icon themes
+	xml.xmlNewStringChild( node, 0, (const xmlChar*)XML_APP_ICON_THEME, GetApplicationIconTheme().ascii() );
+	xml.xmlNewStringChild( node, 0, (const xmlChar*)XML_EMOTICON_THEME, GetEmoticonTheme().ascii() );
+	xml.xmlNewStringChild( node, 0, (const xmlChar*)XML_USER_ICON_THEME, GetUserListIconTheme().ascii() );
+	
+	// filelist naming scheme version
+	xml.xmlNewStringChild( node, 0, (const xmlChar*)XML_FILELIST_NAMING_SCHEME, CString().setNum(GetFilelistNamingSchemeVersion()) );
+	
+	// auto responses
+	xml.xmlNewBoolChild( node, 0, (const xmlChar*)XML_AUTORESPONSE_ENABLED, GetAutoResponderEnabled() );
+	xml.xmlNewBoolChild( node, 0, (const xmlChar*)XML_AUTORESPONSE_ENABLED_PM, GetAutoResponderEnabledForPM() );
+	DC_AutoResponseObject * aro = 0;
+	while ( (aro=m_lAutoResponses.Next(aro)) != 0 )
+	{
+		node1 = xmlNewChild( node, 0, (const xmlChar*)XML_AUTORESPONSE, 0);
+		xml.xmlNewStringChild( node1, 0, (const xmlChar*)XML_AUTORESPONSE_TRIGGER, aro->m_sTrigger );
+		xml.xmlNewStringChild( node1, 0, (const xmlChar*)XML_AUTORESPONSE_RESPONSE, aro->m_sResponse );
+		xml.xmlNewBoolChild( node1, 0, (const xmlChar*)XML_AUTORESPONSE_CASE_SENS, aro->m_bCaseSensitive );
+	}
+	xml.xmlNewStringChild( node, 0, (const xmlChar*)XML_AUTORESPONSE_DELAY, CString().setNum(GetAutoResponseDelay()) );
+	xml.xmlNewStringChild( node, 0, (const xmlChar*)XML_AUTORESPONSE_IGNORES, GetAutoResponseIgnores() );
+	
 	// save map settings
 	SettingMap::Iterator it;
 	StringMap::Iterator it1;
@@ -825,7 +1021,7 @@ int DCConfig::SaveDCGui()
 	// save file
 	s = sConfigPath + DCGUI_CONFIG;
 
-	if ( xmlSaveFormatFile(s.Data(),doc,1) == -1 )
+	if ( SaveConfigXmlViaTemp(s,doc) == -1 )
 	{
 		err = -1;
 	}
@@ -918,7 +1114,7 @@ int DCConfig::SaveDCHubFilter( CStringList * list )
 	// save file
 	s = sConfigPath + DCGUI_HUBFILTER;
 
-	if ( xmlSaveFormatFile(s.Data(),doc,1) == -1 )
+	if ( SaveConfigXmlViaTemp(s,doc) == -1 )
 	{
 		err = -1;
 	}
@@ -964,9 +1160,13 @@ int DCConfig::LoadDCFriendList( CStringList * list )
 							else if ( xml_name == XML_IMAGE )
 								FriendObject->m_sImageFileName = xml.content(n2);
 							else if ( xml_name == XML_SEND_IMAGE )
-								FriendObject->m_bSendImage = xml.content(n2).asINT();
+								FriendObject->m_bSendImage = xml.getBoolChild(n2);
 							else if ( xml_name == XML_AUTO_SEC )
-								FriendObject->m_bAutoSec = xml.content(n2).asINT();
+								FriendObject->m_bAutoSec = xml.getBoolChild(n2);
+							else if ( xml_name == XML_PERM_SLOT )
+								FriendObject->m_bPermSlot = xml.getBoolChild(n2);
+							else if ( xml_name == XML_IGNORE )
+								FriendObject->m_bIgnore = xml.getBoolChild(n2);
 						}
 
 						if ( FriendObject->m_sName != "" )
@@ -1009,12 +1209,14 @@ int DCConfig::SaveDCFriendList( CStringList * list )
 		xml.xmlNewStringChild( node, 0, (const xmlChar*)XML_IMAGE, FriendObject->m_sImageFileName );
 		xml.xmlNewBoolChild( node, 0, (const xmlChar*)XML_SEND_IMAGE, FriendObject->m_bSendImage );
 		xml.xmlNewBoolChild( node, 0, (const xmlChar*)XML_AUTO_SEC, FriendObject->m_bAutoSec );
+		xml.xmlNewBoolChild( node, 0, (const xmlChar*)XML_PERM_SLOT, FriendObject->m_bPermSlot );
+		xml.xmlNewBoolChild( node, 0, (const xmlChar*)XML_IGNORE, FriendObject->m_bIgnore );
 	}
 
 	// save file
 	s = sConfigPath + DCGUI_FRIENDLIST;
 
-	if ( xmlSaveFormatFile(s.Data(),doc,1) == -1 )
+	if ( SaveConfigXmlViaTemp(s,doc) == -1 )
 	{
 		err = -1;
 	}
@@ -1037,27 +1239,40 @@ QPixmap * DCConfig::GetUserIcon( bool op, eClientMode clientmode, eUserAwayMode
 		case eus288KBPS:
 		case eus336KBPS:
 		case eus56KBPS:
+		case eus0005:
+		case eus001:
+		case eus002:
+		case eus005:
 			x = 0;
 			break;
 		case eusISDN:
+		case eus01:
 			x = 16;
 			break;
 		case eusSATELLITE:
 			x = 32;
 			break;
 		case eusCABLE:
+		case eus02:
 			x = 64;
 			break;
 		case eusDSL:
+		case eus05:
+		case eus1:
 			x = 80;
 			break;
 		case eusLANT1:
 		case eusLANT3:
+		case eus2:
+		case eus5:
+		case eus10:
+		case eus20:
+		case eus50:
+		case eus100:
 			x = 96;
 			break;
 		default:
-			x = -1;
-			y = -1;
+			x = 128;
 			break;
 	}
 
@@ -1584,6 +1799,8 @@ CString DCConfig::GetChatColor( eChatColor type ) const
 			return m_sChatColor_17;
 		case eccPUBLICCHATMENICK:
 			return m_sChatColor_18;
+		case eccOPNICK:
+			return m_sChatColor_19;
 		default:
 			break;
 	}
@@ -1650,6 +1867,9 @@ void DCConfig::SetChatColor( eChatColor type, CString s )
 		case eccPUBLICCHATMENICK:
 			m_sChatColor_18 = s;
 			break;
+		case eccOPNICK:
+			m_sChatColor_19 = s;
+			break;
 		default:
 			break;
 	}
@@ -1788,38 +2008,6 @@ bool DCConfig::CheckLogChatNickNameFilter( CString nick )
 	return FALSE;
 }
 
-/** */
-CString DCConfig::GetChatTextCodec( eChatTextCodec e )
-{
-	switch(e)
-	{
-		case ectcINPUT:
-			return m_sChatInputCodec;
-		case ectcOUTPUT:
-			return m_sChatOutputCodec;
-		default:
-			break;
-	}
-
-	return "";
-}
-
-/** */
-void DCConfig::SetChatTextCodec( eChatTextCodec e, CString s )
-{
-	switch(e)
-	{
-		case ectcINPUT:
-			m_sChatInputCodec = s;
-			break;
-		case ectcOUTPUT:
-			m_sChatOutputCodec = s;
-			break;
-		default:
-			break;
-	}
-}
-
 /** */
 bool DCConfig::GetClientColumn( eClientColumn e )
 {
@@ -1965,3 +2153,231 @@ void DCConfig::SetClientColumnWidth( eClientColumn e, int i )
 			break;
 	}
 }
+
+/** */
+void DCConfig::SetUserMenuCommands( CList<DC_UserMenuCommand> * list )
+{
+	DC_UserMenuCommand * umc = 0, *umc1;
+	
+	if (!list)
+	{
+		return;
+	}
+	
+	m_lUserMenuCommands.Clear();
+	
+	while( (umc=list->Next(umc)) != 0)
+	{
+		umc1 = new DC_UserMenuCommand();
+		umc1->m_sName    = umc->m_sName;
+		umc1->m_sCommand = umc->m_sCommand;
+		umc1->m_nContext = umc->m_nContext;
+		m_lUserMenuCommands.Add( umc1 );
+	}
+	
+	return;
+}
+
+/** */
+long DCConfig::GetUserMenuCommands( CList<DC_UserMenuCommand> * list)
+{
+	DC_UserMenuCommand * umc = 0, *umc1;
+	
+	if (!list)
+	{
+		return 0;
+	}
+	
+	list->Clear();
+	
+	while( (umc=m_lUserMenuCommands.Next(umc)) != 0 )
+	{
+		umc1 = new DC_UserMenuCommand();
+		umc1->m_sName    = umc->m_sName;
+		umc1->m_sCommand = umc->m_sCommand;
+		umc1->m_nContext = umc->m_nContext;
+		
+		list->Add( umc1 );
+	}
+	
+	return list->Count();
+}
+
+/** */
+void DCConfig::SetUserChatCommands( CList<DC_UserChatCommand> * list )
+{
+	DC_UserChatCommand * ucc = 0, *ucc1;
+	
+	if (!list)
+	{
+		return;
+	}
+	
+	m_lUserChatCommands.Clear();
+	
+	while( (ucc=list->Next(ucc)) != 0 )
+	{
+		ucc1 = new DC_UserChatCommand();
+		ucc1->m_sName    = ucc->m_sName;
+		ucc1->m_sCommand = ucc->m_sCommand;
+		m_lUserChatCommands.Add( ucc1 );
+		
+	}
+	
+	return;
+}
+
+/** */
+long DCConfig::GetUserChatCommands( CList<DC_UserChatCommand> * list )
+{
+	DC_UserChatCommand * ucc = 0, *ucc1;
+	
+	if (!list)
+	{
+		return 0;
+	}
+	
+	list->Clear();
+	
+	while( (ucc=m_lUserChatCommands.Next(ucc)) != 0 )
+	{
+		ucc1 = new DC_UserChatCommand();
+		ucc1->m_sName    = ucc->m_sName;
+		ucc1->m_sCommand = ucc->m_sCommand;
+		
+		list->Add( ucc1 );
+	}
+	
+	return list->Count();
+}
+
+/** */
+void DCConfig::SetAutoResponses( CList<DC_AutoResponseObject> * list )
+{
+	DC_AutoResponseObject * aro = 0, * aro1 = 0;
+	
+	if (!list)
+	{
+		return;
+	}
+	
+	m_lAutoResponses.Clear();
+	
+	while( (aro=list->Next(aro)) != 0 )
+	{
+		aro1 = new DC_AutoResponseObject();
+		aro1->m_sTrigger       = aro->m_sTrigger;
+		aro1->m_bCaseSensitive = aro->m_bCaseSensitive;
+		aro1->m_sResponse      = aro->m_sResponse;
+		
+		m_lAutoResponses.Add( aro1 );
+	}
+	
+	return;
+}
+
+/** */
+long DCConfig::GetAutoResponses( CList<DC_AutoResponseObject> * list )
+{
+	DC_AutoResponseObject * aro = 0, * aro1 = 0;
+	
+	if (!list)
+	{
+		return 0;
+	}
+	
+	list->Clear();
+	
+	while( (aro=m_lAutoResponses.Next(aro)) != 0)
+	{
+		aro1 = new DC_AutoResponseObject();
+		aro1->m_sTrigger       = aro->m_sTrigger;
+		aro1->m_bCaseSensitive = aro->m_bCaseSensitive;
+		aro1->m_sResponse      = aro->m_sResponse;
+		
+		list->Add( aro1 );
+	}
+	
+	return list->Count();
+}
+
+void DCConfig::RenameStoredFilelists()
+{
+	QDir dir;
+	QStringList listing;
+	
+	printf("Renaming stored filelists ...\n");
+	
+	dir.setPath( g_pConfig->GetFileListPath().Data() );
+	
+	listing = dir.entryList();
+	
+	for (QStringList::Iterator it = listing.begin(); it != listing.end(); ++it)
+	{
+		QString filename = *it;
+		
+		if ( (filename == ".") || (filename == "..") )
+		{
+			// skip
+		}
+		else
+		{
+			QFile file;
+			file.setName( g_pConfig->GetFileListPath().Data() + filename );
+			if (file.open( IO_ReadOnly ))
+			{
+				QString line, nick, hubhost;
+				file.readLine( line, (Q_ULONG) 2048 );
+				
+				if (line == "---HEADER START---\n")
+				{
+					while (line != "---HEADER END---\n")
+					{
+						file.readLine( line, (Q_ULONG) 2048 );
+						
+						if (line.left(5) == "NICK=")
+						{
+							nick = line.mid(5);
+							nick = nick.replace("\n", "");
+						}
+						else if (line.left(8) == "HUBHOST=")
+						{
+							hubhost = line.mid(8);
+							hubhost = hubhost.replace("\n", "");
+						}
+					}
+				}
+				else
+				{
+					printf("%s is not a valknut filelist\n", filename.ascii());
+					file.close();
+					continue;
+				}
+				
+				file.close();
+				
+				QString newname = nick + "-" + hubhost + ".filelist";
+				newname = newname.replace('/', "_");
+				newname = newname.replace('\\', "_");
+				newname = newname.replace(':', "_");
+				
+				printf("Renaming %s to %s ...", filename.ascii(), newname.ascii());
+				if (dir.rename(filename, newname) == TRUE)
+				{
+					printf(" OK\n");
+				}
+				else
+				{
+					printf(" renaming failed!\n");
+				}
+			}
+			else
+			{
+				printf("Couldn't open %s!\n", filename.ascii());
+			}
+		}
+	}
+	
+	// finished, set variable so renaming will not be done again
+	this->SetFilelistNamingSchemeVersion(2);
+}
