@@ -59,12 +59,6 @@ typedef enum eUserSound {
 	eusFIRSTRECEIVE
 } eUserSound;
 
-/** */
-typedef enum eChatTextCodec {
-	ectcINPUT=0,
-	ectcOUTPUT
-};
-
 /** */
 typedef enum eTimeStamp {
 	etsPRIVATECHAT,
@@ -114,6 +108,7 @@ typedef enum eChatColor {
 	eccPUBLICPRIVATECHATTEXT,
 	eccCHATSAY,
 	eccPUBLICCHATMENICK,
+	eccOPNICK
 } eChatColor;
 
 typedef enum eClientColumn {
@@ -184,6 +179,61 @@ public:
 	bool m_bDescription;
 };
 
+/** Stores a user command for custom menus */
+class DLL_EXPORT DC_UserMenuCommand : public CObject {
+public:
+	/** */
+	DC_UserMenuCommand() {
+		m_nID      = 0;
+		m_nContext = 0;
+		
+	};
+	/** */
+	virtual ~DC_UserMenuCommand() {};
+	/** The name of the menu command */
+	CString m_sName;
+	/** The command to send to the hub */
+	CString m_sCommand;
+	/** Menu id */
+	int m_nID;
+	/** Context is which menu(s) to put the command into */
+	int m_nContext;
+};
+
+/** Stores a user custom chat command */
+class DLL_EXPORT DC_UserChatCommand : public CObject {
+public:
+	/** */
+	DC_UserChatCommand() {};
+	/** */
+	virtual ~DC_UserChatCommand() {};
+	
+	/** Name of the command */
+	CString m_sName;
+	/** Command string to run */
+	CString m_sCommand;
+};
+
+/** Stores an auto response trigger and response pair */
+class DC_AutoResponseObject : public CObject {
+public:
+	/** */
+	DC_AutoResponseObject() {
+		m_sTrigger = "";
+		m_sResponse = "";
+		m_bCaseSensitive = FALSE;
+	};
+	/** */
+	virtual ~DC_AutoResponseObject() {};
+	
+	/** Trigger QT regexp */
+	CString m_sTrigger;
+	/** Response */
+	CString m_sResponse;
+	/** Case sensitive or not */
+	bool m_bCaseSensitive;
+};
+
 class DCConfig : public CConfig {
 public:
 	/** construtor */
@@ -225,6 +275,10 @@ public:
 	bool GetChatShowJoinsAndParts() const;
 	/** */
 	void SetChatShowJoinsAndParts( bool b );
+	/** */
+	bool GetChatShowJoinsAndPartsOnlyFav() const;
+	/** */
+	void SetChatShowJoinsAndPartsOnlyFav( bool b );
 
 	/** */
 	eUnit GetUnit() const;
@@ -246,7 +300,11 @@ public:
 	CString GetLanguageFile() const;
 	/** */
 	void SetLanguageFile( CString languagefile );
-        /** */
+	/** */
+	bool GetTabBar() const;
+	/** */
+	void SetTabBar( bool b );
+	/** */
 	bool GetToolBar() const;
 	/** */
 	void SetToolBar( bool b );
@@ -365,10 +423,6 @@ public:
 	int GetChatMessageMaxLines() const;
 	/** */
 	void SetChatMessageMaxLines( int i );
-	/** */
-	CString GetChatTextCodec( eChatTextCodec e );
-	/** */
-	void SetChatTextCodec( eChatTextCodec e, CString s );
 
 	/** */
 	int GetFloodCount() const;
@@ -449,11 +503,90 @@ public:
 	/** */
 	void SetUserPhotoFileName( CString s );
 	
+	/** Get the defualt remote text encoding for filelist browsers */
+	QString GetDefaultFBRemoteEncoding() const;
+	/** Set the default remote text encoding for filelist browsers */
+	void SetDefaultFBRemoteEncoding ( QString s );
+	
 	/** */
 	QString GetValknutDataPath() const;
 	/** */
 	void SetValknutDataPath( QString s );
-
+	
+	/** Get the list of user chat commands */
+	long GetUserChatCommands( CList<DC_UserChatCommand> * list );
+	/** Set the list of user chat commands */
+	void SetUserChatCommands( CList<DC_UserChatCommand> * list );
+	
+	/** Get the list of user menu commands */
+	long GetUserMenuCommands( CList<DC_UserMenuCommand> * list );
+	/** Set the list of user menu commands*/
+	void SetUserMenuCommands( CList<DC_UserMenuCommand> * list );
+	/** Loads the user icons */
+	void SetUserIcons( QPixmap p );	
+	
+	/** Gets the theme for application icons */
+	QString GetApplicationIconTheme() const;
+	/** Sets the theme for application icons */
+	void SetApplicationIconTheme( QString s );
+	
+	/** Gets the theme for emoticons */
+	QString GetEmoticonTheme() const;
+	/** Sets the theme for emoticons */
+	void SetEmoticonTheme( QString s );
+	
+	/** Gets the theme for userlist icons */
+	QString GetUserListIconTheme() const;
+	/** Sets the theme for userlist icons */
+	void SetUserListIconTheme( QString s );
+	
+	/** Gets the regexp of nicks to not open chat windows for */
+	QString GetSuppressedNicks() const;
+	/** Sets the regexp of nicks to not open chat windows for */
+	void SetSuppressedNicks( QString s );
+	
+	/** Gets the action for double clicking a user */
+	QString GetDoubleClickAction() const;
+	/** Sets the action for double clicking a user */
+	void SetDoubleClickAction( QString s );
+	
+	/** Gets if auto responder is enabled for main chat */
+	bool GetAutoResponderEnabled() const;
+	/** Enables/Disables the auto responder for main chat */
+	void SetAutoResponderEnabled( bool b );
+	/** Gets the list of auto response triggers and responses */
+	long GetAutoResponses( CList<DC_AutoResponseObject> * list );
+	/** Sets the list of auto response triggers and responses */
+	void SetAutoResponses( CList<DC_AutoResponseObject> * list );
+	/** Gets the minimum time between auto-responses */
+	long GetAutoResponseDelay() const;
+	/** Sets the minimum time between auto-responses */
+	void SetAutoResponseDelay( long n );
+	/** Gets list of nicks to not auto respond to */
+	CString GetAutoResponseIgnores() const;
+	/** Sets list of nicks to not auto respond to */
+	void SetAutoResponseIgnores( CString s );
+	/** Gets if the auto responder is enabled for PMs */
+	bool GetAutoResponderEnabledForPM() const;
+	/** Enables/Disabled the auto responder for private messages */
+	void SetAutoResponderEnabledForPM( bool b );
+	
+	/** Gets filelist naming scheme version */
+	int GetFilelistNamingSchemeVersion() const;
+	/** Sets filelist naming scheme version */
+	void SetFilelistNamingSchemeVersion( int i );
+	/** Renames the filelists */
+	void RenameStoredFilelists();
+	
+	/** Get show folders in right pane of filelist browser*/
+	bool GetFoldersInRightPane() const;
+	/** Set show folders in right pane of filelist browser */
+	void SetFoldersInRightPane( bool b );
+	/** Get show subfolder sizes in left pane of filelist browser */
+	bool GetFolderSizesInLeftPane() const;
+	/** Set show subfolder sizes in left pane of filelist browser */
+	void SetFolderSizesInLeftPane( bool b );
+	
 private:
 	/** */
 	QString m_sValknutDataPath;
@@ -468,6 +601,8 @@ private:
 	/** */
 	bool bShowTransferWin;
 	/** */
+	bool bTabBar;
+	/** */
 	bool bToolBar;
 	/** */
 	bool bStatusBar;
@@ -650,15 +785,18 @@ private:
 	CString m_sChatColor_17;
 	/** */
 	CString m_sChatColor_18;
+	/** Colour for operator nick */
+	CString m_sChatColor_19;
 	/** */
 	bool bOpenClientWindowsMinimized;
 	/** */
 	bool m_bChatShowJoinsAndParts;
 	/** */
-	CString m_sChatInputCodec;
-	/** */
-	CString m_sChatOutputCodec;
+	bool m_bChatShowJoinsAndPartsOnlyFav;
 
+	/** The default remote text encoding for filelist browsers */
+	QString m_sDefaultFBRemoteEncoding;
+	
 	/** */
 	bool m_bClientColumnComment;
 	/** */
@@ -697,6 +835,53 @@ private:
 
 	/** */
 	CString m_sUserPhotoFileName;
+	
+	/** The list of user customized chat commands */
+	CList<DC_UserChatCommand> m_lUserChatCommands;
+
+	/** The list of user customized menu commands */
+	CList<DC_UserMenuCommand> m_lUserMenuCommands;
+	
+	/** Theme for application icons */
+	QString m_sApplicationIconTheme;
+	
+	/** Theme for emoticons */
+	QString m_sEmoticonTheme;
+	
+	/** Theme for userlist icons */
+	QString m_sUserListIconTheme;
+	
+	/** Regexp of nicks to not open chat windows for */
+	QString m_sSuppressedNicks;
+	
+	/** Action to do on double clicking a user */
+	QString m_sDoubleClickAction;
+	
+	/** True if auto responder is enabled for main chat */
+	bool m_bAutoResponderEnabled;
+	
+	/** True if auto responder is enabled for private messages */
+	bool m_bAutoResponderEnabledForPM;
+	
+	/** The list of auto response triggers and responses */
+	CList<DC_AutoResponseObject> m_lAutoResponses;
+	
+	/** The time between consequtive auto-responses for a user */
+	long m_nAutoResponseDelay;
+	
+	/** A list of nicks to not auto respond to */
+	CString m_sAutoResponseIgnores;
+	
+	/** The filelist naming scheme version.
+	  * This is set internally so that the filelists are renamed
+	  * once only when upgrading to 0.3.10.
+	  */
+	int m_nFilelistNamingSchemeVersion;
+	
+	/** Show folders in right pane of filelist browser? */
+	bool m_bFoldersInRightPane;
+	/** Show subfolder sizes in left pane of filelist browser? */
+	bool m_bFolderSizesInLeftPane;
 };
 
 /** global config */
@@ -711,6 +896,12 @@ inline bool DCConfig::GetOpenPrivateChatWindow() const
 inline void DCConfig::SetOpenPrivateChatWindow( bool b )
 { bOpenPrivateChatWindow = b; }
 /** */
+inline bool DCConfig::GetTabBar() const
+{ return bTabBar; }
+/** */
+inline void DCConfig::SetTabBar( bool b )
+{ bTabBar = b; }
+/** */
 inline bool DCConfig::GetToolBar() const
 { return bToolBar; }
 /** */
@@ -928,16 +1119,102 @@ inline bool DCConfig::GetChatShowJoinsAndParts() const
 inline void DCConfig::SetChatShowJoinsAndParts( bool b )
 { m_bChatShowJoinsAndParts = b; }
 /** */
+inline bool DCConfig::GetChatShowJoinsAndPartsOnlyFav() const
+{ return m_bChatShowJoinsAndPartsOnlyFav; }
+/** */
+inline void DCConfig::SetChatShowJoinsAndPartsOnlyFav( bool b )
+{ m_bChatShowJoinsAndPartsOnlyFav = b; }
+/** */
 inline CString DCConfig::GetUserPhotoFileName() const
 { return m_sUserPhotoFileName; }
 /** */
 inline void DCConfig::SetUserPhotoFileName( CString s )
 { m_sUserPhotoFileName = s; }
 /** */
+inline QString DCConfig::GetDefaultFBRemoteEncoding() const
+{ return m_sDefaultFBRemoteEncoding; }
+/** */
+inline void DCConfig::SetDefaultFBRemoteEncoding( QString s )
+{ m_sDefaultFBRemoteEncoding = s; }
+/** */
 inline QString DCConfig::GetValknutDataPath() const
 { return m_sValknutDataPath; }
 /** */
 inline void DCConfig::SetValknutDataPath( QString s )
 { m_sValknutDataPath = s; }
-
+/** */
+inline QString DCConfig::GetApplicationIconTheme() const
+{ return m_sApplicationIconTheme; }
+/** */
+inline void DCConfig::SetApplicationIconTheme( QString s )
+{ m_sApplicationIconTheme = s; }
+/** */
+inline QString DCConfig::GetEmoticonTheme() const
+{ return m_sEmoticonTheme; }
+/** */
+inline void DCConfig::SetEmoticonTheme( QString s )
+{ m_sEmoticonTheme = s; }
+/** */
+inline QString DCConfig::GetUserListIconTheme() const
+{ return m_sUserListIconTheme; }
+/** */
+inline void DCConfig::SetUserListIconTheme( QString s )
+{ m_sUserListIconTheme = s; }
+/** */
+inline void DCConfig::SetUserIcons( QPixmap p )
+{ m_UserIcons = p; }
+/** */
+inline QString DCConfig::GetSuppressedNicks() const
+{ return m_sSuppressedNicks; }
+/** */
+inline void DCConfig::SetSuppressedNicks( QString s )
+{ m_sSuppressedNicks = s; }
+/** */
+inline QString DCConfig::GetDoubleClickAction() const
+{ return m_sDoubleClickAction; }
+/** */
+inline void DCConfig::SetDoubleClickAction( QString s )
+{ m_sDoubleClickAction = s; }
+/** */
+inline bool DCConfig::GetAutoResponderEnabled() const
+{ return m_bAutoResponderEnabled; }
+/** */
+inline void DCConfig::SetAutoResponderEnabled( bool b )
+{ m_bAutoResponderEnabled = b; }
+/** */
+inline long DCConfig::GetAutoResponseDelay() const
+{ return m_nAutoResponseDelay; }
+/** */
+inline void DCConfig::SetAutoResponseDelay( long n )
+{ m_nAutoResponseDelay = n; }
+/** */
+inline CString DCConfig::GetAutoResponseIgnores() const
+{ return m_sAutoResponseIgnores; }
+/** */
+inline void DCConfig::SetAutoResponseIgnores( CString s )
+{ m_sAutoResponseIgnores = s; }
+/** */
+inline bool DCConfig::GetAutoResponderEnabledForPM() const
+{ return m_bAutoResponderEnabledForPM; }
+/** */
+inline void DCConfig::SetAutoResponderEnabledForPM( bool b )
+{ m_bAutoResponderEnabledForPM = b; }
+/** */
+inline int DCConfig::GetFilelistNamingSchemeVersion() const
+{ return m_nFilelistNamingSchemeVersion; }
+/** */
+inline void DCConfig::SetFilelistNamingSchemeVersion( int i )
+{ m_nFilelistNamingSchemeVersion = i; }
+/** */
+inline bool DCConfig::GetFoldersInRightPane() const
+{ return m_bFoldersInRightPane; }
+/** */
+inline void DCConfig::SetFoldersInRightPane( bool b)
+{ m_bFoldersInRightPane = b; }
+/** */
+inline bool DCConfig::GetFolderSizesInLeftPane() const
+{ return m_bFolderSizesInLeftPane; }
+/** */
+inline void DCConfig::SetFolderSizesInLeftPane( bool b )
+{ m_bFolderSizesInLeftPane = b; }
 #endif
