@@ -27,6 +27,7 @@
 #include <qtextcodec.h>
 #include <qtooltip.h>
 #include <qclipboard.h>
+#include <qmessagebox.h>
 
 #include <dcwidget.h>
 #include <dcmenuhandler.h>
@@ -50,18 +51,27 @@ DCFileBrowser::DCFileBrowser( QWidget* parent, const char *name, int wflags, boo
 	
 	// set width mode
 	ListView_DIRECTORY->setColumnWidthMode( 0, QListView::Maximum );
-	// hide the column header
-	ListView_DIRECTORY->header()->hide();
+	ListView_FILES->setColumnWidthMode( 0, QListView::Maximum );
+	ListView_FILES->setColumnWidthMode( 1, QListView::Maximum );
+	ListView_FILES->setColumnWidthMode( 2, QListView::Maximum );
+	ListView_FILES->setColumnWidthMode( 3, QListView::Maximum );
+	ListView_FILES->setColumnWidthMode( 4, QListView::Maximum );
+	
+	// hide the column header - only if not using size column
+	if ( g_pConfig->GetFolderSizesInLeftPane() == FALSE )
+	{
+		ListView_DIRECTORY->header()->hide();
+	}
+
+	m_pCodec     = 0;
+	m_pCodecRemote = 0;
+	m_nShareSize = 0;
+	m_nFileCount = 0;
 
 	if ( !viewonly )
 	{
 		InitDocument();
 	}
-	
-	m_pCodec     = 0;
-	m_pCodecRemote = 0;
-	m_nShareSize = 0;
-	m_nFileCount = 0;
 }
 
 /** */
@@ -86,6 +96,7 @@ void DCFileBrowser::InitDocument()
 	connect( ListView_DIRECTORY, SIGNAL(collapsed(QListViewItem*)), this, SLOT(slotCollapsedDirectory(QListViewItem*)) );
 	connect( ListView_DIRECTORY, SIGNAL(contextMenuRequested( QListViewItem *, const QPoint &, int )), this, SLOT(slotRightButtonClickedDirectory(QListViewItem*, const QPoint &, int )) );
 	connect( ListView_FILES, SIGNAL(contextMenuRequested( QListViewItem *, const QPoint &, int )), this, SLOT(slotRightButtonClickedFiles(QListViewItem*, const QPoint &, int )) );
+	connect( ListView_FILES, SIGNAL(doubleClicked( QListViewItem *, const QPoint &, int)), this, SLOT(slotItemDoubleClicked( QListViewItem *, const QPoint &, int )) );
 
  	connect( ToolButton_OPEN, SIGNAL(clicked()), this, SLOT(slotFileOpen()) );
  	connect( ToolButton_SAVE, SIGNAL(clicked()), this, SLOT(slotFileSave()) );
@@ -107,7 +118,16 @@ void DCFileBrowser::InitDocument()
 	
 	ComboBox_RENCODING->insertItem("DEFAULT");
 	ComboBox_RENCODING->insertStringList(lst);
-	ComboBox_RENCODING->setCurrentItem(0);
+	
+	for(i=0;i<ComboBox_RENCODING->count();i++)
+	{
+		if ( ComboBox_RENCODING->text(i) == g_pConfig->GetDefaultFBRemoteEncoding() )
+		{
+			ComboBox_RENCODING->setCurrentItem(i);
+			break;
+		}
+	}
+	slotCodecRemoteChange(ComboBox_RENCODING->currentText());	
 }
 
 /** */
@@ -269,21 +289,37 @@ void DCFileBrowser::slotCurrentChangedDirectory()
 
 		for ( FileItem = ((DCFileBrowserListItem*)item)->m_pFileList.first(); FileItem; FileItem = ((DCFileBrowserListItem*)item)->m_pFileList.next() )
 		{
-			size += CString(FileItem->m_sSize.ascii()).asULL();
+			if ( (g_pConfig->GetFoldersInRightPane() == FALSE) && (FileItem->m_bIsDir == TRUE) )
+			{
+				// do not show it
+				continue;
+			}
+			
+			size += FileItem->m_sSize.toULongLong();
 			s = dir.Extension(FileItem->m_sName.ascii()).Data();
 
 			if ( s != "" )
 				s += " " + tr("file");
-			items++;
 
 			DC_QListViewItem *item1 = new DC_QListViewItem(ListView_FILES);
-			item1->myvalue = CString(FileItem->m_sSize.ascii()).asULL();
+			item1->myvalue = FileItem->m_sSize.toULongLong();
 			item1->mycol   = 1;
+			
+			if ( FileItem->m_bIsDir )
+			{
+				s = tr("Folder");
+				item1->setPixmap(0,m_FolderPixmap);
+			}
+			else
+			{
+				items++;
+			}
 
 			item1->setText(0,FileItem->m_sName);
-			item1->setText(1,FileItem->m_sSize);
-			item1->setText(2,s);
-			item1->setText(3,FileItem->m_sHash);
+			item1->setText(1,CUtils::GetSizeString(FileItem->m_sSize.toULongLong(), g_pConfig->GetUnit()).Data() );
+			item1->setText(2,FileItem->m_sSize);
+			item1->setText(3,s);
+			item1->setText(4,FileItem->m_sHash);
 		}
 	}
 
@@ -327,8 +363,7 @@ void DCFileBrowser::slotRightButtonClickedFiles( QListViewItem * item, const QPo
 /** */
 void DCFileBrowser::LoadFileList( QString s )
 {
-	CXml xml;
-	char ch;
+	QByteArray ba;
 	m_sFileName = s;
 
 	QFile file( s );
@@ -344,6 +379,10 @@ void DCFileBrowser::LoadFileList( QString s )
 	if ( file.open( IO_ReadOnly ) )
 	{
 		QTextStream stream( &file );
+		
+		// readLine() will convert from file UTF-8 to QString 16bit unicode
+		stream.setEncoding( QTextStream::UnicodeUTF8 );
+		
 		QString line;
 
 		line = stream.readLine();
@@ -357,33 +396,44 @@ void DCFileBrowser::LoadFileList( QString s )
 
 				if ( line.find("TYPE=") == 0 )
 				{
-					m_sType = xml.FromUtf8(line.mid(5).ascii()).Data();
+					m_sType = line.mid(5);
 				}
 				else if ( line.find("NICK=") == 0 )
 				{
-					m_sNick = xml.FromUtf8(line.mid(5).ascii()).Data();
+					m_sNick = line.mid(5);
 				}
 				else if ( line.find("HUBNAME=") == 0 )
 				{
-					m_sHubName = xml.FromUtf8(line.mid(8).ascii()).Data();
+					m_sHubName = line.mid(8);
 				}
 				else if ( line.find("HUBHOST=") == 0 )
 				{
-					m_sHubHost = xml.FromUtf8(line.mid(8).ascii()).Data();
+					m_sHubHost = line.mid(8);
 				}
 			}
 
-			// read content ...
-			while ( !stream.atEnd() )
-			{
-				stream.readRawBytes(&ch,1);
-				m_sList += ch;
-			}
+			// the loop reading one byte at a time was replaced
+			// with this
+			ba = stream.device()->readAll();
 			
-			if ( m_sType != "XMLFILELIST" )
+			// WARNING! ba.data() includes trailing garbage!
+			// it must only be used in conjunction with ba.size()
+			
+			// convert the byte array to QString 16bit unicode
+			if ( m_sType == "XMLFILELIST" )
 			{
-				// convert from utf8
-				m_sList = xml.FromUtf8(m_sList.ascii()).Data();
+				// XML lists should be in UTF-8
+				m_sList = QString::fromUtf8( ba.data(), ba.size() );
+			}
+			else
+			{
+				// text lists get converted to UTF-8 by CXml in cdownloadmanager.cpp
+				// we want it in UTF-8
+				// but without the &NN xml substitutions
+				CXml xml;
+				CString list;
+				list.Set( ba.data(), ba.size() );
+				m_sList = QString::fromUtf8( xml.FromUtf8( list, "UTF-8" ).Data() );
 			}
 		}
 
@@ -404,13 +454,16 @@ void DCFileBrowser::AddDirectory( QListViewItem * item, QString name )
 
 	item->setText(0,name);
 	item->setPixmap(0,m_FolderPixmap);
+	
+	((DCFileBrowserListItem*)item)->m_nBytes = 0;
 }
 
 /** */
 void DCFileBrowser::AddFile( QListViewItem * item, DCFileItem * fileitem )
 {
 	// increment filecounter
-	m_nFileCount++;
+	if ( fileitem->m_bIsDir == FALSE )
+		m_nFileCount++;
 
 	// recode filename
 	if ( m_pCodec )
@@ -420,10 +473,21 @@ void DCFileBrowser::AddFile( QListViewItem * item, DCFileItem * fileitem )
 	if ( fileitem->m_sSize == "" )
 		fileitem->m_sSize = tr("<wrong length>");
 	else
-		m_nShareSize += CString(fileitem->m_sSize.ascii()).asULL();
+		if ( fileitem->m_bIsDir == FALSE )
+			m_nShareSize += fileitem->m_sSize.toULongLong();
 
 	// add fileitem to filelist
 	((DCFileBrowserListItem*)item)->m_pFileList.append(fileitem);
+	
+	// add to folder contents size
+	((DCFileBrowserListItem*)item)->m_nBytes += fileitem->m_sSize.toULongLong();
+	
+	DCFileItem * pFileItem = ((DCFileBrowserListItem*)item)->m_pFileItem;
+	
+	if ( pFileItem != 0 )
+	{
+		pFileItem->m_sSize = QString().setNum( ((DCFileBrowserListItem*)item)->m_nBytes );
+	}
 }
 
 /** */
@@ -458,6 +522,7 @@ void DCFileBrowser::InitTree( QString nick, QString hubname, QString hubhost, QS
 	item = new DCFileBrowserListItem( ListView_DIRECTORY );
 	item->setText(0,tr("Root Directory"));
 	item->setPixmap(0,g_pIconLoader->GetPixmap(eiFOLDER_RED));
+	((DCFileBrowserListItem*)item)->m_pFileItem = 0;
 
 	if ( m_sType != "XMLFILELIST" )
 		InitTXTTree(item);			
@@ -515,15 +580,26 @@ void DCFileBrowser::InitTXTTree( QListViewItem * item )
 
 				FileItem->m_sName = s1.mid(0,d);
 				FileItem->m_sSize = s1.right(s1.length()-d-1);
+				FileItem->m_bIsDir = FALSE;
 				
 				AddFile( item, FileItem );
 			}
 			else // parse dir entry
 			{
+				DCFileItem * FileItem = new DCFileItem();
+				
+				FileItem->m_sName = s1;
+				FileItem->m_sSize = "0";
+				FileItem->m_bIsDir = TRUE;
+				
+				AddFile( item, FileItem );
+				
 				if ( item == 0 )
 					item = new DCFileBrowserListItem( ListView_DIRECTORY );
 				else
 					item = new DCFileBrowserListItem( item );
+				
+				((DCFileBrowserListItem*)item)->m_pFileItem = FileItem;
 
 				AddDirectory( item, s1 );
 			}
@@ -531,6 +607,15 @@ void DCFileBrowser::InitTXTTree( QListViewItem * item )
 
 		j = i+2;
 	}
+	
+	// fixup the folder sizes
+	QListViewItem * cur = ListView_DIRECTORY->firstChild();
+	while ( cur )
+	{
+		CalcDirSize( cur );
+		
+		cur = cur->nextSibling();
+	}
 }
 
 /** */
@@ -541,7 +626,9 @@ void DCFileBrowser::InitXMLTree( QListViewItem * item )
 
 	xml = new CXml();
 	
-	if ( xml->ParseMemory(m_sList.ascii(),m_sList.length()) == TRUE )
+	// strlen(m_sList.ascii()) replaces m_sList.length() to fix bug
+	// with codec for C strings set to UTF-8
+	if ( xml->ParseMemory(m_sList.ascii(),strlen(m_sList.ascii())) == TRUE )
 	{
 		for(node=xml->doc()->children;node!=0;node=node->next)
 		{
@@ -567,19 +654,34 @@ void DCFileBrowser::ParseXMLTree( CXml * xml, xmlNodePtr node, QListViewItem * i
 			QListViewItem * item1;
 			
 			item1 = new DCFileBrowserListItem( item );
+			((DCFileBrowserListItem*)item1)->m_pFileItem = 0;
 			
 			s = xml->prop(node,"Name");
-			AddDirectory(item1,QString().fromUtf8(s.Data()));
+			AddDirectory(item1,QString::fromUtf8(s.Data()));
 			
 			ParseXMLTree(xml,node->xmlChildrenNode,item1);
+			
+			DCFileItem * FileItem = new DCFileItem();
+			FileItem->m_sName = QString::fromUtf8(s.Data());
+			FileItem->m_sSize = QString().setNum(((DCFileBrowserListItem*)item1)->m_nBytes);
+			FileItem->m_bIsDir = TRUE;
+			AddFile(item,FileItem);
+			((DCFileBrowserListItem*)item1)->m_pFileItem = FileItem;
+						
+			if ( g_pConfig->GetFolderSizesInLeftPane() == TRUE )
+			{
+				CString size = CUtils::GetSizeString( ((DCFileBrowserListItem*)item1)->m_nBytes, g_pConfig->GetUnit() );
+				item1->setText( 1, size.Data() );
+			}
 		}
 		else if ( !xmlStrcmp( node->name, (const xmlChar*)"File" ) )
 		{
 			DCFileItem * FileItem = new DCFileItem();
 			s = xml->prop(node,"Name");
-			FileItem->m_sName = QString().fromUtf8(s.Data());
+			FileItem->m_sName = QString::fromUtf8(s.Data());
 			FileItem->m_sSize = xml->prop(node,"Size").Data();
 			FileItem->m_sHash = xml->prop(node,"TTH").Data();
+			FileItem->m_bIsDir = FALSE;
 			
 			AddFile(item,FileItem);
 		}
@@ -640,11 +742,31 @@ void DCFileBrowser::CreateMenu( QListViewItem *, bool direntry, int column )
 	QString localName = QString::null;
 	QPtrList<QListViewItem> selitems;
 	QListViewItem * curitem;
+	bool hasFolderItems = FALSE;
 	
 	if ( direntry )
+	{
 		i = selectedItems( ListView_DIRECTORY, selitems );
+	}
 	else
+	{
 		i = selectedItems( ListView_FILES, selitems );
+		
+		QPtrListIterator<QListViewItem> plit ( selitems );
+		
+		QListViewItem * cur = 0;
+		
+		while ( (cur = plit.current()) != 0 )
+		{
+			++plit;
+			
+			if ( cur->text(3) == tr("Folder") )
+			{
+				hasFolderItems = TRUE;
+				break;
+			} 
+		}
+	}
 
 	// nothing on no selection
 	if ( i == 0 )
@@ -663,11 +785,12 @@ void DCFileBrowser::CreateMenu( QListViewItem *, bool direntry, int column )
 	DCMenuHandler::InsertMenu(m, emiDOWNLOAD);
 	DCMenuHandler::InsertMenu(m, emiDOWNLOAD_TO);
 	// disable 'download as' on directories
-	DCMenuHandler::InsertMenu(m, emiDOWNLOAD_AS, (direntry ? FALSE : TRUE));
-	DCMenuHandler::InsertMenu(m, emiDOWNLOAD_IN, (direntry ? FALSE : TRUE));
+	DCMenuHandler::InsertMenu(m, emiDOWNLOAD_AS, ((direntry ? FALSE : TRUE) && !hasFolderItems));
+	DCMenuHandler::InsertMenu(m, emiDOWNLOAD_IN, ((direntry ? FALSE : TRUE) && !hasFolderItems));
 	DCMenuHandler::InsertMenu(m, emiSEPARATOR);
 	DCMenuHandler::InsertMenu(m, emiCOPY_COLUMN_TO_CLIPBOARD, (direntry ? FALSE : TRUE));
 	DCMenuHandler::InsertMenu(m, emiCOPY_ROW_TO_CLIPBOARD, (direntry ? FALSE : TRUE));
+	DCMenuHandler::InsertMenu(m, emiCOPYMAGNETLINK, ((direntry ? FALSE : TRUE) && !hasFolderItems ));
 	DCMenuHandler::InsertMenu(m, emiSEPARATOR);
 
 	id = m->exec(QCursor::pos());
@@ -706,6 +829,41 @@ void DCFileBrowser::CreateMenu( QListViewItem *, bool direntry, int column )
 
 		cb->setText(s);
 	}
+	else if ( id == emiCOPYMAGNETLINK )
+	{
+		//magnet:?xt=urn:tree:tiger:EOSA5AGTL5SD3VWCF3R2OH2WMGXV3S3R7SYN4YA&xl=708780032&dn=FC-6-i386-disc1.iso
+		QString all = "";
+		QClipboard *cb = QApplication::clipboard();
+		
+		for ( curitem = selitems.first(); curitem; curitem = selitems.next() )
+		{
+			// add tth
+			all += "magnet:?xt=urn:tree:tiger:";
+			all += curitem->text(4);
+			
+			// add filesize
+			all += "&xl=";
+			all += curitem->text(2);
+			
+			// add filename
+			all += "&dn=";
+			QString filename = curitem->text(0);
+			
+			QUrl::encode(filename);
+			
+			//fixme DC++ replaces spaces with + character
+			//filename = filename.replace(' ','+');
+			
+			all += filename;
+			
+			all += "\n";
+		}
+		
+		// remove trailing "\n"
+		all = all.stripWhiteSpace();
+		
+		cb->setText(all);
+	}
 	// Check if the user wants a non-default destination
 	else if ( id == emiDOWNLOAD_TO )
 	{
@@ -778,7 +936,7 @@ void DCFileBrowser::CreateMenu( QListViewItem *, bool direntry, int column )
 			// add transfer to the waitlist
 			m_FileTool.CheckFile( m_sNick.ascii(), m_sHubName.ascii(), m_sHubHost.ascii(),
 				  remotePathAndFile.ascii(), localname.ascii(), localpath.ascii(), localrootpath.ascii(), eltFILE,
-		  		  size, "", TRUE);
+		  		  size, curitem->text(4).ascii(), TRUE);
 		}
 	}
 
@@ -787,12 +945,18 @@ void DCFileBrowser::CreateMenu( QListViewItem *, bool direntry, int column )
 	{
                 for ( curitem = selitems.first(); curitem; curitem = selitems.next() )
                 {
-			// directory download
+			// directory download from left pane
 			if ( direntry )
 			{
 				s = curitem->text(0);
 					
-				DownloadPath( rootPath, s, localName, curitem );
+				DownloadPath( rootPath, s, QString::null, curitem );
+			}
+			else if ( curitem->text(3) == tr("Folder") ) // directory download from right pane
+			{
+				QListViewItem * diritem = GetDirItem( curitem );
+				
+				DownloadPath( rootPath, diritem->text(0), QString::null, diritem );
 			}
 			// Single file download
 			else
@@ -814,7 +978,7 @@ void DCFileBrowser::CreateMenu( QListViewItem *, bool direntry, int column )
 				else
 				    remoteName = curitem->text(0);
 				
-				DownloadFile( rootPath, "", localName, remotePath, remoteName, ((DC_QListViewItem *)curitem)->myvalue, curitem->text(3) );
+				DownloadFile( rootPath, "", localName, remotePath, remoteName, ((DC_QListViewItem *)curitem)->myvalue, curitem->text(4) );
 			}
 		}
 	}
@@ -838,13 +1002,16 @@ void DCFileBrowser::DownloadPath( QString rootPath, QString localPath, QString l
 	
 	for ( FileItem = ((DCFileBrowserListItem*)item)->m_pFileList.first(); FileItem; FileItem = ((DCFileBrowserListItem*)item)->m_pFileList.next() )
 	{
-		if ( m_pCodec ) {
-		    remoteFile = m_pCodec->fromUnicode(FileItem->m_sName);
-		} else {
-		    remoteFile = FileItem->m_sName;
+		if ( FileItem->m_bIsDir == FALSE )
+		{
+			if ( m_pCodec ) {
+		    	remoteFile = m_pCodec->fromUnicode(FileItem->m_sName);
+			} else {
+		    	remoteFile = FileItem->m_sName;
+			}
+			DownloadFile( rootPath, localPath, localName, remotePath, 
+				remoteFile, FileItem->m_sSize.toULongLong(), FileItem->m_sHash );
 		}
-		DownloadFile( rootPath, localPath, localName, remotePath, 
-				remoteFile, CString(FileItem->m_sSize.ascii()).asULL(), FileItem->m_sHash );
 	}
 
 	item = item->firstChild();
@@ -891,3 +1058,149 @@ void DCFileBrowser::DownloadFile( QString rootPath, QString localPath, QString l
 	
 	
 }
+
+/** Double click to download the item, copied from DCFileBrowser::CreateMenu */
+void DCFileBrowser::slotItemDoubleClicked( QListViewItem * item, const QPoint &, int )
+{
+	QString rootPath = QString::null;
+	QString localName = QString::null;
+	
+	if ( item == 0 ) // null pointer check, just in case
+	{
+		return;
+	}
+	
+	if ( item->text(3) == tr("Folder") )
+	{
+		// we cannot open the folder since that would delete the existing
+		// QListViewItems, which is not allowed in this slot and causes
+		// random crashes
+		
+		/* QListViewItem * diritem = ListView_DIRECTORY->currentItem();
+		
+		if ( diritem == 0 )
+		{
+			printf("Double click a folder needs a ListView_DIRECTORY->currentItem()\n");
+			return;
+		}
+		
+		for ( diritem = diritem->firstChild(); diritem != 0; diritem = diritem->nextSibling() )
+		{
+			if ( diritem->text(0) == item->text(0) )
+			{
+				ListView_DIRECTORY->clearSelection();
+				ListView_DIRECTORY->ensureItemVisible( diritem );
+				ListView_DIRECTORY->setSelected( diritem, TRUE );
+				ListView_DIRECTORY->setCurrentItem( diritem );
+				break;
+			}
+		} */
+		
+		QListViewItem * diritem = GetDirItem( item );
+		
+		// ask for confirmation since this is unusual behaviour
+		int confirm = QMessageBox::question(
+			this,
+			tr("Download contents?"),
+			tr("Download the contents of \"") + diritem->text(0) + "\" ?",
+			QMessageBox::Yes,
+			QMessageBox::No
+		);
+		
+		if ( confirm == QMessageBox::Yes )
+		{
+			DownloadPath( rootPath, diritem->text(0), localName, diritem );
+		}
+	}
+	else
+	{
+		QString remotePath;
+		QString remoteName;
+		
+		// get the full remote path
+		remotePath = CreateRemotePath(ListView_DIRECTORY->currentItem());
+		
+		if ( m_pCodec )
+			remoteName = m_pCodec->fromUnicode(item->text(0));
+		else
+			remoteName = item->text(0);
+		
+		DownloadFile( rootPath, "", localName, remotePath, remoteName, ((DC_QListViewItem *)item)->myvalue, item->text(4) );
+	}
+}
+
+/** Only needed for non-xml filelists */
+ulonglong DCFileBrowser::CalcDirSize( QListViewItem * item )
+{
+	ulonglong total = 0;
+
+	DCFileBrowserListItem * cur = (DCFileBrowserListItem*) item;
+	
+	QPtrListIterator<DCFileItem> it( cur->m_pFileList );
+	for ( ; it.current(); ++it)
+	{
+		if ( it.current()->m_bIsDir == TRUE )
+		{
+			QListViewItem * child = item->firstChild();
+			
+			while ( child )
+			{
+				if ( child->text(0) == it.current()->m_sName )
+				{
+					total += CalcDirSize( child );
+					break;
+				}
+				
+				child = child->nextSibling();
+			}
+		}
+		else
+		{
+			total += it.current()->m_sSize.toULongLong();
+		}
+	}
+	
+	DCFileItem * pFileItem = cur->m_pFileItem;
+	if ( pFileItem != 0 )
+	{
+		pFileItem->m_sSize = QString().setNum(total);
+	}
+	
+	cur->m_nBytes = total;
+	
+	if ( g_pConfig->GetFolderSizesInLeftPane() == TRUE )
+	{
+		CString size = CUtils::GetSizeString( total, g_pConfig->GetUnit() );
+		cur->setText( 1, size.Data() );
+	}
+	
+	//printf( "CalcDirSize for %s = %lld\n", item->text(0).ascii(), total );
+	
+	return total;
+}
+
+/** */
+QListViewItem * DCFileBrowser::GetDirItem( QListViewItem * item )
+{
+	QListViewItem * diritem = ListView_DIRECTORY->currentItem();
+	
+	if ( item == 0 )
+	{
+		return 0;
+	}
+	
+	if ( diritem == 0 )
+	{
+		return 0;
+	}
+	
+	for ( diritem = diritem->firstChild(); diritem != 0; diritem = diritem->nextSibling() )
+	{
+		if ( diritem->text(0) == item->text(0) )
+		{
+			return diritem;
+		}
+	}
+	
+	return 0;
+}
