@@ -66,6 +66,7 @@
 DCGuiApp::DCGuiApp()
 {
 	StringMap * map;
+	StringMap * tabBarMap;
 	int width,height;
 	bool b;
 
@@ -123,6 +124,13 @@ DCGuiApp::DCGuiApp()
 			pTransferViewDock->setFixedExtentHeight((*map)["HEIGHT"].toInt());
 			pTransferViewDock->setFixedExtentWidth((*map)["WIDTH"].toInt());
 		}
+		
+		bool nl = TRUE;
+		if ( (*map)["NL"] == "FALSE" )
+		{
+			nl = FALSE;
+		}
+		moveDockWindow( pTransferViewDock, Qt::Dock((*map)["DOCK"].toInt()), nl, (*map)["INDEX"].toInt(), (*map)["EXTRAOFFSET"].toInt() );
 	}
 	else
 	{
@@ -138,6 +146,8 @@ DCGuiApp::DCGuiApp()
 	viewTabBar->setOn(true);
 	viewToolBar->setOn(true);
 	viewStatusBar->setOn(true);
+	
+	m_bWasMaximized = FALSE;
 
 	// init main window
 	if ( g_pConfig->GetMap("MAINVIEW",map) == TRUE )
@@ -145,10 +155,12 @@ DCGuiApp::DCGuiApp()
 		if ( (*map)["MAXIMIZED"].toInt() == 1 )
 		{
 			showMaximized();
+			m_bWasMaximized = TRUE;
 		}
 		else
 		{
 			show();
+			m_bWasMaximized = FALSE;
 		}
 
 		if ( (*map)["DOCKED"].toInt() == 0 )
@@ -163,6 +175,8 @@ DCGuiApp::DCGuiApp()
 		}
 	}
 
+	if ( g_pConfig->GetTabBar() == FALSE )
+		viewTabBar->toggle();
 	if ( g_pConfig->GetToolBar() == FALSE )
 		viewToolBar->toggle();
 	if ( g_pConfig->GetStatusBar() == FALSE )
@@ -179,6 +193,45 @@ DCGuiApp::DCGuiApp()
 		m_bTransferViewDockVisible = FALSE;
 	}
 
+	// restore tabbar position and state
+	if ( g_pConfig->GetMap("TABBAR",tabBarMap) == TRUE )
+	{
+		int x = (*tabBarMap)["X"].toInt();
+		int y = (*tabBarMap)["Y"].toInt();
+		int w = (*tabBarMap)["WIDTH"].toInt();
+		int h = (*tabBarMap)["HEIGHT"].toInt();
+		
+		if ( (*tabBarMap)["DOCKED"].toInt() == 0 )
+		{
+			windowsTabBar->undock();
+			
+			if ( (w != 0) && (h != 0) )
+			{
+				windowsTabBar->resize(w,h);
+			}
+		
+			windowsTabBar->move(x,y);
+		}
+		else
+		{
+			windowsTabBar->dock();
+			
+			bool nl = TRUE;
+			if ( (*tabBarMap)["NL"] == "FALSE" )
+			{
+				nl = FALSE;
+			}
+			
+			moveDockWindow( windowsTabBar, Qt::Dock((*tabBarMap)["DOCK"].toInt()), nl, (*tabBarMap)["INDEX"].toInt(), (*tabBarMap)["EXTRAOFFSET"].toInt() );
+			
+			/* if ( (w != 0) && (h != 0) )
+			{
+				windowsTabBar->setFixedExtentWidth(w);
+				windowsTabBar->setFixedExtentHeight(h);
+			} */
+		}
+	}
+
 	g_pTranslator = new DCTranslator();
 
 	// install event filter on qApp ;-)
@@ -511,6 +564,7 @@ void DCGuiApp::initToolBar()
 	///////////////////////////////////////////////////////////////////
 	// TOOLBAR
 	fileToolbar = new QToolBar(this, tr("Action"));
+	fileToolbar->setLabel(tr("Toolbar"));
 
 	QWhatsThis::whatsThisButton(fileToolbar);
 
@@ -557,6 +611,7 @@ void DCGuiApp::initToolBar()
 	
 	
 	windowsTabBar = new QToolBar(this, tr("Tab Bar"));
+	windowsTabBar->setLabel(tr("Tab Bar"));
 	
 	// init tabbar
 	g_pConnectionManager->InitTabBar(windowsTabBar);
@@ -609,10 +664,11 @@ void DCGuiApp::initView()
 	setDockMenuEnabled(FALSE);
 
 	// create update manager
-	new DCUpdateManager(pWorkspace,0,WDestructiveClose);
-	g_pUpdateManager->installEventFilter(this);
-	g_pUpdateManager->hide();
-	g_pUpdateManager->setEnabled(FALSE);
+	//new DCUpdateManager(pWorkspace,0,WDestructiveClose);
+	//g_pUpdateManager->installEventFilter(this);
+	//g_pUpdateManager->hide();
+	//g_pUpdateManager->setEnabled(FALSE);
+	g_pUpdateManager = 0;
 	
 	// create connection manager view
 	new DCConnectionManager(pWorkspace,0,WDestructiveClose);
@@ -628,6 +684,7 @@ void DCGuiApp::initView()
 
 	// create dock window for transfer view
 	pTransferViewDock = new QDockWindow( QDockWindow::InDock, this );
+	pTransferViewDock->setCaption(tr("Transfer List"));
 	pTransferViewDock->setResizeEnabled(TRUE);
 	moveDockWindow(pTransferViewDock,Qt::DockBottom);
 	pTransferViewDock->hide();
@@ -877,18 +934,18 @@ void DCGuiApp::deinitTray()
 /** */
 bool DCGuiApp::queryExit()
 {
-	int exit=QMessageBox::information(this, tr("Quit..."),
+	int exit=QMessageBox::question(this, tr("Quit..."),
 				tr("Do you really want to quit?"),
 				QMessageBox::Ok, QMessageBox::Cancel);
 
-	if (exit==1)
+	if (exit==QMessageBox::Ok)
 	{
 	}
 	else
 	{
 	}
 
-	return (exit==1);
+	return (exit==QMessageBox::Ok);
 }
 
 /** */
@@ -1043,7 +1100,7 @@ bool DCGuiApp::eventFilter(QObject* object, QEvent* event)
 			s += QString().setNum(e->m_pFileManagerInfo->m_nProgress,'f',2) + "%";
 		}
 
-		if ( s == "" )
+		if ( CFileManager::Instance()->IsCreateShareListRunning() == FALSE )
 		{
 			statusFMLabel->hide();
 			statusBar()->message(tr("Refresh share finished."));
@@ -1256,6 +1313,7 @@ void DCGuiApp::slotFileCreateShare()
 bool DCGuiApp::slotFileQuit()
 {
 	StringMap * map;
+	StringMap * tabBarMap;
 
 	statusBar()->message(tr("Exiting application..."));
 
@@ -1290,6 +1348,7 @@ bool DCGuiApp::slotFileQuit()
 	// close all hub windows
 	g_pConnectionManager->CloseDisconnectedHubs();
 
+	g_pConfig->SetTabBar(viewTabBar->isOn());
 	g_pConfig->SetToolBar(viewToolBar->isOn());
 	g_pConfig->SetStatusBar(viewStatusBar->isOn());
 	g_pConfig->SetAppGeometry( width(), height() );
@@ -1302,6 +1361,37 @@ bool DCGuiApp::slotFileQuit()
 	(*map)["WIDTH"]  = QString().setNum(pTransferViewDock->width());
 	(*map)["HEIGHT"] = QString().setNum(pTransferViewDock->height());
 	(*map)["DOCKED"] = QString().setNum((pTransferViewDock->area()!=0));
+	
+	Qt::Dock dock3;
+	int index3;
+	bool nl3;
+	int extraOffset3;
+	if ( getLocation( pTransferViewDock, dock3, index3, nl3, extraOffset3 ) == TRUE )
+	{
+		(*map)["DOCK"] = QString().setNum(dock3);
+		
+		QString nlstring;
+		if (nl3 == TRUE)
+		{
+			nlstring = "TRUE";
+		}
+		else
+		{
+			nlstring = "FALSE";
+		}
+		
+		(*map)["NL"] = nlstring;
+		(*map)["INDEX"] = QString().setNum(index3);
+		(*map)["EXTRAOFFSET"] = QString().setNum(extraOffset3);
+	}	
+
+	// save tabbar position and state
+	g_pConfig->GetMap("TABBAR", tabBarMap);
+	(*tabBarMap)["X"]	= QString().setNum(windowsTabBar->x());
+	(*tabBarMap)["Y"]	= QString().setNum(windowsTabBar->y());
+	(*tabBarMap)["WIDTH"]	= QString().setNum(windowsTabBar->width());
+	(*tabBarMap)["HEIGHT"]	= QString().setNum(windowsTabBar->height());
+	(*tabBarMap)["DOCKED"]	= QString().setNum((windowsTabBar->area()!=0));
 
 	g_pConfig->SetShowTransferWin( pTransferViewDock->isVisible() );
 
@@ -1334,6 +1424,30 @@ bool DCGuiApp::slotFileQuit()
 	{
 		(*map)["DOCK"] = QString().setNum(dock);
 	}
+	
+	Qt::Dock dock2;
+	int index2;
+	bool nl2;
+	int extraOffset2;
+	
+	if ( getLocation( windowsTabBar, dock2, index2, nl2, extraOffset2 ) == TRUE )
+	{
+		(*tabBarMap)["DOCK"] = QString().setNum(dock2);
+		
+		QString nlstring;
+		if (nl2 == TRUE)
+		{
+			nlstring = "TRUE";
+		}
+		else
+		{
+			nlstring = "FALSE";
+		}
+		
+		(*tabBarMap)["NL"] = nlstring;
+		(*tabBarMap)["INDEX"] = QString().setNum(index2);
+		(*tabBarMap)["EXTRAOFFSET"] = QString().setNum(extraOffset2);
+	}
 
 	// save gui config
 	g_pConfig->SaveDCGui();
@@ -1407,6 +1521,7 @@ void DCGuiApp::slotViewDock()
 {
 	if ( !m_pTrayIcon )
 	{
+		QMessageBox::information(this, tr("Dock Application"), tr("The system tray icon must be enabled to dock the application."));
 		return;
 	}
 	
@@ -1416,7 +1531,14 @@ void DCGuiApp::slotViewDock()
 
 	if ( m_bTray )
 	{
-		show();
+		if ( m_bWasMaximized )
+		{
+			showMaximized();
+		}
+		else
+		{
+			show();
+		}
 		// show dcgui in front of any applications after undock
 		raise();
 		// restore desktop position
@@ -1440,6 +1562,8 @@ void DCGuiApp::slotViewDock()
 		{
 			// save old desktop position
 			m_DesktopPosition = pos();
+			// save old maximized status
+			m_bWasMaximized = isMaximized();
 			// hide application
 			hide();
 		
@@ -1470,6 +1594,7 @@ void DCGuiApp::slotHelpAbout()
 	if ( f.open(IO_ReadOnly) == TRUE )
 	{
 		QTextStream t( &f );
+		t.setEncoding(QTextStream::UnicodeUTF8);
 		dlg->TextEdit_LICENSE->setText( t.read() );
 	}
 	else
@@ -1490,7 +1615,7 @@ void DCGuiApp::slotHelpManual()
 		QProcess * pr = new QProcess();
 		QByteArray buf;
 		pr->addArgument(g_pConfig->GetBrowser().Data());
-		pr->addArgument("http://dcgui.berlios.de/doc/manual.html");
+		pr->addArgument("http://wxdcgui.sourceforge.net/doc/manual.html");
 		pr->launch(buf);
 		delete pr;
 	}
@@ -1662,8 +1787,16 @@ void DCGuiApp::showMDIWidget( QWidget * widget )
 		}
 		else if ( widget->isVisible() )
 		{
-			widget->setEnabled(FALSE);
-			widget->hide();
+			if ( pWorkspace->activeWindow() == widget )
+			{
+				widget->setEnabled(FALSE);
+				widget->hide();
+			}
+			else
+			{
+				widget->setEnabled(TRUE);
+				widget->setFocus();
+			}
 		}
 		else if ( widget->isMaximized() )
 		{
@@ -1721,11 +1854,25 @@ void DCGuiApp::slotActionUsers()
 /** show/hide update manager window */
 void DCGuiApp::slotActionUpdateManager()
 {
-	statusBar()->message(tr("Show/hide update manager window ..."));
-
-	showMDIWidget(g_pUpdateManager);
-
-	statusBar()->message(tr("Ready."));
+	if ( g_pUpdateManager != 0 )
+	{
+		statusBar()->message(tr("Show/hide update manager window ..."));
+		
+		showMDIWidget(g_pUpdateManager);
+		
+		statusBar()->message(tr("Ready."));
+	}
+	else
+	{
+		QMessageBox::information(
+			this,
+			tr("Update Manager"),
+			tr("The update manager is disabled."),
+			QMessageBox::Ok,
+			QMessageBox::NoButton,
+			QMessageBox::NoButton
+		);
+	}
 }
 
 /** show the tray menu icon */
