@@ -233,11 +233,13 @@ void DCHubListManager::InitDocument()
 	connect( ToolButton_CONNECT, SIGNAL(clicked()), this, SLOT(slotConnect()) );
 
 	// public list
-	connect( ListView_PUBLIC,SIGNAL(doubleClicked(QListViewItem*)), this, SLOT(slotDoubleClicked(QListViewItem*)) );
+	connect( ListView_PUBLIC,SIGNAL(doubleClicked(QListViewItem*, const QPoint &, int)), this, SLOT(slotDoubleClicked(QListViewItem*, const QPoint &, int)) );
+	connect( ListView_PUBLIC,SIGNAL(returnPressed(QListViewItem*)), this, SLOT(slotReturnPressed(QListViewItem*)) );
 	connect( ListView_PUBLIC,SIGNAL(contextMenuRequested( QListViewItem *, const QPoint &, int )), this, SLOT(slotRightButtonClickedPublicList(QListViewItem*, const QPoint &, int )) );
 
 	// bookmark list
-	connect( ListView_BOOKMARKS,SIGNAL(doubleClicked(QListViewItem*)), this, SLOT(slotDoubleClicked(QListViewItem*)) );
+	connect( ListView_BOOKMARKS,SIGNAL(doubleClicked(QListViewItem*, const QPoint &, int)), this, SLOT(slotDoubleClicked(QListViewItem*, const QPoint &, int)) );
+	connect( ListView_BOOKMARKS,SIGNAL(returnPressed(QListViewItem*)), this, SLOT(slotReturnPressed(QListViewItem*)) );
 	connect( ListView_BOOKMARKS,SIGNAL(contextMenuRequested( QListViewItem *, const QPoint &, int )), this, SLOT(slotRightButtonClickedBookmarkList(QListViewItem*, const QPoint &, int )) );
 
 	connect( TabWidget_SERVERLIST,SIGNAL(currentChanged(QWidget*)), this, SLOT(slotTabWidgetCurrentChange(QWidget*)) );
@@ -301,8 +303,8 @@ void DCHubListManager::DeInitDocument()
 	// save hub view settings
 	g_pConfig->GetMap("HUBVIEW",map);
 
-	(*map)["X"]          = QString().setNum(x());
-	(*map)["Y"]          = QString().setNum(y());
+	(*map)["X"]          = QString().setNum(parentWidget()->x());
+	(*map)["Y"]          = QString().setNum(parentWidget()->y());
 	(*map)["WIDTH"]      = QString().setNum(width());
 	(*map)["HEIGHT"]     = QString().setNum(height());
 	(*map)["VISIBLE"]    = QString().setNum(isVisible());
@@ -571,9 +573,9 @@ void DCHubListManager::ShowPublicHubList()
 			item->myvalue = hubitem->m_sUserCount.asULL();
 			item->mycol = 3;
 
-			item->setText(0,hubitem->m_sName.Data());
-			item->setText(1,hubitem->m_sHost.Data());
-			item->setText(2,hubitem->m_sDescription.Data());
+			item->setText(0,QString::fromUtf8(hubitem->m_sName.Data()));
+			item->setText(1,QString::fromUtf8(hubitem->m_sHost.Data()));
+			item->setText(2,QString::fromUtf8(hubitem->m_sDescription.Data()));
 			item->setText(3,hubitem->m_sUserCount.Data());
 		}
 	}
@@ -634,7 +636,14 @@ void DCHubListManager::slotUpdatePublicHubList()
 }
 
 /** */
-void DCHubListManager::slotDoubleClicked( QListViewItem * item )
+void DCHubListManager::slotDoubleClicked( QListViewItem * item, const QPoint &, int )
+{
+	if ( item )
+		g_pConnectionManager->Connect( item->text(0).ascii(), item->text(1).ascii() );
+}
+
+/** */
+void DCHubListManager::slotReturnPressed( QListViewItem * item )
 {
 	if ( item )
 		g_pConnectionManager->Connect( item->text(0).ascii(), item->text(1).ascii() );
@@ -1070,6 +1079,9 @@ void DCHubListManager::slotRightButtonClickedBookmarkList( QListViewItem * /*ite
 			RemoveBookmark(curitem);
 		}
 		
+		// save changes to profile list, some may have been deleted
+		g_pConfig->SaveHubProfile();
+		
 		// update myinfo
 		g_pConnectionManager->SendMyInfoToConnectedServers();
 	}
