@@ -65,6 +65,17 @@
 #include <dclib/core/cxml.h>
 #include <dclib/chttp.h>
 
+/* column numbers for the search results listview */
+#define RESULTS_COLUMN_FILE		0
+#define RESULTS_COLUMN_SIZE		1
+#define RESULTS_COLUMN_TTH		2
+#define RESULTS_COLUMN_NICK		3
+#define RESULTS_COLUMN_FREESLOTS	4
+#define RESULTS_COLUMN_TOTALSLOTS	5
+#define RESULTS_COLUMN_HUB		6
+#define RESULTS_COLUMN_PATH		7
+#define RESULTS_COLUMN_HOST		8
+
 DCHubSearch * g_pHubSearch = 0;
 
 /** */
@@ -170,6 +181,8 @@ void DCHubSearch::InitDocument()
 	
 	connect( Combobox_SEARCH, SIGNAL(activated(int)), this, SLOT(slotSearchSelected(int)) );
 	
+	connect( PushButton_PURGEHISTORY, SIGNAL(clicked()), this, SLOT(slotPurgeHistory()) );
+	
 	m_bSearchState = FALSE;
 
 	connect( &m_Timer, SIGNAL(timeout()), this, SLOT(timerDone()) );
@@ -187,8 +200,8 @@ void DCHubSearch::DeInitDocument()
 	// save search view settings
 	g_pConfig->GetMap("SEARCHVIEW",map);
 
-	(*map)["X"]         = QString().setNum(x());
-	(*map)["Y"]         = QString().setNum(y());
+	(*map)["X"]         = QString().setNum(parentWidget()->x());
+	(*map)["Y"]         = QString().setNum(parentWidget()->y());
 	(*map)["WIDTH"]     = QString().setNum(width());
 	(*map)["HEIGHT"]    = QString().setNum(height());
 	(*map)["VISIBLE"]   = QString().setNum(isVisible());
@@ -226,14 +239,15 @@ void DCHubSearch::ResizeListViewColumn()
 	{
 		width = ListView_SEARCHRESULT->width();
 
-		ListView_SEARCHRESULT->setColumnWidth( 0, ((width*7)/32) );
-		ListView_SEARCHRESULT->setColumnWidth( 1, ((width*3)/32) );
-		ListView_SEARCHRESULT->setColumnWidth( 2, ((width*4)/32) );
-		ListView_SEARCHRESULT->setColumnWidth( 3, ((width*4)/32) );
-		ListView_SEARCHRESULT->setColumnWidth( 4, ((width*1)/32) );
-		ListView_SEARCHRESULT->setColumnWidth( 5, ((width*4)/32) );
-		ListView_SEARCHRESULT->setColumnWidth( 6, ((width*6)/32) );
-		ListView_SEARCHRESULT->setColumnWidth( 7, ((width*3)/32) );
+		ListView_SEARCHRESULT->setColumnWidth( RESULTS_COLUMN_FILE, ((width*7)/37) );
+		ListView_SEARCHRESULT->setColumnWidth( RESULTS_COLUMN_SIZE, ((width*3)/37) );
+		ListView_SEARCHRESULT->setColumnWidth( RESULTS_COLUMN_TTH, ((width*4)/37) );
+		ListView_SEARCHRESULT->setColumnWidth( RESULTS_COLUMN_NICK, ((width*4)/37) );
+		ListView_SEARCHRESULT->setColumnWidth( RESULTS_COLUMN_FREESLOTS, ((width*3)/37) );
+		ListView_SEARCHRESULT->setColumnWidth( RESULTS_COLUMN_TOTALSLOTS, ((width*3)/37) );
+		ListView_SEARCHRESULT->setColumnWidth( RESULTS_COLUMN_HUB, ((width*4)/37) );
+		ListView_SEARCHRESULT->setColumnWidth( RESULTS_COLUMN_PATH, ((width*6)/37) );
+		ListView_SEARCHRESULT->setColumnWidth( RESULTS_COLUMN_HOST, ((width*3)/37) );
 	}
 
 	if ( ListView_SEARCHRESULTUSER->isVisible() )
@@ -708,16 +722,24 @@ bool DCHubSearch::AddSearchResult( CMessageSearchResult * MessageSearchResult, D
 	CDir().SplitPathFile(s.ascii(),spath,sname);
 
 	item->myvalue = MessageSearchResult->m_nSize;
-	item->mycol   = 1;
+	item->mycol   = RESULTS_COLUMN_SIZE;
 
-	item->setText(0,sname.Data());
-	item->setText(1,CUtils::GetSizeString(MessageSearchResult->m_nSize,g_pConfig->GetUnit()).Data());
-	item->setText(2,MessageSearchResult->m_sHash.Data());
-	item->setText(3,MessageSearchResult->m_sNick.Data());
-	item->setText(4,QString().setNum(MessageSearchResult->m_nFreeSlot)+'/'+QString().setNum(MessageSearchResult->m_nMaxSlot));
-	item->setText(5,MessageSearchResult->m_sHubName.Data());
-	item->setText(6,spath.Data());
-	item->setText(7,MessageSearchResult->m_sHubHost.Data());
+	item->setText(RESULTS_COLUMN_FILE,QString::fromUtf8(sname.Data()));
+	if (MessageSearchResult->m_bFolder)
+	{
+		item->setText(RESULTS_COLUMN_SIZE,tr("Folder"));
+	}
+	else
+	{
+		item->setText(RESULTS_COLUMN_SIZE,CUtils::GetSizeString(MessageSearchResult->m_nSize,g_pConfig->GetUnit()).Data());
+	}
+	item->setText(RESULTS_COLUMN_TTH,MessageSearchResult->m_sHash.Data());
+	item->setText(RESULTS_COLUMN_NICK,QString::fromUtf8(MessageSearchResult->m_sNick.Data()));
+	item->setText(RESULTS_COLUMN_FREESLOTS,QString().setNum(MessageSearchResult->m_nFreeSlot));
+	item->setText(RESULTS_COLUMN_TOTALSLOTS,QString().setNum(MessageSearchResult->m_nMaxSlot));
+	item->setText(RESULTS_COLUMN_HUB,QString::fromUtf8(MessageSearchResult->m_sHubName.Data()));
+	item->setText(RESULTS_COLUMN_PATH,QString::fromUtf8(spath.Data()));
+	item->setText(RESULTS_COLUMN_HOST,MessageSearchResult->m_sHubHost.Data());
 
 	item->setVisible(visible);
 	item->setEnabled(visible);
@@ -954,7 +976,7 @@ void DCHubSearch::UpdateHidden( bool SearchFileOnly, int FreeSlots, bool TextFil
 		bool visible = TRUE;
 		QString path_file;
 
-		path_file = iter.current()->text(6) + iter.current()->text(0);
+		path_file = iter.current()->text(RESULTS_COLUMN_PATH) + iter.current()->text(RESULTS_COLUMN_FILE);
 
 		if ( SearchFileOnly )
 		{
@@ -999,7 +1021,7 @@ void DCHubSearch::UpdateHidden( bool SearchFileOnly, int FreeSlots, bool TextFil
 		
 		if ( visible == TRUE )
 		{
-			if ( iter.current()->text(4).section("/",0,0).toInt() < FreeSlots )
+			if ( iter.current()->text(RESULTS_COLUMN_FREESLOTS).toInt() < FreeSlots )
 			{
 				visible = FALSE;
 			}
@@ -1332,18 +1354,19 @@ void DCHubSearch::slotDoubleClickedSearchResult( QListViewItem * /*item*/ )
 
 	curitem   = ListView_SEARCHRESULT->currentItem();
 	size      = ((DC_QListViewItem*)curitem)->myvalue;
-	localname = curitem->text(0);
+	localname = curitem->text(RESULTS_COLUMN_FILE);
 
 	// add file as new source
-	FileTool.AddFileSource( curitem->text(3).ascii(),
-				curitem->text(5).ascii(),
-				curitem->text(7).ascii(),
-				(curitem->text(6)+curitem->text(0)).ascii(),
+	FileTool.AddFileSource( curitem->text(RESULTS_COLUMN_NICK).ascii(),
+				curitem->text(RESULTS_COLUMN_HUB).ascii(),
+				curitem->text(RESULTS_COLUMN_HOST).ascii(),
+				(curitem->text(RESULTS_COLUMN_PATH)+curitem->text(RESULTS_COLUMN_FILE)).ascii(),
 				localname.ascii(),
 				"",
 				"",
 				eltFILE,
-				size );
+				size,
+				curitem->text(RESULTS_COLUMN_TTH).ascii() );
 
 	return;
 /*
@@ -1391,16 +1414,26 @@ void DCHubSearch::slotRightButtonClickedSearchResult( QListViewItem * /*item*/,
 	ulonglong size;
         QPtrList<QListViewItem> selitems;
         QListViewItem * curitem, * item;
-	bool isadmin;
+	bool isadmin, allowDL;
 	CXml xml;
 
 	// user have select items ?
 	selectedItems( ListView_SEARCHRESULT, selitems );
+	
+	// cannot download folders yet
+	allowDL = true;
+	for ( curitem = selitems.first(); curitem; curitem = selitems.next() )
+	{
+		if ( curitem->text(RESULTS_COLUMN_SIZE) == tr("Folder") )
+		{
+			allowDL = false;
+		}
+	}
 
 	// check if admin for admin-menu
 	for ( isadmin = FALSE, curitem = selitems.first(); curitem; curitem = selitems.next() )
 	{
-		if ( g_pConnectionManager->IsAdmin(curitem->text(5).ascii(),curitem->text(7).ascii()) == FALSE )
+		if ( g_pConnectionManager->IsAdmin(curitem->text(RESULTS_COLUMN_HUB).ascii(),curitem->text(RESULTS_COLUMN_HOST).ascii()) == FALSE )
 		{
 			isadmin = FALSE;
 			break;
@@ -1415,20 +1448,21 @@ void DCHubSearch::slotRightButtonClickedSearchResult( QListViewItem * /*item*/,
 
 	m = new QPopupMenu(this);
 
-	DCMenuHandler::InsertMenu( m, emiDOWNLOAD, (selitems.count() > 0) );
-	DCMenuHandler::InsertMenu( m, emiDOWNLOAD_TO, (selitems.count() > 0) );
-	DCMenuHandler::InsertMenu( m, emiDOWNLOAD_AS, (selitems.count() > 0) );
+	DCMenuHandler::InsertMenu( m, emiDOWNLOAD, (selitems.count() > 0) && allowDL );
+	DCMenuHandler::InsertMenu( m, emiDOWNLOAD_TO, (selitems.count() > 0) && allowDL );
+	DCMenuHandler::InsertMenu( m, emiDOWNLOAD_AS, (selitems.count() > 0) && allowDL );
 	DCMenuHandler::InsertMenu( m, emiSEPARATOR );
-	DCMenuHandler::InsertMenu( m, emiDOWNLOAD_REPAIR, (selitems.count() > 0) );
-	DCMenuHandler::InsertMenu( m, emiDOWNLOAD_BIN_SECTORS, (selitems.count() > 0) );
+	DCMenuHandler::InsertMenu( m, emiDOWNLOAD_REPAIR, (selitems.count() > 0) && allowDL );
+	DCMenuHandler::InsertMenu( m, emiDOWNLOAD_BIN_SECTORS, (selitems.count() > 0) && allowDL );
 	DCMenuHandler::InsertMenu( m, emiSEPARATOR );
-	DCMenuHandler::InsertMenu( m, emiDOWNLOAD_IN, (selitems.count() > 0) );
+	DCMenuHandler::InsertMenu( m, emiDOWNLOAD_IN, (selitems.count() > 0) && allowDL );
 	DCMenuHandler::InsertMenu( m, emiSEPARATOR );
 	DCMenuHandler::InsertMenu( m, emiBROWSE_USER_FILES, (selitems.count() > 0) );
 	DCMenuHandler::InsertMenu( m, emiSEPARATOR );
 	DCMenuHandler::InsertMenu( m, emiCOPY_COLUMN_TO_CLIPBOARD, (selitems.count() > 0) );
 	DCMenuHandler::InsertMenu( m, emiCOPY_ROW_TO_CLIPBOARD, (selitems.count() > 0) );
 	DCMenuHandler::InsertMenu( m, emiCOPYDCFILELINK, (selitems.count() == 1) );
+	DCMenuHandler::InsertMenu( m, emiCOPYMAGNETLINK, (selitems.count() > 0) );
 	DCMenuHandler::InsertMenu( m, emiSEPARATOR );
 	DCMenuHandler::InsertMenu( m, emiCONNECT_TO_HUB, (selitems.count() > 0) );
 	DCMenuHandler::InsertMenu( m, emiADD_BOOKMARK, (selitems.count() > 0) );
@@ -1465,21 +1499,49 @@ void DCHubSearch::slotRightButtonClickedSearchResult( QListViewItem * /*item*/,
                 for ( curitem = selitems.first(); curitem; curitem = selitems.next() )
                 {
 			s += "DCFILE://";
-			s += xml.ToUTF8(curitem->text(7).ascii()) + "?";
-			s += "file=" + CHttp::Encode((curitem->text(6) + curitem->text(0)).ascii()) + "&";
+			s += xml.ToUTF8(curitem->text(RESULTS_COLUMN_HOST).ascii()) + "?";
+			s += "file=" + CHttp::Encode((curitem->text(RESULTS_COLUMN_PATH) + curitem->text(RESULTS_COLUMN_FILE)).ascii()) + "&";
 			s += "size=" + CString().setNum(((DC_QListViewItem*)curitem)->myvalue) + "&";
-			s += "nick=" + CHttp::Encode(curitem->text(3).ascii()) + "&";
-			s += "hub="  + CHttp::Encode(curitem->text(5).ascii());
+			s += "nick=" + CHttp::Encode(curitem->text(RESULTS_COLUMN_NICK).ascii()) + "&";
+			s += "hub="  + CHttp::Encode(curitem->text(RESULTS_COLUMN_HUB).ascii());
 			
-			if ( curitem->text(2) != "" )
+			if ( curitem->text(RESULTS_COLUMN_TTH) != "" )
 			{
 				s += "&";
-				s += "hash="  + CHttp::Encode(curitem->text(2).ascii());
+				s += "hash="  + CHttp::Encode(curitem->text(RESULTS_COLUMN_TTH).ascii());
 			}
 		}
 
 		cb->setText(s.Data());
 	}
+	else if ( id == emiCOPYMAGNETLINK )
+	{
+		//magnet:?xt=urn:tree:tiger:EOSA5AGTL5SD3VWCF3R2OH2WMGXV3S3R7SYN4YA&xl=708780032&dn=FC-6-i386-disc1.iso
+		QClipboard *cb = QApplication::clipboard();
+		QString text = "";
+		
+		for ( curitem = selitems.first(); curitem; curitem = selitems.next() )
+		{
+			text += "magnet:?xt=urn:tree:tiger:";
+			text += curitem->text(2).mid(4);
+			
+			// cannot do size since no exact size column in search results dialog
+			//text += "&xl=";
+			//text += curitem->text();
+			
+			text += "&dn=";
+			QString filename = curitem->text(0);
+			QUrl::encode(filename);
+			text += filename;
+			
+			text += "\n";
+		}
+		
+		// remove trailing "\n"
+		text = text.stripWhiteSpace();
+		
+		cb->setText(text);
+	}
 	else if ( (id == emiDOWNLOAD) || (id == emiDOWNLOAD_AS) || (id == emiDOWNLOAD_TO)
 		|| (id == emiDOWNLOAD_REPAIR) || (id == emiDOWNLOAD_BIN_SECTORS) )
 	{
@@ -1498,7 +1560,7 @@ void DCHubSearch::slotRightButtonClickedSearchResult( QListViewItem * /*item*/,
                 for ( curitem = selitems.first(); curitem; curitem = selitems.next() )
                 {
 			size = ((DC_QListViewItem *)curitem)->myvalue;
-			localname = curitem->text(0);
+			localname = curitem->text(RESULTS_COLUMN_FILE);
 
 			if ( (id == emiDOWNLOAD_AS) || (id == emiDOWNLOAD_REPAIR) || (id == emiDOWNLOAD_BIN_SECTORS) )
 			{
@@ -1532,9 +1594,9 @@ void DCHubSearch::slotRightButtonClickedSearchResult( QListViewItem * /*item*/,
 					break;
 				}
 
-			filetool.CheckFile( curitem->text(3).ascii(), curitem->text(5).ascii(), curitem->text(7).ascii(),
-				(curitem->text(6)+curitem->text(0)).ascii(), localname.ascii(), "", localrootpath.ascii(), eltFILE,
-				size, curitem->text(2).ascii(), FALSE, repair );
+			filetool.CheckFile( curitem->text(RESULTS_COLUMN_NICK).ascii(), curitem->text(RESULTS_COLUMN_HUB).ascii(), curitem->text(RESULTS_COLUMN_HOST).ascii(),
+				(curitem->text(RESULTS_COLUMN_PATH)+curitem->text(RESULTS_COLUMN_FILE)).ascii(), localname.ascii(), "", localrootpath.ascii(), eltFILE,
+				size, curitem->text(RESULTS_COLUMN_TTH).ascii(), FALSE, repair );
 		}
 
 		// redraw the list
@@ -1566,9 +1628,9 @@ void DCHubSearch::slotRightButtonClickedSearchResult( QListViewItem * /*item*/,
                 for ( curitem = selitems.first(); curitem; curitem = selitems.next() )
                 {
 			// add transfer to the waitlist
-			filetool.CheckFile( curitem->text(3).ascii(), curitem->text(5).ascii(), curitem->text(7).ascii(),
-					  (curitem->text(6)+curitem->text(0)).ascii(), localname.ascii(), localpath.ascii(), localrootpath.ascii(), eltFILE,
-					  size, curitem->text(2).ascii(), TRUE);
+			filetool.CheckFile( curitem->text(RESULTS_COLUMN_NICK).ascii(), curitem->text(RESULTS_COLUMN_HUB).ascii(), curitem->text(RESULTS_COLUMN_HOST).ascii(),
+					  (curitem->text(RESULTS_COLUMN_PATH)+curitem->text(RESULTS_COLUMN_FILE)).ascii(), localname.ascii(), localpath.ascii(), localrootpath.ascii(), eltFILE,
+					  size, curitem->text(RESULTS_COLUMN_TTH).ascii(), TRUE);
 		}
 
 		// redraw the list
@@ -1579,8 +1641,8 @@ void DCHubSearch::slotRightButtonClickedSearchResult( QListViewItem * /*item*/,
 		/** add transfer to the waitlist */
                 for ( curitem = selitems.first(); curitem; curitem = selitems.next() )
                 {
-			CString hubname(curitem->text(5).ascii());
-			CString hubhost(curitem->text(7).ascii());
+			CString hubname(curitem->text(RESULTS_COLUMN_HUB).ascii());
+			CString hubhost(curitem->text(RESULTS_COLUMN_HOST).ascii());
 
 			if ( g_pConnectionManager->IsHubOnline(hubname,hubhost) == ehsNONE )
 			{
@@ -1597,7 +1659,7 @@ void DCHubSearch::slotRightButtonClickedSearchResult( QListViewItem * /*item*/,
 				}
 			}
 
-			g_pTransferView->DLM_QueueAdd( curitem->text(3).ascii(), curitem->text(5).ascii(), curitem->text(7).ascii(),
+			g_pTransferView->DLM_QueueAdd( curitem->text(RESULTS_COLUMN_NICK).ascii(), curitem->text(RESULTS_COLUMN_HUB).ascii(), curitem->text(RESULTS_COLUMN_HOST).ascii(),
 						DC_USER_FILELIST_HE3, DC_USER_FILELIST_HE3, "", "", eltBUFFER,
 						0, 0, 0, "" );
 		}
@@ -1639,7 +1701,7 @@ void DCHubSearch::slotRightButtonClickedSearchResult( QListViewItem * /*item*/,
 		// connect to the hub
                 for ( curitem = selitems.first(); curitem; curitem = selitems.next() )
                 {
-			g_pConnectionManager->Connect( curitem->text(5).ascii(), curitem->text(7).ascii() );
+			g_pConnectionManager->Connect( curitem->text(RESULTS_COLUMN_HUB).ascii(), curitem->text(RESULTS_COLUMN_HOST).ascii() );
 		}
 	}
 	else if ( id == emiADD_BOOKMARK )
@@ -1647,7 +1709,7 @@ void DCHubSearch::slotRightButtonClickedSearchResult( QListViewItem * /*item*/,
 		// bookmark this hub
                 for ( curitem = selitems.first(); curitem; curitem = selitems.next() )
                 {
-			g_pHubListManager->AddBookmark( curitem->text(5), curitem->text(7), "" );
+			g_pHubListManager->AddBookmark( curitem->text(RESULTS_COLUMN_HUB), curitem->text(RESULTS_COLUMN_HOST), "" );
 		}
 	}
 	else if ( id == emiDISABLE_GROUP )
@@ -1703,14 +1765,14 @@ void DCHubSearch::slotRightButtonClickedSearchResult( QListViewItem * /*item*/,
 			selitems.remove(item);
 
 			// kick this user
-			g_pConnectionManager->OPKick( item->text(5), item->text(7), item->text(3), message );
+			g_pConnectionManager->OPKick( item->text(RESULTS_COLUMN_HUB), item->text(RESULTS_COLUMN_HOST), item->text(RESULTS_COLUMN_NICK), message );
 
 			// search for same entry and remove it
 			for ( curitem = selitems.first(); curitem; )
 			{
-				if ( (curitem->text(3) == item->text(3)) &&
-				     (curitem->text(5) == item->text(5)) &&
-				     (curitem->text(7) == item->text(7)) )
+				if ( (curitem->text(RESULTS_COLUMN_NICK) == item->text(RESULTS_COLUMN_NICK)) &&
+				     (curitem->text(RESULTS_COLUMN_HUB) == item->text(RESULTS_COLUMN_HUB)) &&
+				     (curitem->text(RESULTS_COLUMN_HOST) == item->text(RESULTS_COLUMN_HOST)) )
 				{
 					selitems.remove(curitem);
 					curitem = selitems.current();
@@ -1739,14 +1801,14 @@ void DCHubSearch::slotRightButtonClickedSearchResult( QListViewItem * /*item*/,
 			selitems.remove(item);
 
 			// force move this user
-			g_pConnectionManager->OPForceMove( item->text(5), item->text(7), item->text(3), message, host );
+			g_pConnectionManager->OPForceMove( item->text(RESULTS_COLUMN_HUB), item->text(RESULTS_COLUMN_HOST), item->text(RESULTS_COLUMN_NICK), message, host );
 
 			// search for same entry and remove it
 			for ( curitem = selitems.first(); curitem; )
 			{
-				if ( (curitem->text(3) == item->text(3)) &&
-				     (curitem->text(5) == item->text(5)) &&
-				     (curitem->text(7) == item->text(7)) )
+				if ( (curitem->text(RESULTS_COLUMN_NICK) == item->text(RESULTS_COLUMN_NICK)) &&
+				     (curitem->text(RESULTS_COLUMN_HUB) == item->text(RESULTS_COLUMN_HUB)) &&
+				     (curitem->text(RESULTS_COLUMN_HOST) == item->text(RESULTS_COLUMN_HOST)) )
 				{
 					selitems.remove(curitem);
 					curitem = selitems.current();
@@ -1899,3 +1961,11 @@ void DCHubSearch::RemoveSelectedSearch( QPtrList<QListViewItem> * list )
 		delete curitem;
 	}
 }
+
+/** Clears the history of search strings */
+void DCHubSearch::slotPurgeHistory()
+{
+	Combobox_SEARCH->clear();
+	this->slotReset();
+	m_pSearchHistory->Clear();
+}
