@@ -201,6 +201,10 @@ QPopupMenu * DCMenuHandler::_InsertMenu( QPopupMenu * popupmenu, eMenuItems menu
 				popupmenu->insertItem( QIconSet( g_pIconLoader->GetPixmap(eiFILEFIND) ), QObject::tr("Search for clones"), menuitems );
 				popupmenu->setItemEnabled( menuitems, enabled );
 				break;
+		case emiSEARCH_FILE_CLONE_TTH:
+				popupmenu->insertItem( QIconSet( g_pIconLoader->GetPixmap(eiFILEFIND) ), QObject::tr("Search for clones by TTH"), menuitems );
+				popupmenu->setItemEnabled( menuitems, enabled );
+				break;				
 		case emiUSER_CAPTION:
 				popupmenu->insertItem( text, emiUSER_CAPTION );
 				popupmenu->setItemEnabled( emiUSER_CAPTION, enabled );
@@ -293,6 +297,10 @@ QPopupMenu * DCMenuHandler::_InsertMenu( QPopupMenu * popupmenu, eMenuItems menu
 				popupmenu->insertItem( QIconSet( g_pIconLoader->GetPixmap(eiEDITCOPY) ), QObject::tr("Copy Link"), menuitems );
 				popupmenu->setItemEnabled( menuitems, enabled );
 				break;
+		case emiCOPYMAGNETLINK:
+				popupmenu->insertItem( QIconSet( g_pIconLoader->GetPixmap(eiEDITCOPY) ), QObject::tr("Copy magnet link"), menuitems );
+				popupmenu->setItemEnabled( menuitems, enabled );
+				break;
 		case emiCLEAR:
 				popupmenu->insertItem( QIconSet( g_pIconLoader->GetPixmap(eiEDITDELETE) ), QObject::tr("Clear"), menuitems );
 				popupmenu->setItemEnabled( menuitems, enabled );
@@ -414,23 +422,15 @@ QPopupMenu * DCMenuHandler::_InsertMenu( QPopupMenu * popupmenu, eMenuItems menu
 				popupmenu->insertItem( QIconSet( g_pIconLoader->GetPixmap(eiFLAG_SWEDEN) ), QObject::tr("Swedish"), menuitems );
 				popupmenu->setItemEnabled( menuitems, enabled );
 				break;
-		case emiTEXT_INPUT_ENCODING:
-				popupmenu->insertItem( QIconSet( QPixmap() ), QObject::tr("Input"), menuitems );
-				popupmenu->setItemEnabled( menuitems, enabled );
-				break;
-		case emiTEXT_OUTPUT_ENCODING:
-				popupmenu->insertItem( QIconSet( QPixmap() ), QObject::tr("Output"), menuitems );
-				popupmenu->setItemEnabled( menuitems, enabled );
-				break;
 		// submenus
-		case emisTEXT_ENCODING:
-				menu = new QPopupMenu(popupmenu);
-				popupmenu->insertItem( QObject::tr("Text Encoding"), menu );
-				break;
 		case emisGROUP:
 				menu = new QPopupMenu(popupmenu);
 				popupmenu->insertItem( QObject::tr("Group"), menu );
 				break;
+		case emisCHAT_RIGHTCLICK_USER:
+				menu = new QPopupMenu(popupmenu);
+				popupmenu->insertItem( QObject::tr("User: ") + text, menu );
+				break;
 		case emiCUSTOM:
 				menu = new QPopupMenu(popupmenu);
 				popupmenu->insertItem( QObject::tr("Custom"), menu );
