@@ -45,6 +45,9 @@
 #include <qimage.h>
 #include <qcursor.h>
 #include <qpopupmenu.h>
+#include <qtextcodec.h>
+#include <qhttp.h>
+#include <qprocess.h>
 
 #include <cfloatspinbox.h>
 
@@ -70,6 +73,11 @@
 #include <dclib/cutils.h>
 #include <math.h>
 
+/* need VERSION for HTTP User-Agent */
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 /** */
 DCOptions::DCOptions()
 {
@@ -87,6 +95,10 @@ DCOptions::~DCOptions()
 /** */
 void DCOptions::InitDocument()
 {
+	QTextCodec *codec;
+	QStringList codecList;
+	int i;
+	
 	// insert speed
 	ComboBox_SPEED->insertItem( "28.8Kbps" );
 	ComboBox_SPEED->insertItem( "33.6Kbps" );
@@ -100,7 +112,48 @@ void DCOptions::InitDocument()
 	ComboBox_SPEED->insertItem( "LAN(T3)" );
 	ComboBox_SPEED->insertItem( "Microwave" );
 	ComboBox_SPEED->insertItem( "Wireless" );
+	ComboBox_SPEED->insertItem( "0.005" );
+	ComboBox_SPEED->insertItem( "0.01" );
+	ComboBox_SPEED->insertItem( "0.02" );
+	ComboBox_SPEED->insertItem( "0.05" );
+	ComboBox_SPEED->insertItem( "0.1" );
+	ComboBox_SPEED->insertItem( "0.2" );
+	ComboBox_SPEED->insertItem( "0.5" );
+	ComboBox_SPEED->insertItem( "1" );
+	ComboBox_SPEED->insertItem( "2" );
+	ComboBox_SPEED->insertItem( "5" );
+	ComboBox_SPEED->insertItem( "10" );
+	ComboBox_SPEED->insertItem( "20" );
+	ComboBox_SPEED->insertItem( "50" );
+	ComboBox_SPEED->insertItem( "100" );
+	
+	// remote filelist browser default encoding combo box
+	for (i=0;(codec = QTextCodec::codecForIndex(i));i++)
+	{
+		codecList << codec->name();
+	}
 
+	codecList.sort();
+	ComboBox_DEFAULTRENCODING->insertItem("DEFAULT");
+	ComboBox_DEFAULTRENCODING->insertStringList(codecList);
+	
+	ComboBox_REMOTE_ENCODING->insertItem("UTF-8");
+	ComboBox_REMOTE_ENCODING->insertItem("WINDOWS-1252");
+	ComboBox_REMOTE_ENCODING->insertItem("WINDOWS-1251");
+	ComboBox_REMOTE_ENCODING->insertItem("WINDOWS-1256");
+	ComboBox_REMOTE_ENCODING->insertItem("WINDOWS-1257");
+	ComboBox_REMOTE_ENCODING->insertItem("ISO-8859-1");
+	ComboBox_REMOTE_ENCODING->insertItem("ISO-8859-2");
+	ComboBox_REMOTE_ENCODING->insertItem("ISO-8859-7");
+	ComboBox_REMOTE_ENCODING->insertItem("ISO-8859-8");
+	ComboBox_REMOTE_ENCODING->insertItem("ISO-8859-9");
+	ComboBox_REMOTE_ENCODING->insertItem("ISO-2022-JP");
+	ComboBox_REMOTE_ENCODING->insertItem("SHIFT-JIS");
+	ComboBox_REMOTE_ENCODING->insertItem("CP949");
+	ComboBox_REMOTE_ENCODING->insertItem("KOI8-R");
+	ComboBox_REMOTE_ENCODING->insertItem("GB18030");
+	ComboBox_REMOTE_ENCODING->insertItem("TIS-620");
+	
 	// set width mode
 	ListView_OPTIONTREE->setColumnWidthMode( 0, QListView::Maximum );
 	// hide the column header
@@ -113,8 +166,23 @@ void DCOptions::InitDocument()
 	SpinBox_MAXUPLOADRATE->setMaxValue(99999999);
 	SpinBox_MAXDOWNLOADRATE->setMinValue(0);
 	SpinBox_MAXDOWNLOADRATE->setMaxValue(99999999);
+	SpinBox_AUTOSEARCHINTERVAL->setMinValue(60);     // one minute
+	SpinBox_AUTOSEARCHINTERVAL->setMaxValue(604800); // one week
+	SpinBox_SMALLFILESIZE->setMinValue(1024);        // 1KiB
+	SpinBox_SMALLFILESIZE->setMaxValue(104857600);   // 100MiB
+	SpinBox_AUTORESPONSEDELAY->setMinValue(0);
+	SpinBox_AUTORESPONSEDELAY->setMaxValue(3600);
+
+	// set up icons theme lists
+	setupIconThemeComboBoxes();
 
 	// setup the option tree
+	plviEncoding = new QListViewItem( ListView_OPTIONTREE, 0 );
+	plviEncoding->setText(0, tr( "Encoding" ) );
+	
+	plviAutoResponse = new QListViewItem( ListView_OPTIONTREE, 0 );
+	plviAutoResponse->setText(0, tr( "Auto Responses" ) );
+	
 	plviSecurity = new QListViewItem( ListView_OPTIONTREE, 0 );
 	plviSecurity->setText( 0, tr( "Security" ) );
 
@@ -130,8 +198,16 @@ void DCOptions::InitDocument()
 	plviGUI = new QListViewItem( ListView_OPTIONTREE, 0 );
 	plviGUI->setText( 0, tr( "GUI" ) );
 	plviGUI->setOpen( true );
+	plviGUIFilelistBrowser = new QListViewItem( plviGUI, 0 );
+	plviGUIFilelistBrowser->setText(0, tr( "Filelist browser" ) );
+	plviGUIIcons =  new QListViewItem( plviGUI, 0 );
+	plviGUIIcons->setText(0, tr ( "Icons" ) );
 	plviGUIUser = new QListViewItem( plviGUI, 0 );
 	plviGUIUser->setText( 0, tr( "User" ) );
+	plviGUIMenus = new QListViewItem( plviGUI, 0 );
+	plviGUIMenus->setText(0, tr( "Menus" ) );
+	plviGUICommands = new QListViewItem( plviGUI, 0 );
+	plviGUICommands->setText( 0, tr( "Commands" ) );
 	plviGUIChat = new QListViewItem( plviGUI, 0 );
 	plviGUIChat->setText( 0, tr( "Chat" ) );
 	plviGUITransfer = new QListViewItem( plviGUI, 0 );
@@ -164,7 +240,19 @@ void DCOptions::InitDocument()
 	connect( ToolButton_PHOTO, SIGNAL(clicked()), this, SLOT(slotChangePhoto()) );
 
 	connect( ListView_OPTIONTREE, SIGNAL(clicked( QListViewItem * )), this, SLOT(slotListViewOptionClicked( QListViewItem * )) );
-
+	
+	connect( listView_MENU_COMMANDS, SIGNAL(contextMenuRequested( QListViewItem *, const QPoint &, int )), this, SLOT(slotContextMenuUserMenuCommands(QListViewItem *, const QPoint &, int)));
+	listView_MENU_COMMANDS->setDefaultRenameAction( QListView::Accept );
+	listView_MENU_COMMANDS->setShowSortIndicator( TRUE );
+	
+	connect( listView_CHAT_COMMANDS, SIGNAL(contextMenuRequested( QListViewItem *, const QPoint &, int )), this, SLOT(slotContextMenuUserChatCommands(QListViewItem *, const QPoint &, int)));
+	listView_CHAT_COMMANDS->setDefaultRenameAction( QListView::Accept );
+	listView_CHAT_COMMANDS->setShowSortIndicator( TRUE );
+	
+	connect( listView_AUTO_RESPONSES, SIGNAL(contextMenuRequested( QListViewItem *, const QPoint &, int)), this, SLOT(slotContextMenuAutoResponses(QListViewItem *, const QPoint &, int)));
+	listView_AUTO_RESPONSES->setDefaultRenameAction( QListView::Accept );
+	listView_AUTO_RESPONSES->setShowSortIndicator( TRUE );
+	
 	connect( ToolButton_BROWSESOUNDFILECONNECT, SIGNAL(clicked()), this, SLOT(slotBrowseSoundFileConnect()) );
 	connect( ToolButton_BROWSESOUNDFILEDISCONNECT, SIGNAL(clicked()), this, SLOT(slotBrowseSoundFileDisconnect()) );
 	connect( ToolButton_BROWSESOUNDFILESEND, SIGNAL(clicked()), this, SLOT(slotBrowseSoundFileSend()) );
@@ -189,6 +277,7 @@ void DCOptions::InitDocument()
 	connect( ToolButton_ADDSHAREDFOLDER, SIGNAL(clicked()), this, SLOT(slotAddSharedFolder()) );
 	connect( ToolButton_REMOVESHAREDFOLDER, SIGNAL(clicked()), this, SLOT(slotRemoveSharedFolder()) );
 	connect( ToolButton_EDITSHAREDFOLDER, SIGNAL(clicked()), this, SLOT(slotEditSharedFolder()) );
+	connect( CheckBox_DONTSHAREDOTFILES, SIGNAL(clicked()), this, SLOT(slotDontShareDotFilesChanged()) );
 
 	//
 	connect( PushButton_SAVE, SIGNAL(clicked()), this, SLOT(slotSave()) );
@@ -242,7 +331,11 @@ void DCOptions::InitDocument()
 	connect( CheckBox_SOUNDPLAYFIRSTRECEIVE, SIGNAL(clicked()), this, SLOT(slotSoundPlayFirstReceive()) );
 
 	connect( ToolButton_BROWSELOGFILE, SIGNAL(clicked()), this, SLOT(slotBrowseLogFile()) );
-
+	
+	connect( CheckBox_CHATSHOWJOINSANDPARTS, SIGNAL(clicked()), this, SLOT(slotEnableShowJoinsParts()) );
+	
+	connect( PushButton_GENERATE_CERT, SIGNAL(clicked()), this, SLOT(slotGenerateKeyCert()) );
+	
 	// style options
 	ComboBox_THEME->clear();
 
@@ -441,8 +534,11 @@ void DCOptions::slotDoubleClickedChatColor( QListBoxItem * item )
 	QPixmap p(10,10);
 	QColor color(item->pixmap()->convertToImage().pixel(0,0));
 	color = QColorDialog::getColor(color);
-	p.fill(color);
-	ListBox_CHATCOLOR->changeItem( p, item->text(), ListBox_CHATCOLOR->currentItem() );
+	if ( color.isValid() )
+	{
+		p.fill(color);
+		ListBox_CHATCOLOR->changeItem( p, item->text(), ListBox_CHATCOLOR->currentItem() );
+	}
 }
 
 /** */
@@ -612,39 +708,42 @@ void DCOptions::slotGetInterface()
 /** */
 void DCOptions::slotGetInternetIP()
 {
-	QString ip="";
-	DCGetURL * d;
+	PushButton_GETINTERNETIP->setEnabled(FALSE);
+	
+	QHttpRequestHeader header( "GET", "/" );
+	header.setValue( "Host", "whatismyip.org" );
+	QString useragent = QString("Valknut/") + VERSION;
+	//printf("User-Agent=%s\n", useragent.ascii());
+	header.setValue( "User-Agent", useragent );
+	
+	http = new QHttp();
+	connect( http, SIGNAL(done(bool)), this, SLOT(slotGotInternetIP(bool)) );
+	http->setHost( "whatismyip.org" );
+	http->request( header );
+}
 
-	// http://whatismyip.org/
-	d = new DCGetURL("http://dcgui.berlios.de/ip.php");
-
-	if ( d->exec() == QDialog::Accepted )
+/** */
+void DCOptions::slotGotInternetIP( bool error )
+{
+	if (error == TRUE)
 	{
-		CString s;
-		CByteArray in;
-		d->GetData(&in);
-		s.Set((char*)in.Data(),in.Size());
-
-		if ( s != "" )
-		{
-			ip = s.Data();
-		}
-
-		ip = ip.replace( QRegExp("<ip>"), "" );
-		ip = ip.replace( QRegExp("</ip>"), "" );
-		
-		if ( ip == "" )
-		{
-			QMessageBox::about(this,tr("Latest release..."),
-					tr("Can't get your ip") );
-		}
-		else
-		{
-			LineEdit_IP->setText(ip);
-		}
+		QMessageBox mb ( tr("Error getting IP"),
+				 http->errorString(),
+				 QMessageBox::Warning,
+				 QMessageBox::Ok,
+				 0,
+				 0
+		);
+		mb.exec();
 	}
-
-	delete d;
+	else
+	{
+		LineEdit_IP->setText(QString(http->readAll()));
+	}
+	
+	delete http;
+	
+	PushButton_GETINTERNETIP->setEnabled(TRUE);
 }
 
 /** */
@@ -1009,7 +1108,14 @@ void DCOptions::SetValues()
 {
 	int i;
 	CList<DCConfigShareFolder> list;
+	CList<DC_UserMenuCommand> umclist;
 	DCConfigShareFolder * csf;
+	DC_UserMenuCommand * umc;
+	QListViewItem * qlvitem;
+	CList<DC_UserChatCommand> ucclist;
+	DC_UserChatCommand * ucc;
+	CList<DC_AutoResponseObject> arlist;
+	DC_AutoResponseObject * aro = 0;
 
 	// identify
 	LineEdit_NICK->setText(g_pConfig->GetNick().Data());
@@ -1018,6 +1124,7 @@ void DCOptions::SetValues()
 	CheckBox_ANTISPAM->setChecked(g_pConfig->GetAntiSpam());
 	LineEdit_DESCRIPTION->setText(g_pConfig->GetDescription(TRUE).Data());
 	CheckBox_DESCRIPTION_TAG->setChecked(g_pConfig->GetDescriptionTag());
+	CheckBox_EXT_HUB_COUNT->setChecked(g_pConfig->GetUseExtendedHubCount());
 	LineEdit_AWAYPREFIX->setText(g_pConfig->GetAwayPrefix().Data());
 
 	SetUserPhoto(g_pConfig->GetUserPhotoFileName().Data());
@@ -1030,6 +1137,43 @@ void DCOptions::SetValues()
 			break;
 		}
 	}
+	
+	for(i=0;i<ComboBox_DEFAULTRENCODING->count();i++)
+	{
+		if ( ComboBox_DEFAULTRENCODING->text(i) == g_pConfig->GetDefaultFBRemoteEncoding() )
+		{
+			ComboBox_DEFAULTRENCODING->setCurrentItem(i);
+			break;
+		}
+	}
+	
+	for(i=0;i<ComboBox_APPLICATION_ICONS->count();i++)
+	{
+		if ( ComboBox_APPLICATION_ICONS->text(i) == g_pConfig->GetApplicationIconTheme() )
+		{
+			ComboBox_APPLICATION_ICONS->setCurrentItem(i);
+			break;
+		}
+	}
+	
+	for(i=0;i<ComboBox_EMOTICONS->count();i++)
+	{
+		if ( ComboBox_EMOTICONS->text(i) == g_pConfig->GetEmoticonTheme() )
+		{
+			ComboBox_EMOTICONS->setCurrentItem(i);
+			break;
+		}
+	}
+	
+	for(i=0;i<ComboBox_USERLIST_ICONS->count();i++)
+	{
+		if ( ComboBox_USERLIST_ICONS->text(i) == g_pConfig->GetUserListIconTheme() )
+		{
+			ComboBox_USERLIST_ICONS->setCurrentItem(i);
+			break;
+		}
+	}
+	
 
 	TextEdit_AWAYMESSAGE->setText(QString(g_pConfig->GetAwayMessage().Data()).replace( QRegExp("\\r"), "" ));
 
@@ -1047,8 +1191,28 @@ void DCOptions::SetValues()
 	{
 		new QListViewItem( ListView_SHAREDFOLDERS, csf->m_sPath.Data(), csf->m_sAlias.Data() );
 	}
-
+	
+	g_pConfig->GetUserMenuCommands(&umclist);
+	umc = 0;
+	while( (umc=umclist.Next(umc)) != 0 )
+	{
+		qlvitem = new QListViewItem( listView_MENU_COMMANDS, umc->m_sName.Data(), QString().setNum(umc->m_nContext), umc->m_sCommand.Data());
+		qlvitem->setRenameEnabled(0, TRUE);
+		qlvitem->setRenameEnabled(1, TRUE);
+		qlvitem->setRenameEnabled(2, TRUE);
+	}
+	
+	g_pConfig->GetUserChatCommands(&ucclist);
+	ucc = 0;
+	while( (ucc=ucclist.Next(ucc)) != 0 )
+	{
+		qlvitem = new QListViewItem( listView_CHAT_COMMANDS, ucc->m_sName.Data(), ucc->m_sCommand.Data() );
+		qlvitem->setRenameEnabled(0, TRUE);
+		qlvitem->setRenameEnabled(1, TRUE);
+	}
+	
 	CheckBox_AUTORECREATESHARELIST->setChecked(g_pConfig->GetAutoRecreateShareList());
+	CheckBox_DONTSHAREDOTFILES->setChecked(g_pConfig->GetDontShareDotFiles());
 
 	// transfer->settings
 	SpinBox_SAVEDOWNLOADQUEUETIME->setValue( g_pConfig->GetDownloadQueueTime() );
@@ -1059,6 +1223,8 @@ void DCOptions::SetValues()
 	SpinBox_MAXSIMULTANEOUSUPLOADS->setValue( g_pConfig->GetMaxUpload() );
 	SpinBox_TRANSFERRESENDTIMEOUT->setValue(g_pConfig->GetTransferResendTimeout());
 	SpinBox_TRANSFERRESPONSETIMEOUT->setValue(g_pConfig->GetTransferResponseTimeout());
+	SpinBox_AUTOSEARCHINTERVAL->setValue(g_pConfig->GetAutoSearchInterval());
+	SpinBox_SMALLFILESIZE->setValue(g_pConfig->GetSmallFileSize());
 
 	switch ( g_pConfig->GetHubOfflineTransferClose() )
 	{
@@ -1078,6 +1244,7 @@ void DCOptions::SetValues()
 
 	CheckBox_TRANSFERAUTOSEARCH->setChecked(g_pConfig->GetTransferAutoSearch());
 	CheckBox_DISABLEHASHLIST->setChecked(g_pConfig->GetDisableHashList());
+	CheckBox_COMPRESSEDTRANSFERS->setChecked(g_pConfig->GetCompressedTransfers());
 	SpinBox_RECREATESHARELISTTIME->setValue(g_pConfig->GetRecreateShareListTime());
 
 	// connection->mode
@@ -1169,11 +1336,13 @@ void DCOptions::SetValues()
 	CheckBox_EMOTICONS->setChecked(g_pConfig->GetEnableEmoticons());
 	CheckBox_SHOWCHATSENDBUTTON->setChecked(g_pConfig->GetShowChatSendButton());
 	CheckBox_CHATSHOWJOINSANDPARTS->setChecked(g_pConfig->GetChatShowJoinsAndParts());
+	CheckBox_CHATSHOWJOINSANDPARTSONLYFAV->setChecked(g_pConfig->GetChatShowJoinsAndPartsOnlyFav());
 	CheckBox_CHATRECVOFFLINEMESSAGES->setChecked(g_pConfig->GetChatRecvOfflineMessages());
 	CheckBox_CHATSENDOFFLINEMESSAGES->setChecked(g_pConfig->GetChatSendOfflineMessages());
 	SpinBox_CHATMAXPARAGRAPH->setValue(g_pConfig->GetChatMaxParagraph());
 	SpinBox_CHATMESSAGEMAXLINES->setValue( g_pConfig->GetChatMessageMaxLines() );
-
+	LineEdit_SUPPRESS_NICKS->setText( g_pConfig->GetSuppressedNicks() );
+	
 	for(i=0;i<ComboBox_SENDCHAT->count();i++)
 	{
 		if ( g_pConfig->GetSendChat() == CString(ComboBox_SENDCHAT->text(i).ascii()) )
@@ -1242,7 +1411,10 @@ void DCOptions::SetValues()
 	c.setNamedColor(g_pConfig->GetChatColor(eccPUBLICCHATMENICK).Data());
 	p.fill(c);
 	ListBox_CHATCOLOR->insertItem(p,tr("Public Chat ME Nick"));
-
+	c.setNamedColor(g_pConfig->GetChatColor(eccOPNICK).Data());
+	p.fill(c);
+	ListBox_CHATCOLOR->insertItem(p,tr("Operator Nick"));
+	
 	// gui->client
 	CheckBox_CLIENTWINDOWSMINIMIZED->setChecked(g_pConfig->GetOpenClientWindowsMinimized());
 	CheckBox_USERLISTRIGHTALIGNMENT->setChecked(g_pConfig->GetUserListRightAlignment());
@@ -1257,6 +1429,15 @@ void DCOptions::SetValues()
 	CheckBox_CLIENTCOLUMNS_LOCKPK->setChecked(g_pConfig->GetClientColumn(eclcLOCKPK));
 	CheckBox_CLIENTCOLUMNS_SUPPORTS->setChecked(g_pConfig->GetClientColumn(eclcSUPPORTS));
 	
+	if ( g_pConfig->GetDoubleClickAction() == ComboBox_DOUBLECLICK->text(1) )
+	{
+		ComboBox_DOUBLECLICK->setCurrentItem(1);
+	}
+	else
+	{
+		ComboBox_DOUBLECLICK->setCurrentItem(0);
+	}
+	
 	// sound
 	CheckBox_SOUNDDISABLED->setChecked(g_pConfig->GetSoundDisabled());
 	CheckBox_SOUNDDISABLEDAWAY->setChecked(g_pConfig->GetSoundDisabledAway());
@@ -1275,6 +1456,9 @@ void DCOptions::SetValues()
 
 	// log
 	CheckBox_LOGFILE->setChecked(g_pConfig->GetLogFile());
+	CheckBox_LOGDOWNLOADS->setChecked(g_pConfig->GetLogFinishedDownloads());
+	CheckBox_LOGUPLOADS->setChecked(g_pConfig->GetLogFinishedUploads());
+	CheckBox_LOGDETAILS->setChecked(g_pConfig->GetLogDetails());
 	LineEdit_LOGFILENAME->setText( g_pConfig->GetLogFileName().Data() );
 	CheckBox_TIMESTAMPPRIVATECHAT->setChecked(g_pConfig->GetTimeStamp(etsPRIVATECHAT));
 	CheckBox_TIMESTAMPHUBCHAT->setChecked(g_pConfig->GetTimeStamp(etsHUBCHAT));
@@ -1326,6 +1510,38 @@ void DCOptions::SetValues()
 	slotLogChatEnableLogging();
 	// init away
 	slotAutoAwayMode();
+	// init show joins parts only fav enabled/disabled
+	slotEnableShowJoinsParts();
+	
+	// auto response settings
+	CheckBox_ENABLE_AUTO_RESPONDER->setChecked(g_pConfig->GetAutoResponderEnabled());
+	CheckBox_ENABLE_AUTO_RESPONDER_FOR_PM->setChecked(g_pConfig->GetAutoResponderEnabledForPM());
+	SpinBox_AUTORESPONSEDELAY->setValue(g_pConfig->GetAutoResponseDelay());
+	LineEdit_AUTO_RESPONSE_IGNORE->setText(g_pConfig->GetAutoResponseIgnores().Data());
+	
+	// fill auto response list box
+	g_pConfig->GetAutoResponses(&arlist);
+	aro = 0;
+	while( (aro=arlist.Next(aro)) != 0)
+	{
+		QString cs;
+		if (aro->m_bCaseSensitive == TRUE)
+		{
+			cs = tr("Yes");
+		}
+		else
+		{
+			cs = tr("No");
+		}
+		qlvitem = new QListViewItem( listView_AUTO_RESPONSES, aro->m_sTrigger.Data(), cs, aro->m_sResponse.Data() );
+		qlvitem->setRenameEnabled(0, TRUE);
+		qlvitem->setRenameEnabled(2, TRUE);
+	}
+	
+	ComboBox_REMOTE_ENCODING->setCurrentText( g_pConfig->GetRemoteEncoding().Data() );
+	
+	CheckBox_FOLDERS_RIGHT_PANE->setChecked(g_pConfig->GetFoldersInRightPane());
+	CheckBox_SIZES_LEFT_PANE->setChecked(g_pConfig->GetFolderSizesInLeftPane());
 }
 
 /** */
@@ -1339,6 +1555,7 @@ void DCOptions::GetValues()
 	g_pConfig->SetDescription(LineEdit_DESCRIPTION->text().ascii());
 	g_pConfig->SetAwayPrefix(LineEdit_AWAYPREFIX->text().ascii());
 	g_pConfig->SetDescriptionTag(CheckBox_DESCRIPTION_TAG->isChecked());
+	g_pConfig->SetUseExtendedHubCount(CheckBox_EXT_HUB_COUNT->isChecked());
 	g_pConfig->SetEMail(LineEdit_EMAIL->text().ascii());
 	g_pConfig->SetAntiSpam(CheckBox_ANTISPAM->isChecked());
 	g_pConfig->SetCheckPrivateAddressSpace(CheckBox_CHECKPRIVATEADDRESSSPACE->isChecked());
@@ -1349,15 +1566,18 @@ void DCOptions::GetValues()
 	g_pConfig->SetChatMaxParagraph(SpinBox_CHATMAXPARAGRAPH->value());
 	g_pConfig->SetTransferAutoSearch(CheckBox_TRANSFERAUTOSEARCH->isChecked());
 	g_pConfig->SetDisableHashList(CheckBox_DISABLEHASHLIST->isChecked());
+	g_pConfig->SetCompressedTransfers(CheckBox_COMPRESSEDTRANSFERS->isChecked());
 
 	g_pConfig->SetUserPhotoFileName(ToolButton_PHOTO->textLabel().ascii());
 
 	g_pConfig->SetSpeed(ComboBox_SPEED->currentText().ascii());
+	g_pConfig->SetDefaultFBRemoteEncoding(ComboBox_DEFAULTRENCODING->currentText());
 	g_pConfig->SetDownloadFolder(LineEdit_DOWNLOADFOLDER->text().ascii());
 	g_pConfig->SetDownloadFinishedFolder(LineEdit_DOWNLOADFINISHEDFOLDER->text().ascii());
 	g_pConfig->SetValknutDataPath(LineEdit_DATAFOLDER->text());
 	
 	g_pConfig->SetAutoRecreateShareList(CheckBox_AUTORECREATESHARELIST->isChecked());
+	g_pConfig->SetDontShareDotFiles(CheckBox_DONTSHAREDOTFILES->isChecked());
 	g_pConfig->SetRecreateShareListTime(SpinBox_RECREATESHARELISTTIME->value());
 
 	g_pConfig->SetFloodOpKickMessage(LineEdit_FLOODOPKICKMESSAGE->text().replace( QRegExp("\\n"), "\xd\xa" ).ascii());
@@ -1368,11 +1588,21 @@ void DCOptions::GetValues()
 	g_pConfig->SetTransferKey(LineEdit_TRANSFERKEY->text().ascii());
 
 	SaveSharedFolders();
+	SaveUserMenuCommands();
+	SaveUserChatCommands();
+
+	g_pConfig->SetAutoResponderEnabled(CheckBox_ENABLE_AUTO_RESPONDER->isChecked());
+	g_pConfig->SetAutoResponderEnabledForPM(CheckBox_ENABLE_AUTO_RESPONDER_FOR_PM->isChecked());
+	g_pConfig->SetAutoResponseDelay(SpinBox_AUTORESPONSEDELAY->value());
+	g_pConfig->SetAutoResponseIgnores(LineEdit_AUTO_RESPONSE_IGNORE->text().ascii());
+	SaveAutoResponses();
 
 	g_pConfig->SetMaxUpload( SpinBox_MAXSIMULTANEOUSUPLOADS->value() );
 	g_pConfig->SetMaxUploadRate( floor((SpinBox_MAXUPLOADRATE->cleanText().toFloat()*1024)+0.5) );
 	g_pConfig->SetMaxDownloadRate( floor((SpinBox_MAXDOWNLOADRATE->cleanText().toFloat()*1024)+0.5) );
 	g_pConfig->SetDownloadQueueTime(SpinBox_SAVEDOWNLOADQUEUETIME->value());
+	g_pConfig->SetAutoSearchInterval(SpinBox_AUTOSEARCHINTERVAL->value());
+	g_pConfig->SetSmallFileSize(SpinBox_SMALLFILESIZE->value());
 
 	g_pConfig->SetTCPListenPort(LineEdit_TCPLISTENPORT->text().toInt(&ok));
 	g_pConfig->SetUDPListenPort(LineEdit_UDPLISTENPORT->text().toInt(&ok));
@@ -1412,6 +1642,7 @@ void DCOptions::GetValues()
 	g_pConfig->SetChatSendOfflineMessages(CheckBox_CHATSENDOFFLINEMESSAGES->isChecked());
 	g_pConfig->SetChatRecvOfflineMessages(CheckBox_CHATRECVOFFLINEMESSAGES->isChecked());
 	g_pConfig->SetChatMessageMaxLines( SpinBox_CHATMESSAGEMAXLINES->value() );
+	g_pConfig->SetSuppressedNicks( LineEdit_SUPPRESS_NICKS->text() );
 
 	// sound settings
 	g_pConfig->SetSoundDisabled(CheckBox_SOUNDDISABLED->isChecked());
@@ -1435,7 +1666,8 @@ void DCOptions::GetValues()
 
 	// gui->chat
 	g_pConfig->SetChatShowJoinsAndParts(CheckBox_CHATSHOWJOINSANDPARTS->isChecked());
-
+	g_pConfig->SetChatShowJoinsAndPartsOnlyFav(CheckBox_CHATSHOWJOINSANDPARTSONLYFAV->isChecked());
+	
 	// gui->client
 	g_pConfig->SetOpenClientWindowsMinimized(CheckBox_CLIENTWINDOWSMINIMIZED->isChecked());
 	g_pConfig->SetUserListRightAlignment(CheckBox_USERLISTRIGHTALIGNMENT->isChecked());
@@ -1449,9 +1681,29 @@ void DCOptions::GetValues()
 	g_pConfig->SetClientColumn(eclcIP,CheckBox_CLIENTCOLUMNS_IP->isChecked());
 	g_pConfig->SetClientColumn(eclcLOCKPK,CheckBox_CLIENTCOLUMNS_LOCKPK->isChecked());
 	g_pConfig->SetClientColumn(eclcSUPPORTS,CheckBox_CLIENTCOLUMNS_SUPPORTS->isChecked());
+	g_pConfig->SetDoubleClickAction(ComboBox_DOUBLECLICK->currentText());
+	
+	// gui->icons
+	if ( ComboBox_APPLICATION_ICONS->count() > 0 )
+	{
+		g_pConfig->SetApplicationIconTheme(ComboBox_APPLICATION_ICONS->currentText());
+	}
+	if ( ComboBox_EMOTICONS->count() > 0 )
+	{
+		g_pConfig->SetEmoticonTheme(ComboBox_EMOTICONS->currentText());
+	}
+	if ( ComboBox_USERLIST_ICONS->count() > 0 )
+	{
+		g_pConfig->SetUserListIconTheme(ComboBox_USERLIST_ICONS->currentText());
+	}
+	// reload icons with new theme
+	g_pIconLoader->Load();
 	
 	// log settings
 	g_pConfig->SetLogFile(CheckBox_LOGFILE->isChecked());
+	g_pConfig->SetLogFinishedDownloads(CheckBox_LOGDOWNLOADS->isChecked());
+	g_pConfig->SetLogFinishedUploads(CheckBox_LOGUPLOADS->isChecked());
+	g_pConfig->SetLogDetails(CheckBox_LOGDETAILS->isChecked());
 	g_pConfig->SetLogFileName( LineEdit_LOGFILENAME->text().ascii() );
 	g_pConfig->SetTimeStamp(etsPRIVATECHAT,CheckBox_TIMESTAMPPRIVATECHAT->isChecked());
 	g_pConfig->SetTimeStamp(etsHUBCHAT,CheckBox_TIMESTAMPHUBCHAT->isChecked());
@@ -1541,6 +1793,12 @@ void DCOptions::GetValues()
 	g_pConfig->SetChatColor(eccPUBLICPRIVATECHATTEXT,QColor(ListBox_CHATCOLOR->item(i++)->pixmap()->convertToImage().pixel(0,0)).name().ascii());
 	g_pConfig->SetChatColor(eccCHATSAY,QColor(ListBox_CHATCOLOR->item(i++)->pixmap()->convertToImage().pixel(0,0)).name().ascii());
 	g_pConfig->SetChatColor(eccPUBLICCHATMENICK,QColor(ListBox_CHATCOLOR->item(i++)->pixmap()->convertToImage().pixel(0,0)).name().ascii());
+	g_pConfig->SetChatColor(eccOPNICK,QColor(ListBox_CHATCOLOR->item(i++)->pixmap()->convertToImage().pixel(0,0)).name().ascii());
+	
+	g_pConfig->SetRemoteEncoding(ComboBox_REMOTE_ENCODING->currentText().ascii());
+	
+	g_pConfig->SetFoldersInRightPane(CheckBox_FOLDERS_RIGHT_PANE->isChecked());
+	g_pConfig->SetFolderSizesInLeftPane(CheckBox_SIZES_LEFT_PANE->isChecked());
 }
 
 /** */
@@ -1565,6 +1823,76 @@ void DCOptions::SaveSharedFolders()
 	g_pConfig->SetSharedFolders( &sharedfolders );
 }
 
+void DCOptions::SaveUserMenuCommands()
+{
+	CList<DC_UserMenuCommand> commands;
+	DC_UserMenuCommand * umc;
+	
+	commands.Clear();
+	
+	QListViewItemIterator it( listView_MENU_COMMANDS );
+	
+	for ( ; it.current(); ++it )
+	{
+		umc = new DC_UserMenuCommand();
+		umc->m_sName    = it.current()->text(0).ascii();
+		umc->m_nContext = it.current()->text(1).toInt();
+		umc->m_sCommand = it.current()->text(2).ascii();
+		commands.Add(umc);
+	}
+	
+	g_pConfig->SetUserMenuCommands( &commands );
+}
+
+void DCOptions::SaveUserChatCommands()
+{
+	CList<DC_UserChatCommand> commands;
+	DC_UserChatCommand * ucc;
+	
+	commands.Clear();
+	
+	QListViewItemIterator it( listView_CHAT_COMMANDS );
+	
+	for( ; it.current(); ++it )
+	{
+		ucc = new DC_UserChatCommand();
+		ucc->m_sName    = it.current()->text(0).ascii();
+		ucc->m_sCommand = it.current()->text(1).ascii();
+		
+		commands.Add(ucc);
+	}
+	
+	g_pConfig->SetUserChatCommands( &commands );
+}
+
+void DCOptions::SaveAutoResponses()
+{
+	CList<DC_AutoResponseObject> autoresponses;
+	DC_AutoResponseObject * aro = 0;
+	
+	autoresponses.Clear();
+	
+	QListViewItemIterator it( listView_AUTO_RESPONSES );
+	for ( ; it.current(); ++it)
+	{
+		aro = new DC_AutoResponseObject();
+		aro->m_sTrigger = it.current()->text(0).ascii();
+		if (it.current()->text(1) == tr("Yes"))
+		{
+			aro->m_bCaseSensitive = TRUE;
+		}
+		else
+		{
+			aro->m_bCaseSensitive = FALSE;
+		}
+		aro->m_sResponse = it.current()->text(2).ascii();
+		
+		autoresponses.Add(aro);
+	}
+	
+	g_pConfig->SetAutoResponses( &autoresponses );
+}
+
 /** */
 void DCOptions::slotCreateShareList()
 {
@@ -1827,13 +2155,23 @@ void DCOptions::slotListViewOptionClicked( QListViewItem * item )
 	else if ( item == plviGUIUser )
 	{
 		TabWidget_DCGUI->setCurrentPage(3);
-		TabWidget_GUI->setCurrentPage(3);
+		TabWidget_GUI->setCurrentPage(5);
+	}
+	else if ( item == plviGUIMenus )
+	{
+		TabWidget_DCGUI->setCurrentPage(3);
+		TabWidget_GUI->setCurrentPage(4);
 	}
 	else if ( item == plviGUIChat )
 	{
 		TabWidget_DCGUI->setCurrentPage(3);
 		TabWidget_GUI->setCurrentPage(2);
 	}
+	else if (item == plviGUICommands )
+	{
+		TabWidget_DCGUI->setCurrentPage(3);
+		TabWidget_GUI->setCurrentPage(3);
+	}
 	else if ( item == plviGUITransfer )
 	{
 		TabWidget_DCGUI->setCurrentPage(3);
@@ -1844,6 +2182,16 @@ void DCOptions::slotListViewOptionClicked( QListViewItem * item )
 		TabWidget_DCGUI->setCurrentPage(3);
 		TabWidget_GUI->setCurrentPage(0);
 	}
+	else if ( item == plviGUIIcons )
+	{
+		TabWidget_DCGUI->setCurrentPage(3);
+		TabWidget_GUI->setCurrentPage(6);
+	}
+	else if ( item == plviGUIFilelistBrowser )
+	{
+		TabWidget_DCGUI->setCurrentPage(3);
+		TabWidget_GUI->setCurrentPage(7);
+	}
 	else if ( item == plviSound )
 	{
 		TabWidget_DCGUI->setCurrentPage(4);
@@ -1860,4 +2208,439 @@ void DCOptions::slotListViewOptionClicked( QListViewItem * item )
 	{
 		TabWidget_DCGUI->setCurrentPage(7);
 	}
+	else if ( item == plviAutoResponse )
+	{
+		TabWidget_DCGUI->setCurrentPage(8);
+	}
+	else if ( item == plviEncoding )
+	{
+		TabWidget_DCGUI->setCurrentPage(9);
+	}
+}
+
+/** */
+void DCOptions::slotEnableShowJoinsParts()
+{
+	CheckBox_CHATSHOWJOINSANDPARTSONLYFAV->setEnabled(CheckBox_CHATSHOWJOINSANDPARTS->isChecked());
+}
+
+/** */
+void DCOptions::slotContextMenuUserMenuCommands( QListViewItem * item, const QPoint & pos, int col)
+{
+	QPopupMenu *m;
+	QListViewItem *newitem;
+	int id;
+	
+	m = new QPopupMenu(this);
+	
+	DCMenuHandler::InsertMenu( m, emiADD );
+	DCMenuHandler::InsertMenu( m, emiREMOVE );
+	
+	id = m->exec(QCursor::pos());
+	
+	delete m;
+	
+	if ( id == -1 )
+	{
+		return;
+	}
+	else if ( id == emiADD )
+	{
+		newitem = new QListViewItem( listView_MENU_COMMANDS, "Say hello", "1", "<%[myNI]> hello|");
+		newitem->setRenameEnabled(0, TRUE);
+		newitem->setRenameEnabled(1, TRUE);
+		newitem->setRenameEnabled(2, TRUE);
+	}
+	else if ( id == emiREMOVE )
+	{
+		if (item != 0)
+		{
+			delete item;
+		}
+	}
+}
+
+/** */
+void DCOptions::slotContextMenuUserChatCommands( QListViewItem * item, const QPoint & pos, int col )
+{
+	QPopupMenu *m;
+	QListViewItem *newitem;
+	int id;
+	
+	m = new QPopupMenu(this);
+	
+	DCMenuHandler::InsertMenu( m, emiADD );
+	DCMenuHandler::InsertMenu( m, emiREMOVE );
+	
+	id = m->exec(QCursor::pos());
+	
+	delete m;
+	
+	if ( id == -1 )
+	{
+		return;
+	}
+	else if ( id == emiADD )
+	{
+		newitem = new QListViewItem( listView_CHAT_COMMANDS, "/s", "/sh somescript.sh" );
+		newitem->setRenameEnabled(0, TRUE);
+		newitem->setRenameEnabled(1, TRUE);
+	}
+	else if ( id == emiREMOVE )
+	{
+		if (item != 0)
+		{
+			delete item;
+		}
+	}
+}
+
+/** */
+void DCOptions::slotContextMenuAutoResponses( QListViewItem * item, const QPoint & pos, int col )
+{
+	QPopupMenu *m = 0;
+	QListViewItem *newitem;
+	int id, cs_yes, cs_no;
+	
+	m = new QPopupMenu(this);
+	
+	DCMenuHandler::InsertMenu( m, emiADD );
+	DCMenuHandler::InsertMenu( m, emiREMOVE );
+	cs_yes = m->insertItem(tr("Case sensitive"));
+	cs_no = m->insertItem(tr("Case insensitive"));
+	
+	id = m->exec(QCursor::pos());
+	
+	delete m;
+	
+	if ( id == -1)
+	{
+		return;
+	}
+	else if ( id == emiADD )
+	{
+		newitem = new QListViewItem( listView_AUTO_RESPONSES, "-trigger", tr("No"), "response" );
+		newitem->setRenameEnabled(0, TRUE);
+		newitem->setRenameEnabled(2, TRUE);
+	}
+	else if ( id == emiREMOVE )
+	{
+		if (item != 0)
+		{
+			delete item;
+		}
+	}
+	else if ( id == cs_yes )
+	{
+		if (item != 0)
+		{
+			item->setText(1, tr("Yes"));
+		}
+	}
+	else if ( id == cs_no )
+	{
+		if (item != 0)
+		{
+			item->setText(1, tr("No"));
+		}
+	}
+}
+
+/** */
+void DCOptions::setupIconThemeComboBoxes()
+{
+	QDir appiconsdir, emoticonsdir, usericonsdir;
+	QString basedir;
+	
+	basedir = g_pConfig->GetValknutDataPath();
+	
+	ComboBox_APPLICATION_ICONS->clear();
+	ComboBox_USERLIST_ICONS->clear();
+	ComboBox_EMOTICONS->clear();
+	
+	appiconsdir.setPath( basedir + "/icons/appl/" );
+	appiconsdir.setFilter( QDir::Dirs | QDir::Readable | QDir::Executable );
+	appiconsdir.setSorting( QDir::Name );
+	
+	const QFileInfoList * applist = appiconsdir.entryInfoList();
+	if ( applist != 0 )
+	{
+		QFileInfoListIterator appit( *applist );
+		QFileInfo *appfi;
+		
+		while ( (appfi = appit.current()) != 0 )
+		{
+			if ( (appfi->fileName() == ".") || (appfi->fileName() == "..") )
+			{
+				// skip
+			}
+			else
+			{
+				ComboBox_APPLICATION_ICONS->insertItem(appfi->fileName());
+			}
+			++appit;
+		}
+	}
+	
+	emoticonsdir.setPath( basedir + "/icons/emot/" );
+	emoticonsdir.setFilter( QDir::Dirs | QDir::Readable | QDir::Executable );
+	emoticonsdir.setSorting( QDir::Name );
+	
+	const QFileInfoList * emotlist = emoticonsdir.entryInfoList();
+	if ( emotlist != 0 )
+	{
+		QFileInfoListIterator emotit ( *emotlist );
+		QFileInfo *emotfi;
+		
+		while ( (emotfi = emotit.current()) != 0 )
+		{
+			if ( (emotfi->fileName() == ".") || (emotfi->fileName() == "..") )
+			{
+				// skip
+			}
+			else
+			{ 
+				ComboBox_EMOTICONS->insertItem(emotfi->fileName());
+			}
+			++emotit;
+		}
+	}
+	
+	usericonsdir.setPath( basedir + "/icons/user/" );
+	usericonsdir.setFilter( QDir::Dirs | QDir::Readable | QDir::Executable );
+	usericonsdir.setSorting( QDir::Name );
+	
+	const QFileInfoList * usericonslist = usericonsdir.entryInfoList();
+	if ( usericonslist != 0 )
+	{
+		QFileInfoListIterator uit ( *usericonslist );
+		QFileInfo *userfi;
+		
+		while ( (userfi = uit.current()) != 0 )
+		{
+			if ( (userfi->fileName() == ".") || (userfi->fileName() == "..") )
+			{
+				// skip
+			}
+			else
+			{
+				ComboBox_USERLIST_ICONS->insertItem(userfi->fileName());
+			}
+			++uit;
+		}
+	}
+}
+
+/** Needed so that you don't have to save the dialog before refreshing the list */
+void DCOptions::slotDontShareDotFilesChanged()
+{
+	g_pConfig->SetDontShareDotFiles(CheckBox_DONTSHAREDOTFILES->isChecked());
+}
+
+/** */
+void DCOptions::slotGenerateKeyCert()
+{
+	QProcess * proc1 = 0;
+	QProcess * proc2 = 0;
+	QString keyname = LineEdit_TRANSFERKEY->text();
+	QString certname = LineEdit_TRANSFERCERT->text();
+	QFile key;
+	QFile cert;
+	QDir keypath;
+	QDir certpath;
+	int i = 0;
+	
+	if (keyname == "")
+	{
+		keyname = QString(g_pConfig->GetConfigPath().Data()) + QString("/ssl/valknut.key");
+	}
+	
+	if (certname == "")
+	{
+		certname = QString(g_pConfig->GetConfigPath().Data()) + QString("/ssl/valknut.cert");
+	}
+	
+	keyname = keyname.replace("//", "/");
+	certname = certname.replace("//", "/");
+	
+	key.setName(keyname);
+	cert.setName(certname);
+	
+	if ( key.exists() || cert.exists() )
+	{
+		int ret = QMessageBox::question(
+			this,
+			tr("Overwrite?"),
+			tr("Replace existing key/cert?"),
+			QMessageBox::Yes,
+			QMessageBox::No
+		);
+		
+		if ( ret == QMessageBox::No )
+		{
+			return;
+		}
+	}
+	
+	keypath.setPath(QFileInfo(keyname).dirPath());
+	certpath.setPath(QFileInfo(certname).dirPath());
+	
+	if (keypath.exists() == FALSE)
+	{
+		if ( keypath.mkdir(keypath.path()) == FALSE )
+		{
+			QMessageBox::critical(
+				this,
+				tr("Could not create folder for key"),
+				tr("Could not create ") + keypath.path(),
+				QMessageBox::Ok,
+				QMessageBox::NoButton,
+				QMessageBox::NoButton
+			);
+			return;
+		}
+	}
+	
+	if (certpath.exists() == FALSE)
+	{
+		if ( certpath.mkdir(certpath.path()) == FALSE )
+		{
+			QMessageBox::critical(
+				this,
+				tr("Could not create folder for cert"),
+				tr("Could not create ") + certpath.path(),
+				QMessageBox::Ok,
+				QMessageBox::NoButton,
+				QMessageBox::NoButton
+			);
+			return;
+		}
+	}
+	
+	proc1 = new QProcess(this);
+	proc1->setWorkingDirectory(keypath);
+	proc1->addArgument("openssl");
+	proc1->addArgument("genrsa");
+	proc1->addArgument("-out");
+	proc1->addArgument(keyname);
+	proc1->addArgument("2048");
+	
+	if ( !proc1->start() )
+	{
+		QMessageBox::critical(
+			this,
+			tr("Could not start openssl 1"),
+			tr("Make sure the openssl program is in your PATH."),
+			QMessageBox::Ok,
+			QMessageBox::NoButton,
+			QMessageBox::NoButton
+		);
+		delete proc1;
+		return;
+	}
+	
+	proc1->closeStdin();
+	i = 0;
+	while ( (proc1->isRunning() == TRUE) && (i < 5) )
+	{
+		sleep(1);
+		i++;
+	}
+	
+	if ( proc1->isRunning() == TRUE )
+	{
+		proc1->tryTerminate();
+		sleep(1);
+		if ( proc1->isRunning() == TRUE )
+		{
+			proc1->kill();
+		}
+	}
+	
+	if ( (proc1->normalExit() == FALSE) || (proc1->exitStatus() != 0) )
+	{
+		QMessageBox::critical(
+			this,
+			tr("Error occurred while running openssl 1"),
+			QString(proc1->readStderr()),
+			QMessageBox::Ok,
+			QMessageBox::NoButton,
+			QMessageBox::NoButton
+		);
+		delete proc1;
+		return;
+	}
+	
+	delete proc1;
+	
+	proc2 = new QProcess(this);
+	proc2->setWorkingDirectory(certpath);
+	proc2->addArgument("openssl");
+	proc2->addArgument("req");
+	proc2->addArgument("-x509");
+	proc2->addArgument("-new");
+	proc2->addArgument("-batch");
+	proc2->addArgument("-days");
+	proc2->addArgument("3650");
+	proc2->addArgument("-key");
+	proc2->addArgument(keyname);
+	proc2->addArgument("-out");
+	proc2->addArgument(certname);
+	proc2->addArgument("-subj");
+	proc2->addArgument("/CN=Valknut-" + LineEdit_NICK->text());
+	
+	if ( !proc2->start() )
+	{
+		QMessageBox::critical(
+			this,
+			tr("Could not start openssl 2"),
+			tr("Make sure the openssl program is in your PATH."),
+			QMessageBox::Ok,
+			QMessageBox::NoButton,
+			QMessageBox::NoButton
+		);
+		delete proc2;
+		return;
+	}
+	
+	proc2->closeStdin();
+	i = 0;
+	while ( (proc2->isRunning() == TRUE) && (i < 5) )
+	{
+		sleep(1);
+		i++;
+	}
+	
+	if ( proc2->isRunning() == TRUE )
+	{
+		proc2->tryTerminate();
+		sleep(1);
+		if ( proc2->isRunning() == TRUE )
+		{
+			proc2->kill();
+		}
+	}
+	
+	if ( (proc2->normalExit() == FALSE) || (proc2->exitStatus() != 0) )
+	{
+		QMessageBox::critical(
+			this,
+			tr("Error occurred while running openssl 2"),
+			QString(proc2->readStderr()),
+			QMessageBox::Ok,
+			QMessageBox::NoButton,
+			QMessageBox::NoButton
+		);
+		delete proc2;
+		return;
+	}
+	
+	delete proc2;
+	
+	LineEdit_TRANSFERKEY->setText(keyname);
+	LineEdit_TRANSFERCERT->setText(certname);
+	
+	// set them even if dialog is cancelled since files have been created
+	g_pConfig->SetTransferKey(keyname.ascii());
+	g_pConfig->SetTransferCert(certname.ascii());
 }
